@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -13,14 +13,13 @@ Local artifact's representation.
 See docs/design/ontology.html for the semantics of the Artifacts.
 """
 import abc
-import base64
 import json
 import re
 from collections.abc import Iterable, Sequence
 from datetime import datetime
 from json import JSONDecodeError
 from pathlib import Path
-from typing import Any, ClassVar, Generic, Self, TypeVar
+from typing import Any, ClassVar, Generic, Self, TypeVar, cast, overload
 
 from debian import deb822, debfile
 
@@ -31,7 +30,8 @@ except ImportError:
 
 import debusine.artifacts.models as data_models
 from debusine import utils
-from debusine.artifacts.utils import files_in_meta_file_match_files
+from debusine.artifacts.artifact_utils import files_in_meta_file_match_files
+from debusine.assets import KeyPurpose
 from debusine.client.models import (
     ArtifactCreateRequest,
     FileRequest,
@@ -77,7 +77,7 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
         str, type["LocalArtifact['Any']"]
     ] = {}
 
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs: Any) -> None:
         """Set category to _category by default."""
         kwargs.setdefault("category", self._category)
         if "data" in kwargs:
@@ -85,7 +85,7 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
                 kwargs["data"] = self.create_data(d)
         super().__init__(**kwargs)
 
-    def __init_subclass__(cls, **kwargs):
+    def __init_subclass__(cls, **kwargs: Any) -> None:
         """
         Register subclass into LocalArtifact._local_artifacts_category_to_class.
 
@@ -102,7 +102,7 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
     @classmethod
     def create_data(cls, data_dict: dict[str, Any]) -> AD:
         """Instantiate a data model from a dict."""
-        return cls._data_type(**data_dict)
+        return cast(AD, cls._data_type(**data_dict))
 
     @staticmethod
     def artifact_categories() -> list[str]:
@@ -128,7 +128,7 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
 
     def _check_existing_file(
         self, path_in_artifact: str, new_file: Path | FileRequest
-    ):
+    ) -> None:
         """Check whether the artifact already contains a given file."""
         all_files = self.files | self.remote_files
         if path_in_artifact in all_files:
@@ -144,7 +144,7 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
         *,
         artifact_base_dir: Path | None = None,
         override_name: str | None = None,
-    ):
+    ) -> None:
         """
         Add a local file in the artifact.
 
@@ -209,7 +209,7 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
         self._check_existing_file(path_in_artifact, file_request)
         self.remote_files[path_in_artifact] = file_request
 
-    def validate_model(self):
+    def validate_model(self) -> None:
         """Raise ValueError with an error if the model is not valid."""
         *_, error = pydantic.validate_model(self.__class__, self.__dict__)
 
@@ -369,7 +369,17 @@ class PackageBuildLog(LocalArtifact[data_models.DebianPackageBuildLog]):
         return super()._validate_files_end_in(files, [".build"])
 
 
-def deb822dict_to_dict(element):
+@overload
+def deb822dict_to_dict(
+    element: dict[Any, Any] | deb822.Deb822Dict,
+) -> dict[Any, Any]: ...
+
+
+@overload
+def deb822dict_to_dict(element: Any) -> Any: ...
+
+
+def deb822dict_to_dict(element: Any) -> Any:
     """
     Traverse recursively element converting Deb822Dict to dict.
 
@@ -380,6 +390,7 @@ def deb822dict_to_dict(element):
     Reason is to simplify is that json module cannot encode Deb822Dict. To
     simplify let's convert Deb822Dict as soon as possible to dict.
     """
+    result: Any
     if isinstance(element, (dict, deb822.Deb822Dict)):
         result = {}
         for key, value in element.items():
@@ -487,7 +498,10 @@ class SourcePackage(LocalArtifact[data_models.DebianSourcePackage]):
         dsc_fields = {}
         for file in files:
             if file.suffix == ".dsc":
-                dsc_fields = deb822dict_to_dict(utils.read_dsc(file))
+                dsc = utils.read_dsc(file)
+                if dsc is None:
+                    raise ValueError(f"{file} is not a valid .dsc file")
+                dsc_fields = deb822dict_to_dict(dsc)
 
         data = data_models.DebianSourcePackage(
             name=name,
@@ -505,19 +519,22 @@ class SourcePackage(LocalArtifact[data_models.DebianSourcePackage]):
 
     @pydantic.validator("files")
     @classmethod
-    def files_contain_one_dsc(cls, files: dict[str, Path]):
+    def files_contain_one_dsc(cls, files: dict[str, Path]) -> dict[str, Path]:
         """Raise ValueError when files does not have exactly 1 .dsc file."""
         return cls._validate_exactly_one_file_ends_in(files, ".dsc")
 
     @pydantic.validator("files")
     @classmethod
-    def files_contains_files_in_dsc(cls, files: dict[str, Path]):
+    def files_contains_files_in_dsc(
+        cls, files: dict[str, Path]
+    ) -> dict[str, Path]:
         """
         Validate that set(files) == set(files_in_dsc_file).
 
         Exception: The .dsc file must be in files but not in the .dsc file.
         """
-        return files_in_meta_file_match_files(".dsc", deb822.Dsc, files)
+        files_in_meta_file_match_files(".dsc", deb822.Dsc, files)
+        return files
 
 
 class BinaryPackage(LocalArtifact[data_models.DebianBinaryPackage]):
@@ -829,32 +846,6 @@ class DebianSystemImageArtifact(LocalArtifact[data_models.DebianSystemImage]):
         return files
 
 
-class SigningKeyArtifact(LocalArtifact[data_models.DebusineSigningKey]):
-    """Record the existence of a key in the signing service."""
-
-    _category = data_models.ArtifactCategory.SIGNING_KEY
-
-    @classmethod
-    def create(
-        cls,
-        purpose: data_models.KeyPurpose,
-        fingerprint: str,
-        public_key: bytes,
-    ) -> Self:
-        """Return a SigningKeyArtifact with the given properties."""
-        data = data_models.DebusineSigningKey(
-            purpose=purpose,
-            fingerprint=fingerprint,
-            public_key=base64.b64encode(public_key).decode(),
-        )
-        return cls(category=cls._category, data=data)
-
-    @property
-    def public_key_bytes(self) -> bytes:
-        """Return the public key as bytes."""
-        return base64.b64decode(self.data.public_key.encode())
-
-
 class SigningInputArtifact(LocalArtifact[data_models.DebusineSigningInput]):
     """Input to a Sign task."""
 
@@ -882,7 +873,9 @@ class SigningInputArtifact(LocalArtifact[data_models.DebusineSigningInput]):
 
     @pydantic.validator("files")
     @classmethod
-    def validate_at_least_one_file(cls, files: dict[str, Path]):
+    def validate_at_least_one_file(
+        cls, files: dict[str, Path]
+    ) -> dict[str, Path]:
         """Validate that artifact has at least one file."""
         if not files:
             raise ValueError("Expected at least one file")
@@ -897,15 +890,19 @@ class SigningOutputArtifact(LocalArtifact[data_models.DebusineSigningOutput]):
     @classmethod
     def create(
         cls,
-        purpose: data_models.KeyPurpose,
+        purpose: KeyPurpose,
         fingerprint: str,
         results: Iterable[data_models.SigningResult],
         files: Iterable[Path],
         base_dir: Path,
+        binary_package_name: str | None = None,
     ) -> Self:
         """Return a new SigningOutputArtifact."""
         data = data_models.DebusineSigningOutput(
-            purpose=purpose, fingerprint=fingerprint, results=list(results)
+            purpose=purpose,
+            fingerprint=fingerprint,
+            results=list(results),
+            binary_package_name=binary_package_name,
         )
         artifact = cls(category=cls._category, data=data)
         for file in files:
