@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -11,39 +11,61 @@
 
 import abc
 import enum
-from typing import Any, Literal
+import urllib.parse
+from typing import Any, Literal, cast
 
 try:
     import pydantic.v1 as pydantic
 except ImportError:
     import pydantic as pydantic  # type: ignore
 
+from debusine.assets import KeyPurpose
+
+
+class BaseArtifactDataModel(pydantic.BaseModel):
+    """Base pydantic model for bare/artifact data and their components."""
+
+    class Config:
+        """Set up stricter pydantic Config."""
+
+        validate_assignment = True
+        extra = pydantic.Extra.forbid
+
+    def dict(self, **kwargs: Any) -> dict[str, Any]:
+        """Use aliases by default when serializing."""
+        kwargs.setdefault("by_alias", True)
+        return super().dict(**kwargs)
+
 
 class BareDataCategory(enum.StrEnum):
     """Possible categories for bare data in collection items."""
 
     PACKAGE_BUILD_LOG = "debian:package-build-log"
     PROMISE = "debusine:promise"
+    HISTORICAL_TASK_RUN = "debusine:historical-task-run"
+    TASK_CONFIGURATION = "debusine:task-configuration"
 
     # Only implemented in tests.
     TEST = "debusine:test"
 
 
-class DebusinePromise(pydantic.BaseModel):
+class DebusinePromise(BaseArtifactDataModel):
     """Pydantic model for debusine promise data."""
 
     promise_work_request_id: int
     promise_workflow_id: int
     promise_category: str
 
-    class Config:
+    class Config(BaseArtifactDataModel.Config):
         """Allow extra fields."""
 
         extra = pydantic.Extra.allow
 
     @pydantic.root_validator(pre=True)
     @classmethod
-    def check_no_extra_names_starting_promise(cls, values):
+    def check_no_extra_names_starting_promise(
+        cls, values: dict[str, Any]
+    ) -> dict[str, Any]:
         r"""Forbid extra field names starting with 'promise\\_'."""
         allowed_promise_fields = cls.__fields__.keys()
 
@@ -57,6 +79,171 @@ class DebusinePromise(pydantic.BaseModel):
         return values
 
 
+class RuntimeStatistics(BaseArtifactDataModel):
+    """Runtime statistics about a completed task."""
+
+    #: Runtime duration of task, in seconds.
+    duration: int | None = None
+
+    #: Amount of CPU time used (both user and system), in seconds.
+    cpu_time: int | None = None
+
+    #: Maximum disk space used, in bytes.
+    disk_space: int | None = None
+
+    #: Maximum RAM used, in bytes.
+    memory: int | None = None
+
+    #: Available disk space when the task started, in bytes; may be rounded.
+    available_disk_space: int | None = None
+
+    #: Available RAM when the task started, in bytes; may be rounded.
+    available_memory: int | None = None
+
+    #: Number of CPU cores on the worker that ran the task.
+    cpu_count: int | None = None
+
+
+def _validate_task_type(value: str) -> str:
+    """Ensure value is a valid TASK_TYPE."""
+    from debusine.tasks.models import TaskTypes
+
+    # From python 3.13:
+    # if value not in TaskTypes:
+    try:
+        TaskTypes(value)
+    except ValueError:
+        raise ValueError(f"{value!r} is not a valid task type")
+
+    return value
+
+
+def _validate_no_colons(value: str | None) -> str | None:
+    """
+    Check that value, if set, contains no colons.
+
+    This is used, for example, when colons in a value would break the framing
+    of the computed CollectionItem name.
+    """
+    if value is not None and ":" in value:
+        raise ValueError(f"{value!r} may not contain ':'")
+    return value
+
+
+class DebusineHistoricalTaskRun(BaseArtifactDataModel):
+    """Pydantic model for historical task runs."""
+
+    # Really TaskTypes, but for layering reasons we don't want to import
+    # from debusine.tasks here.
+    task_type: str
+    task_name: str
+    subject: str | None = None
+    context: str | None = None
+    timestamp: int
+    work_request_id: int
+    # Really WorkRequest.Results, but for layering reasons we don't want to
+    # import from debusine.db here.
+    result: str
+    runtime_statistics: RuntimeStatistics
+
+    _validate_task_type = pydantic.validator("task_type", allow_reuse=True)(
+        _validate_task_type
+    )
+
+
+class DebusineTaskConfiguration(BaseArtifactDataModel):
+    """Pydantic model for task configuration."""
+
+    # Really TaskTypes, but for layering reasons we don't want to import
+    # from debusine.tasks here.
+    task_type: str | None = None
+    task_name: str | None = None
+    subject: str | None = None
+    context: str | None = None
+    template: str | None = None
+
+    use_templates: list[str] = []
+    delete_values: list[str] = []
+    default_values: dict[str, Any] = {}
+    override_values: dict[str, Any] = {}
+    lock_values: list[str] = []
+    comment: str = ""
+
+    def name(self) -> str:
+        """Return the bare data item name for this task configuration."""
+        if self.template is not None:
+            return f"template:{self.template}"
+        else:
+            context = urllib.parse.quote(self.context) if self.context else ''
+            return (
+                f"{self.task_type}:{self.task_name}"
+                f":{self.subject or ''}:{context}"
+            )
+
+    @staticmethod
+    def get_lookup_names(
+        task_type: str,
+        task_name: str,
+        subject: str | None,
+        context: str | None,
+    ) -> list[str]:
+        """Compute names to look up to get a task's configuration items."""
+        names: list[str] = []
+        context_encoded = urllib.parse.quote(context) if context else ''
+        # Global (subject=None, context=None)
+        names.append(f"{task_type}:{task_name}::")
+        # Context level (subject=None, context != None)
+        if context_encoded:
+            names.append(f"{task_type}:{task_name}::{context_encoded}")
+        # Subject level (subject != None, context=None)
+        if subject:
+            names.append(f"{task_type}:{task_name}:{subject}:")
+        # Specific-combination level (subject != None, context != None)
+        if subject and context_encoded:
+            names.append(f"{task_type}:{task_name}:{subject}:{context_encoded}")
+        return names
+
+    _validate_task_type = pydantic.validator("task_type", allow_reuse=True)(
+        _validate_task_type
+    )
+    _validate_task_name = pydantic.validator("task_name", allow_reuse=True)(
+        _validate_no_colons
+    )
+    _validate_subject = pydantic.validator("subject", allow_reuse=True)(
+        _validate_no_colons
+    )
+    _validate_template = pydantic.validator("template", allow_reuse=True)(
+        _validate_no_colons
+    )
+
+    @pydantic.root_validator(allow_reuse=True)
+    @classmethod
+    def template_or_task_match(cls, values: dict[str, Any]) -> dict[str, Any]:
+        """Ensure we have either template or the other 4 match fields."""
+        if values.get("template"):
+            present: list[str] = [
+                name
+                for name in ("task_type", "task_name", "subject", "context")
+                if values.get(name)
+            ]
+            if present:
+                raise ValueError(
+                    f"{', '.join(present)} should be empty if template is set"
+                )
+        else:
+            missing: list[str] = [
+                name
+                for name in ("task_type", "task_name")
+                if not values.get(name)
+            ]
+            if missing:
+                raise ValueError(
+                    f"{', '.join(missing)} should be set for non-template"
+                    " task configuration entries"
+                )
+        return values
+
+
 class ArtifactCategory(enum.StrEnum):
     """Possible artifact categories."""
 
@@ -72,7 +259,6 @@ class ArtifactCategory(enum.StrEnum):
     BLHC = "debian:blhc"
     DEBDIFF = "debian:debdiff"
     SYSTEM_IMAGE = "debian:system-image"
-    SIGNING_KEY = "debusine:signing-key"
     SIGNING_INPUT = "debusine:signing-input"
     SIGNING_OUTPUT = "debusine:signing-output"
 
@@ -86,9 +272,9 @@ class CollectionCategory(enum.StrEnum):
     ENVIRONMENTS = "debian:environments"
     SUITE = "debian:suite"
     SUITE_LINTIAN = "debian:suite-lintian"
-    SUITE_SIGNING_KEYS = "debian:suite-signing-keys"
     PACKAGE_BUILD_LOGS = "debian:package-build-logs"
     WORKFLOW_INTERNAL = "debusine:workflow-internal"
+    TASK_CONFIGURATION = "debusine:task-configuration"
     TASK_HISTORY = "debusine:task-history"
 
     # Only implemented in tests.
@@ -101,21 +287,6 @@ SINGLETON_COLLECTION_CATEGORIES = {
 }
 
 
-class BaseArtifactDataModel(pydantic.BaseModel):
-    """Base pydantic model for artifact data and their components."""
-
-    class Config:
-        """Set up stricter pydantic Config."""
-
-        validate_assignment = True
-        extra = pydantic.Extra.forbid
-
-    def dict(self, **kwargs) -> dict[str, Any]:
-        """Use aliases by default when serializing."""
-        kwargs.setdefault("by_alias", True)
-        return super().dict(**kwargs)
-
-
 class ArtifactData(BaseArtifactDataModel, abc.ABC):
     """
     Base class for artifact data.
@@ -274,13 +445,16 @@ class DebianUpload(ArtifactData):
         if files := self.changes_fields.get("Files"):
             for file in files:
                 if (name := file["name"]).endswith(".changes"):
-                    return name
-            return files[0]["name"]
+                    break
+            else:
+                name = files[0]["name"]
+            assert isinstance(name, str)
+            return name
         return None
 
     @staticmethod
     def _changes_architectures(
-        changes_fields: dict[str, Any]
+        changes_fields: dict[str, Any],
     ) -> frozenset[str]:
         return frozenset(changes_fields["Architecture"].split())
 
@@ -290,7 +464,7 @@ class DebianUpload(ArtifactData):
 
     @pydantic.validator("changes_fields")
     @classmethod
-    def metadata_mandatory_fields(cls, data: dict[str, Any]):
+    def metadata_mandatory_fields(cls, data: dict[str, Any]) -> dict[str, Any]:
         """Validate that changes_fields contains Architecture and Files."""
         if "Architecture" not in data:
             raise ValueError("changes_fields must contain Architecture")
@@ -300,7 +474,9 @@ class DebianUpload(ArtifactData):
 
     @pydantic.validator("changes_fields")
     @classmethod
-    def metadata_contains_debs_if_binary(cls, data: dict[str, Any]):
+    def metadata_contains_debs_if_binary(
+        cls, data: dict[str, Any]
+    ) -> dict[str, Any]:
         """
         Validate that binary uploads reference binaries.
 
@@ -327,7 +503,9 @@ class DebianUpload(ArtifactData):
 
     @pydantic.validator("changes_fields")
     @classmethod
-    def metadata_contains_dsc_if_source(cls, data: dict[str, Any]):
+    def metadata_contains_dsc_if_source(
+        cls, data: dict[str, Any]
+    ) -> dict[str, Any]:
         """
         Validate that source uploads contain one and only one source package.
 
@@ -455,9 +633,12 @@ class DebianLintian(ArtifactData):
 
     def get_label(self) -> str:
         """Return a short human-readable label for the artifact."""
-        return min(
-            self.summary.package_filename.keys(), key=lambda name: len(name)
-        )
+        if self.summary.package_filename:
+            return (
+                f"lintian: {', '.join(sorted(self.summary.package_filename))}"
+            )
+        else:
+            return "lintian (empty)"
 
 
 class DebianAutopkgtestResultStatus(enum.StrEnum):
@@ -498,25 +679,6 @@ class DebianAutopkgtest(ArtifactData):
         return self.source_package.name
 
 
-class KeyPurpose(enum.StrEnum):
-    """The purpose of a key."""
-
-    UEFI = "uefi"
-    OPENPGP = "openpgp"
-
-
-class DebusineSigningKey(ArtifactData):
-    """Data for debusine:signing-key artifacts."""
-
-    purpose: KeyPurpose
-    fingerprint: str
-    public_key: str  # base64-encoded bytes
-
-    def get_label(self) -> str:
-        """Return a short human-readable label for the artifact."""
-        return self.fingerprint
-
-
 class DebusineSigningInput(ArtifactData):
     """Input to a Sign task."""
 
@@ -537,7 +699,7 @@ class SigningResult(BaseArtifactDataModel):
 
     @pydantic.root_validator(allow_reuse=True)
     @classmethod
-    def check_exactly_one_result(cls, values):
+    def check_exactly_one_result(cls, values: dict[str, Any]) -> dict[str, Any]:
         """Ensure exactly one of output_file and error_message is present."""
         if (values.get("output_file") is None) == (
             values.get("error_message") is None
@@ -554,6 +716,7 @@ class DebusineSigningOutput(ArtifactData):
     purpose: KeyPurpose
     fingerprint: str
     results: list[SigningResult]
+    binary_package_name: str | None = None
 
     def get_label(self) -> None:
         """Return a short human-readable label for the artifact."""
@@ -569,3 +732,35 @@ class DebDiff(ArtifactData):
     def get_label(self) -> str:
         """Return a short human-readable label for the artifact."""
         return f"debdiff {self.original} {self.new}"
+
+
+def get_source_package_name(
+    artifact_data: (
+        DebianSourcePackage
+        | DebianUpload
+        | DebianBinaryPackage
+        | DebianBinaryPackages
+        | DebianPackageBuildLog
+    ),
+) -> str:
+    """Retrieve the source name of a package."""
+    match artifact_data:
+        case DebianSourcePackage():
+            return artifact_data.name
+        case DebianUpload():
+            return cast(str, artifact_data.changes_fields["Source"])
+        case DebianBinaryPackage() | DebianBinaryPackages():
+            return artifact_data.srcpkg_name
+        case DebianPackageBuildLog():
+            return artifact_data.source
+
+    raise TypeError(f"Unexpected type: {type(artifact_data).__name__}")
+
+
+def get_binary_package_name(artifact_data: DebianBinaryPackage) -> str | None:
+    """Retrieve the binary package name."""
+    match artifact_data:
+        case DebianBinaryPackage():
+            return artifact_data.deb_fields.get("Package")
+
+    raise TypeError(f"Unexpected type: {type(artifact_data).__name__}")
