@@ -1,4 +1,4 @@
-# Copyright 2023-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -12,17 +12,23 @@
 import json
 from collections import deque
 from importlib.metadata import version
+from io import BytesIO
 from itertools import count
 from typing import Any
-from unittest import TestCase, mock, skipIf
+from unittest import mock, skipIf
 
 import requests
 import responses
 
+# Exported from top-level urllib3 module in >= 2.0.0, but that isn't in
+# bookworm.
+from urllib3.response import HTTPHeaderDict  # type: ignore[attr-defined]
+from urllib3.response import HTTPResponse
+
 from debusine.client import exceptions
 from debusine.client.debusine_http_client import DebusineHttpClient
 from debusine.client.models import StrictBaseModel
-from debusine.test import TestHelpersMixin
+from debusine.test import TestCase
 
 try:  # pragma: no cover
     from responses.registries import OrderedRegistry
@@ -39,7 +45,7 @@ class TestResponse(StrictBaseModel):
     page: int | None = None
 
 
-class DebusineHttpClientTests(TestHelpersMixin, TestCase):
+class DebusineHttpClientTests(TestCase):
     """Tests for the DebusineHttpClient."""
 
     def setUp(self) -> None:
@@ -113,7 +119,7 @@ class DebusineHttpClientTests(TestHelpersMixin, TestCase):
             if pageno > 1:
                 body["previous"] = base_url + f"?page={pageno - 1}"
 
-            from responses.matchers import query_string_matcher  # type: ignore
+            from responses.matchers import query_string_matcher
 
             url_without_query_string, _, query_string = url.partition("?")
 
@@ -215,6 +221,30 @@ class DebusineHttpClientTests(TestHelpersMixin, TestCase):
         ):
             debusine_client._api_request("GET", self.api_url, TestResponse)
 
+    @responses.activate
+    def test_api_request_raise_debusine_error_404(self) -> None:
+        """Raise DebusineError even with response.code == 404."""
+        title = "Error"
+        detail = "Workflow template not found"
+
+        self.responses_add_problem(
+            responses.POST,
+            self.api_url,
+            title,
+            detail=detail,
+            status=requests.codes.not_found,
+        )
+        debusine_client = DebusineHttpClient(self.api_url, self.token)
+        with self.assertRaises(exceptions.DebusineError) as debusine_error:
+            debusine_client._api_request("POST", self.api_url, TestResponse)
+
+        exception = debusine_error.exception
+
+        self.assertEqual(
+            exception.problem,
+            {"title": title, "detail": detail, "validation_errors": None},
+        )
+
     @responses.activate
     def test_api_request_raise_403_error_with_detail(self) -> None:
         """Raise TokenDisabledError: server returned HTTP 403 with detail."""
@@ -291,7 +321,7 @@ class DebusineHttpClientTests(TestHelpersMixin, TestCase):
     )
     @responses.activate(registry=OrderedRegistry)
     def test_api_request_retry_errors(self) -> None:  # pragma: no cover
-        """Some HTTP error responsese are retried."""
+        """Some HTTP error responses are retried."""
         debusine_client = DebusineHttpClient(self.api_url, self.token)
 
         for code in (
@@ -364,7 +394,7 @@ class DebusineHttpClientTests(TestHelpersMixin, TestCase):
         self.assert_token_key_included_in_all_requests(self.token)
 
     def test_api_request_create_and_use_session(self) -> None:
-        """_api_request method create a session, uses and re-uses it."""
+        """_api_request method create a session, uses and reuses it."""
         session_patch = mock.patch("requests.Session", autospec=True)
         session_mock = session_patch.start()
         self.addCleanup(session_patch.stop)
@@ -389,7 +419,7 @@ class DebusineHttpClientTests(TestHelpersMixin, TestCase):
         with self.assertRaises(exceptions.UnexpectedResponseError):
             debusine_client._api_request("GET", self.api_url, TestResponse)
 
-        # Same session re-used
+        # Same session reused
         self.assertIs(debusine_client._session, session)
 
     def test_api_request_no_token(self) -> None:
@@ -461,6 +491,37 @@ class DebusineHttpClientTests(TestHelpersMixin, TestCase):
             assert responses.calls[0].request.body is not None
             self.assertEqual(json.loads(responses.calls[0].request.body), data)
 
+    @responses.activate
+    def test_api_request_ok_no_content_type(self) -> None:
+        """_api_request() handles a 200 response with no Content-Type."""
+
+        class EmptyResponse(responses.BaseResponse):
+            """A response without even a Content-Type."""
+
+            def get_response(
+                self, request: requests.PreparedRequest
+            ) -> HTTPResponse:
+                headers = HTTPHeaderDict()
+                body = BytesIO()
+                # See comments in responses._form_response for the mypy
+                # suppressions here.
+                orig_response = HTTPResponse(
+                    body=body,
+                    msg=headers,  # type: ignore[arg-type]
+                )
+                return HTTPResponse(
+                    status=requests.codes.ok,
+                    body=body,
+                    headers=headers,
+                    original_response=orig_response,  # type: ignore[arg-type]
+                    request_method=request.method,
+                )
+
+        debusine_client = DebusineHttpClient(self.api_url, self.token)
+        responses.add(EmptyResponse("PUT", self.api_url))
+
+        debusine_client._api_request("PUT", self.api_url, None)
+
     @responses.activate
     def test_debusine_error_details_combinations_everything(self) -> None:
         """Test combinations of status and body for _debusine_error_details."""
