@@ -26,6 +26,7 @@ from debusine.artifacts.models import ArtifactCategory, ArtifactData
 from debusine.db.context import context
 from debusine.db.models.files import File
 from debusine.db.models.permissions import (
+    AllowWorkers,
     PermissionUser,
     permission_check,
     permission_filter,
@@ -88,7 +89,7 @@ class ArtifactQuerySet(QuerySet["Artifact", A], Generic[A]):
         """Filter to artifacts in the current workspace."""
         return self.filter(workspace=context.require_workspace())
 
-    @permission_filter
+    @permission_filter(workers=AllowWorkers.ALWAYS)
     def can_display(self, user: PermissionUser) -> "ArtifactQuerySet[A]":
         """Keep only Artifacts that can be displayed."""
         # Delegate to workspace can_display check
@@ -107,6 +108,77 @@ class ArtifactQuerySet(QuerySet["Artifact", A], Generic[A]):
             )
         )
 
+    def not_expired(self, at: datetime) -> "ArtifactQuerySet[Any]":
+        """
+        Return queryset with artifacts that have not expired.
+
+        :param at: datetime to check if the artifacts are not expired.
+        :return: artifacts that expire_at is None (do not expire) or
+          expire_at is after the given datetime.
+        """
+        return self.annotate(
+            effective_expiration_delay=Coalesce(
+                "expiration_delay",
+                "workspace__default_expiration_delay",
+            )
+        ).filter(
+            Q(effective_expiration_delay=timedelta(0))
+            | Q(
+                # https://github.com/typeddjango/django-stubs/issues/1548
+                created_at__gt=(
+                    at - F("effective_expiration_delay")  # type: ignore[operator] # noqa: E501
+                )
+            )
+        )
+
+    def expired(self, at: datetime) -> "ArtifactQuerySet[Any]":
+        """
+        Return queryset with artifacts that have expired.
+
+        :param at: datetime to check if the artifacts are expired.
+        :return: artifacts that expire_at is before the given datetime.
+        """
+        return (
+            self.annotate(
+                effective_expiration_delay=Coalesce(
+                    "expiration_delay",
+                    "workspace__default_expiration_delay",
+                )
+            )
+            .exclude(effective_expiration_delay=timedelta(0))
+            .filter(
+                # https://github.com/typeddjango/django-stubs/issues/1548
+                created_at__lte=(
+                    at - F("effective_expiration_delay")  # type: ignore[operator] # noqa: E501
+                )
+            )
+        )
+
+    def part_of_collection_with_retains_artifacts(
+        self,
+    ) -> "ArtifactQuerySet[Any]":
+        """
+        Return Artifacts where the parent_collections has retains_artifacts set.
+
+        :return: Artifacts that are part of a retains_artifacts collection.
+        """
+        # Import here to prevent circular imports
+        from debusine.db.models.collections import Collection
+        from debusine.db.models.work_requests import WorkRequest
+
+        RetainsArtifacts = Collection.RetainsArtifacts
+        return self.filter(
+            Q(parent_collections__retains_artifacts=RetainsArtifacts.ALWAYS)
+            | Q(
+                parent_collections__retains_artifacts=RetainsArtifacts.WORKFLOW,
+                parent_collections__workflow__status__in={
+                    WorkRequest.Statuses.PENDING,
+                    WorkRequest.Statuses.RUNNING,
+                    WorkRequest.Statuses.BLOCKED,
+                },
+            )
+        )
+
 
 class ArtifactManager(models.Manager["Artifact"]):
     """Manager for the Artifact model."""
@@ -141,80 +213,6 @@ class ArtifactManager(models.Manager["Artifact"]):
 
         return artifact
 
-    def not_expired(self, at: datetime) -> QuerySet["Artifact"]:
-        """
-        Return queryset with artifacts that have not expired.
-
-        :param at: datetime to check if the artifacts are not expired.
-        :return: artifacts that expire_at is None (do not expire) or
-          expire_at is after the given datetime.
-        """
-        return (
-            self.get_queryset()
-            .annotate(
-                effective_expiration_delay=Coalesce(
-                    "expiration_delay",
-                    "workspace__default_expiration_delay",
-                )
-            )
-            .filter(
-                Q(effective_expiration_delay=timedelta(0))
-                | Q(
-                    # https://github.com/typeddjango/django-stubs/issues/1548
-                    created_at__gt=(
-                        at - F("effective_expiration_delay")  # type: ignore[operator] # noqa: E501
-                    )
-                )
-            )
-        )
-
-    def expired(self, at: datetime) -> QuerySet["Artifact"]:
-        """
-        Return queryset with artifacts that have expired.
-
-        :param at: datetime to check if the artifacts are expired.
-        :return: artifacts that expire_at is before the given datetime.
-        """
-        return (
-            self.get_queryset()
-            .annotate(
-                effective_expiration_delay=Coalesce(
-                    "expiration_delay",
-                    "workspace__default_expiration_delay",
-                )
-            )
-            .exclude(effective_expiration_delay=timedelta(0))
-            .filter(
-                # https://github.com/typeddjango/django-stubs/issues/1548
-                created_at__lte=(
-                    at - F("effective_expiration_delay")  # type: ignore[operator] # noqa: E501
-                )
-            )
-        )
-
-    def part_of_collection_with_retains_artifacts(self) -> QuerySet["Artifact"]:
-        """
-        Return Artifacts where the parent_collections has retains_artifacts set.
-
-        :return: Artifacts that are part of a retains_artifacts collection.
-        """
-        # Import here to prevent circular imports
-        from debusine.db.models.collections import Collection
-        from debusine.db.models.work_requests import WorkRequest
-
-        RetainsArtifacts = Collection.RetainsArtifacts
-        return self.get_queryset().filter(
-            Q(parent_collections__retains_artifacts=RetainsArtifacts.ALWAYS)
-            | Q(
-                parent_collections__retains_artifacts=RetainsArtifacts.WORKFLOW,
-                parent_collections__workflow__status__in={
-                    WorkRequest.Statuses.PENDING,
-                    WorkRequest.Statuses.RUNNING,
-                    WorkRequest.Statuses.BLOCKED,
-                },
-            )
-        )
-
 
 class Artifact(models.Model):
     """Artifact model."""
@@ -260,7 +258,9 @@ class Artifact(models.Model):
                 {"data": f"invalid artifact data: {e}"}
             ) from e
 
-    @permission_check("{user} cannot display artifact {resource}")
+    @permission_check(
+        "{user} cannot display artifact {resource}", workers=AllowWorkers.ALWAYS
+    )
     def can_display(self, user: PermissionUser) -> bool:
         """Check if the artifact can be displayed."""
         return self.workspace.can_display(user)
@@ -509,7 +509,7 @@ class ArtifactRelationQuerySet(QuerySet["ArtifactRelation", A], Generic[A]):
             artifact__workspace__scope=scope, target__workspace__scope=scope
         )
 
-    @permission_filter
+    @permission_filter(workers=AllowWorkers.ALWAYS)
     def can_display(
         self, user: PermissionUser
     ) -> "ArtifactRelationQuerySet[A]":
@@ -559,7 +559,10 @@ class ArtifactRelation(models.Model):
             )
         ]
 
-    @permission_check("{user} cannot display artifact relation {resource}")
+    @permission_check(
+        "{user} cannot display artifact relation {resource}",
+        workers=AllowWorkers.ALWAYS,
+    )
     def can_display(self, user: PermissionUser) -> bool:
         """Check if the artifact can be displayed."""
         return self.artifact.workspace.can_display(
