@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -9,25 +9,46 @@
 
 """Data models for db collections."""
 
+import re
 from datetime import datetime
 from functools import cached_property
-from typing import Any, ClassVar, Optional, TYPE_CHECKING, TypeAlias
+from typing import (
+    Any,
+    ClassVar,
+    Generic,
+    Optional,
+    TYPE_CHECKING,
+    TypeAlias,
+    TypeVar,
+)
 
 import jsonpath_rw
 from django.conf import settings
 from django.contrib.postgres.constraints import ExclusionConstraint
 from django.contrib.postgres.fields import RangeOperators
+from django.core.exceptions import ValidationError
 from django.db import models
 from django.db.models import CheckConstraint, F, Q, QuerySet, UniqueConstraint
+from django.db.models.fields.json import KT
 from django.urls import reverse
 
 from debusine.artifacts.models import (
+    ArtifactCategory,
     BareDataCategory,
+    BaseArtifactDataModel,
     CollectionCategory,
     DebusinePromise,
     SINGLETON_COLLECTION_CATEGORIES,
 )
+from debusine.client.models import model_to_json_serializable_dict
 from debusine.db.constraints import JsonDataUniqueConstraint
+from debusine.db.context import context
+from debusine.db.models.permissions import (
+    AllowWorkers,
+    PermissionUser,
+    permission_check,
+    permission_filter,
+)
 from debusine.db.models.workspaces import Workspace
 
 if TYPE_CHECKING:
@@ -40,6 +61,24 @@ if TYPE_CHECKING:
 else:
     TypedModelMeta = object
 
+A = TypeVar("A")
+
+#: Regexp matching the structure of collection names
+collection_name_regex = re.compile(r"^[A-Za-z][A-Za-z0-9+._-]*$")
+
+
+def is_valid_collection_name(value: str) -> bool:
+    """Check if value is a valid scope name."""
+    return bool(collection_name_regex.match(value))
+
+
+def validate_collection_name(value: str) -> None:
+    """Validate collection names."""
+    if not is_valid_collection_name(value):
+        raise ValidationError(
+            "%(value)r is not a valid collection name", params={"value": value}
+        )
+
 
 class _CollectionRetainsArtifacts(models.TextChoices):
     """Choices for Collection.retains_artifacts."""
@@ -49,22 +88,44 @@ class _CollectionRetainsArtifacts(models.TextChoices):
     ALWAYS = "always", "Always"
 
 
+class CollectionQuerySet(QuerySet["Collection", A], Generic[A]):
+    """Custom QuerySet for Collection."""
+
+    def in_current_scope(self) -> "CollectionQuerySet[A]":
+        """Filter to collections in the current scope."""
+        return self.filter(workspace__scope=context.require_scope())
+
+    def in_current_workspace(self) -> "CollectionQuerySet[A]":
+        """Filter to collections in the current workspace."""
+        return self.filter(workspace=context.require_workspace())
+
+    @permission_filter(workers=AllowWorkers.ALWAYS)
+    def can_display(self, user: PermissionUser) -> "CollectionQuerySet[A]":
+        """Keep only Collections that can be displayed."""
+        # Delegate to workspace can_display check
+        return self.filter(workspace__in=Workspace.objects.can_display(user))
+
+
 class CollectionManager(models.Manager["Collection"]):
     """Manager for Collection model."""
 
+    def get_queryset(self) -> CollectionQuerySet[Any]:
+        """Use the custom QuerySet."""
+        return CollectionQuerySet(self.model, using=self._db)
+
     @staticmethod
-    def create_singleton(
+    def get_or_create_singleton(
         category: CollectionCategory,
         workspace: Workspace,
         *,
         data: dict[str, Any] | None = None,
-    ) -> "Collection":
+    ) -> "tuple[Collection, bool]":
         """Create a singleton collection."""
         if category not in SINGLETON_COLLECTION_CATEGORIES:
             raise ValueError(
                 f"'{category}' is not a singleton collection category"
             )
-        return Collection.objects.create(
+        return Collection.objects.get_or_create(
             name="_", category=category, workspace=workspace, data=data or {}
         )
 
@@ -72,15 +133,19 @@ class CollectionManager(models.Manager["Collection"]):
 class Collection(models.Model):
     """Model representing a collection."""
 
-    objects = CollectionManager()
+    objects = CollectionManager.from_queryset(CollectionQuerySet)()
 
     RetainsArtifacts: TypeAlias = _CollectionRetainsArtifacts
 
-    name = models.CharField(max_length=255)
+    name = models.CharField(
+        max_length=255, validators=[validate_collection_name]
+    )
     category = models.CharField(max_length=255)
     full_history_retention_period = models.DurationField(null=True, blank=True)
     metadata_only_retention_period = models.DurationField(null=True, blank=True)
-    workspace = models.ForeignKey(Workspace, on_delete=models.PROTECT)
+    workspace = models.ForeignKey(
+        Workspace, on_delete=models.PROTECT, related_name="collections"
+    )
     retains_artifacts = models.CharField(
         max_length=8,
         choices=RetainsArtifacts.choices,
@@ -137,7 +202,7 @@ class Collection(models.Model):
         return CollectionManagerInterface.get_manager_for(self)
 
     def get_absolute_url(self) -> str:
-        """Return the canonical URL to display the item."""
+        """Return the canonical URL to display the collection."""
         return reverse(
             "workspaces:collections:detail",
             kwargs={
@@ -147,9 +212,50 @@ class Collection(models.Model):
             },
         )
 
+    def get_absolute_url_search(self) -> str:
+        """Return the canonical URL to search inside the collection."""
+        return reverse(
+            "workspaces:collections:search",
+            kwargs={
+                "wname": self.workspace.name,
+                "ccat": self.category,
+                "cname": self.name,
+            },
+        )
+
+    @permission_check(
+        "{user} cannot display collection {resource}",
+        workers=AllowWorkers.ALWAYS,
+    )
+    def can_display(self, user: PermissionUser) -> bool:
+        """Check if the collection can be displayed."""
+        return self.workspace.can_display(user)
+
     def __str__(self) -> str:
         """Return id, name, category."""
-        return f"Id: {self.id} Name: {self.name} Category: {self.category}"
+        # Stringify using a valid lookup syntax
+        return f"{self.name}@{self.category}"
+
+
+class CollectionItemQuerySet(QuerySet["CollectionItem"]):
+    """Custom QuerySet for CollectionItem."""
+
+    def artifacts_in_collection(
+        self, parent_collection: Collection, category: ArtifactCategory
+    ) -> "CollectionItemQuerySet":
+        """
+        Filter to artifacts in a given parent collection.
+
+        This is tuned to use the indexes on :py:class:`CollectionItem`.
+        """
+        return self.filter(
+            parent_collection=parent_collection,
+            # Technically redundant with parent_collection, but clues
+            # PostgreSQL into using the correct index.
+            parent_category=parent_collection.category,
+            child_type=CollectionItem.Types.ARTIFACT,
+            category=category,
+        )
 
 
 class CollectionItemManager(models.Manager["CollectionItem"]):
@@ -161,11 +267,14 @@ class CollectionItemManager(models.Manager["CollectionItem"]):
         *,
         parent_collection: Collection,
         name: str,
-        data: dict[str, Any],
+        data: BaseArtifactDataModel | dict[str, Any],
         created_by_user: "User",
         created_by_workflow: Optional["WorkRequest"],
     ) -> "CollectionItem":
         """Create a CollectionItem from bare data."""
+        if isinstance(data, BaseArtifactDataModel):
+            data = model_to_json_serializable_dict(data, exclude_unset=True)
+
         match category:
             case BareDataCategory.PROMISE:
                 # Raise ValueError if data is not valid
@@ -189,7 +298,7 @@ class CollectionItemManager(models.Manager["CollectionItem"]):
         name: str,
         data: dict[str, Any],
         created_by_user: "User",
-        created_by_workflow: Optional["WorkRequest"],
+        created_by_workflow: Optional["WorkRequest"] = None,
     ) -> "CollectionItem":
         """Create a CollectionItem from the artifact."""
         return CollectionItem.objects.create(
@@ -223,7 +332,7 @@ class CollectionItemManager(models.Manager["CollectionItem"]):
             created_by_user=created_by_user,
         )
 
-    def drop_full_history(self, at: datetime):
+    def drop_full_history(self, at: datetime) -> None:
         """
         Drop artifacts from collections after full_history_retention_period.
 
@@ -271,8 +380,10 @@ class _CollectionItemTypes(models.TextChoices):
 class CollectionItem(models.Model):
     """CollectionItem model."""
 
-    objects = CollectionItemManager()
-    active_objects = CollectionItemActiveManager()
+    objects = CollectionItemManager.from_queryset(CollectionItemQuerySet)()
+    active_objects = CollectionItemActiveManager.from_queryset(
+        CollectionItemQuerySet
+    )()
 
     name = models.CharField(max_length=255)
 
@@ -475,6 +586,30 @@ class CollectionItem(models.Model):
                 ),
             ),
         ]
+        indexes = [
+            models.Index(
+                F("parent_collection"),
+                KT("data__package"),
+                KT("data__version"),
+                name="%(app_label)s_ci_suite_source_idx",
+                condition=Q(
+                    parent_category=CollectionCategory.SUITE,
+                    child_type=_CollectionItemTypes.ARTIFACT,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                ),
+            ),
+            models.Index(
+                F("parent_collection"),
+                KT("data__srcpkg_name"),
+                KT("data__srcpkg_version"),
+                name="%(app_label)s_ci_suite_binary_source_idx",
+                condition=Q(
+                    parent_category=CollectionCategory.SUITE,
+                    child_type=_CollectionItemTypes.ARTIFACT,
+                    category=ArtifactCategory.BINARY_PACKAGE,
+                ),
+            ),
+        ]
 
     def save(self, *args: Any, **kwargs: Any) -> None:
         """Populate `parent_category` on save."""
