@@ -10,6 +10,7 @@
 """Unit tests for auth-related models."""
 
 import contextlib
+import re
 from collections.abc import Generator
 from datetime import timedelta
 from typing import ClassVar
@@ -18,6 +19,7 @@ from unittest import mock
 from asgiref.sync import sync_to_async
 from django.contrib.auth import get_user_model
 from django.contrib.auth.models import AnonymousUser
+from django.core.exceptions import ImproperlyConfigured
 from django.db import IntegrityError
 from django.utils import timezone
 
@@ -36,6 +38,8 @@ from debusine.db.models.auth import (
     ClientEnroll,
     GroupAuditLog,
     GroupMembership,
+    GroupRoleBase,
+    GroupRoles,
     system_user,
 )
 from debusine.db.playground import scenarios
@@ -317,6 +321,52 @@ class IdentityTests(TestCase):
         self.assertEqual(str(ident), "salsa:test@debian.org")
 
 
+class GroupRolesTests(TestCase):
+    """Tests for the GroupRoles class."""
+
+    scenario = scenarios.DefaultContext()
+
+    def test_init(self) -> None:
+        r = GroupRoleBase("test")
+        r._setup()
+        self.assertEqual(r.implied_by_scope_roles, frozenset())
+        self.assertEqual(r.implied_by_group_roles, frozenset([r]))
+
+    def test_init_invalid_implication(self) -> None:
+        r = GroupRoleBase("test", implied_by=[Workspace.Roles.OWNER])
+        with self.assertRaisesRegex(
+            ImproperlyConfigured,
+            r"Group roles do not support implications"
+            r" by <enum 'WorkspaceRoles'>",
+        ):
+            r._setup()
+
+    def test_choices(self) -> None:
+        self.assertEqual(
+            GroupRoles.choices,
+            [("admin", "Admin"), ("member", "Member")],
+        )
+
+    def test_implications(self) -> None:
+        self.assertEqual(
+            Group.Roles.ADMIN.implied_by_scope_roles,
+            frozenset([Scope.Roles.OWNER]),
+        )
+        self.assertEqual(
+            Group.Roles.ADMIN.implied_by_group_roles,
+            frozenset([Group.Roles.ADMIN]),
+        )
+
+        self.assertEqual(
+            Group.Roles.MEMBER.implied_by_scope_roles,
+            frozenset([Scope.Roles.OWNER]),
+        )
+        self.assertEqual(
+            Group.Roles.MEMBER.implied_by_group_roles,
+            frozenset([Group.Roles.ADMIN, Group.Roles.MEMBER]),
+        )
+
+
 class GroupManagerTests(TestCase):
     """Test for GroupManager class."""
 
@@ -529,7 +579,7 @@ class GroupTests(TestCase):
         self.group.add_user(self.scenario.user, Group.Roles.ADMIN)
         self.assertRole(self.group, self.scenario.user, Group.Roles.ADMIN)
         with self.assertRaisesRegex(
-            AssertionError, "'admin' != GroupRoles.MEMBER"
+            AssertionError, re.escape(f"'admin' != {Group.Roles.MEMBER!r}")
         ):
             self.assertRole(self.group, self.scenario.user, Group.Roles.MEMBER)
         with self.assertRaisesRegex(
@@ -643,7 +693,10 @@ class GroupTests(TestCase):
         """Try assigning a role using the wrong enum."""
         with self.assertRaisesRegex(
             TypeError,
-            r"ScopeRoles.OWNER cannot be converted to <enum 'WorkspaceRoles'>",
+            re.escape(
+                f"{Scope.Roles.OWNER!r} cannot be converted to"
+                " <enum 'WorkspaceRoles'>"
+            ),
         ):
             self.group.assign_role(self.scenario.workspace, Scope.Roles.OWNER)
 
