@@ -12,18 +12,46 @@ from typing import ClassVar
 from unittest import mock
 
 from django.contrib.auth.models import AnonymousUser
-from django.core.exceptions import ValidationError
+from django.core.exceptions import ImproperlyConfigured, ValidationError
 from django.db import IntegrityError, transaction
 from django.test import override_settings
 
 from debusine.db.context import context
 from debusine.db.models import FileInStore, FileStore, Group, Scope
 from debusine.db.models.permissions import PartialCheckResult
-from debusine.db.models.scopes import ScopeRole, is_valid_scope_name
+from debusine.db.models.scopes import (
+    ScopeRole,
+    ScopeRoleBase,
+    ScopeRoles,
+    is_valid_scope_name,
+)
 from debusine.db.playground import scenarios
 from debusine.test.django import TestCase
 
 
+class ScopeRolesTests(TestCase):
+    """Tests for the ScopeRoles class."""
+
+    def test_implications(self) -> None:
+        self.assertEqual(ScopeRoles.OWNER.label, "Owner")
+        self.assertEqual(
+            ScopeRoles.OWNER.implied_by_scope_roles,
+            frozenset([ScopeRoles.OWNER]),
+        )
+
+    def test_init_invalid_implication(self) -> None:
+        r = ScopeRoleBase("test", implied_by=[Scope.Roles.OWNER])
+        with self.assertRaisesRegex(
+            ImproperlyConfigured,
+            r"Scope roles do not support implications"
+            r" by <enum 'ScopeRoles'>",
+        ):
+            r._setup()
+
+    def test_choices(self) -> None:
+        self.assertEqual(ScopeRoles.choices, [("owner", "Owner")])
+
+
 class ScopeManagerTests(TestCase):
     """Tests for the ScopeManager class."""
 
@@ -188,13 +216,6 @@ class ScopeTests(TestCase):
             scope.context_has_role(self.scenario.user, Scope.Roles.OWNER)
         )
 
-    def test_context_has_role_no_roles(self) -> None:
-        """Test context_has_role with no roles."""
-        with self.assertRaisesRegex(
-            ValueError, r"context_has_role needs at least one role"
-        ):
-            self.scenario.scope.context_has_role(self.scenario.user, ())
-
     def test_context_has_role(self) -> None:
         """Test context_has_role."""
         scope = self.scenario.scope
@@ -203,7 +224,6 @@ class ScopeTests(TestCase):
         with context.local():
             self.scenario.set_current()
             self.assertDeny(scope.context_has_role(user, Scope.Roles.OWNER))
-            self.assertDeny(scope.context_has_role(user, [Scope.Roles.OWNER]))
 
         self.playground.create_group_role(
             scope, Scope.Roles.OWNER, users=[self.scenario.user]
@@ -212,7 +232,6 @@ class ScopeTests(TestCase):
         with context.local():
             self.scenario.set_current()
             self.assertAllow(scope.context_has_role(user, Scope.Roles.OWNER))
-            self.assertAllow(scope.context_has_role(user, [Scope.Roles.OWNER]))
 
     def test_upload_file_stores(self) -> None:
         """`upload_file_stores` returns stores in the correct order."""
@@ -500,6 +519,7 @@ class ScopeRoleTests(TestCase):
     group2: ClassVar[Group]
 
     @classmethod
+    @context.disable_permission_checks()
     def setUpTestData(cls) -> None:
         """Set up common test data."""
         super().setUpTestData()
