@@ -396,6 +396,38 @@ class WorkerManagerTests(TestCase):
         self.assertGreaterEqual(idle_time, 6 * 3600)
         self.assertLessEqual(idle_time, 7 * 3600)
 
+    def test_mark_all_disconnected(self) -> None:
+        """WorkerManager.mark_all_disconnected() affects non-Celery workers."""
+        worker_connected_external = self.playground.create_worker(
+            fqdn="connected-external"
+        )
+        worker_connected_external.mark_connected()
+        worker_connected_signing = self.playground.create_worker(
+            worker_type=WorkerType.SIGNING, fqdn="connected-signing"
+        )
+        worker_connected_signing.mark_connected()
+        worker_connected_celery = Worker.objects.get_or_create_celery()
+        worker_connected_celery.mark_connected()
+        worker_disconnected_external = self.playground.create_worker(
+            fqdn="disconnected-external"
+        )
+        worker_disconnected_signing = self.playground.create_worker(
+            worker_type=WorkerType.SIGNING, fqdn="disconnected-signing"
+        )
+
+        Worker.objects.mark_all_disconnected()
+
+        worker_connected_celery.refresh_from_db()
+        self.assertTrue(worker_connected_celery.connected())
+        for disconnected in (
+            worker_connected_external,
+            worker_connected_signing,
+            worker_disconnected_external,
+            worker_disconnected_signing,
+        ):
+            disconnected.refresh_from_db()
+            self.assertFalse(disconnected.connected())
+
 
 class WorkerTests(TestCase):
     """Tests for the Worker model."""
