@@ -281,6 +281,10 @@ def _lookup_dict(
     """
     Look up collection items using the dictionary lookup syntax.
 
+    Results are returned in a predictable order (currently ordered by
+    collection item ID, although this function only guarantees that the
+    order will be stable between successive calls).
+
     See :ref:`lookup-dict`.
     """
     if (
@@ -363,7 +367,7 @@ def _lookup_dict(
             artifact=item.artifact,
             collection=item.collection,
         )
-        for item in objects.all()
+        for item in objects.order_by("id")
     ]
 
 
