@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -17,12 +17,8 @@ from debusine.artifacts.models import (
     BareDataCategory,
     CollectionCategory,
 )
-from debusine.db.models import (
-    Collection,
-    CollectionItem,
-    WorkRequest,
-    default_workspace,
-)
+from debusine.db.context import context
+from debusine.db.models import Collection, CollectionItem, default_workspace
 from debusine.server.collections import (
     CollectionManagerInterface,
     DebianSuiteManager,
@@ -49,8 +45,8 @@ class CollectionManagerInterfaceTests(TestCase):
             name="test",
             task_name="noop",
         )
-        self.workflow = WorkRequest.objects.create_workflow(
-            template=template, data={}, created_by=self.user
+        self.workflow = self.playground.create_workflow(
+            template, task_data={}, created_by=self.user
         )
 
         self.workspace = default_workspace()
@@ -113,10 +109,11 @@ class CollectionManagerInterfaceTests(TestCase):
             replace=False,
         )
 
+    @context.disable_permission_checks()
     def test_add_artifact(self) -> None:
         """Test add_artifact() call do_add_artifact()."""
         category = ArtifactCategory.TEST
-        artifact, _ = self.create_artifact(category=category)
+        artifact, _ = self.playground.create_artifact(category=category)
 
         self.manager.VALID_ARTIFACT_CATEGORIES = frozenset({category})
 
@@ -132,12 +129,13 @@ class CollectionManagerInterfaceTests(TestCase):
             replace=False,
         )
 
+    @context.disable_permission_checks()
     def test_add_artifact_invalid_category_raise_item_addition_error(
         self,
     ) -> None:
         """Test add_artifact() raise ItemAdditionError(): invalid category."""
         category = "invalid"
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=category,  # type: ignore[arg-type]
         )
 
@@ -153,10 +151,11 @@ class CollectionManagerInterfaceTests(TestCase):
             user=self.user,
         )
 
+    @context.disable_permission_checks()
     def test_add_artifact_any_category_valid(self) -> None:
         """If VALID_ARTIFACT_CATEGORIES is None, any category is valid."""
         category = ArtifactCategory.TEST
-        artifact, _ = self.create_artifact(category=category)
+        artifact, _ = self.playground.create_artifact(category=category)
         self.manager.VALID_ARTIFACT_CATEGORIES = None
 
         with patch.object(self.manager, "do_add_artifact") as mocked:
@@ -247,9 +246,10 @@ class CollectionManagerInterfaceTests(TestCase):
             "test", user=self.user, workflow=self.workflow
         )
 
+    @context.disable_permission_checks()
     def test_remove_artifact(self) -> None:
         """Test remove_artifact() call do_remove_artifact()."""
-        artifact, _ = self.create_artifact()
+        artifact, _ = self.playground.create_artifact()
 
         with patch.object(self.manager, "do_remove_artifact") as mocked:
             self.manager.remove_artifact(
@@ -273,9 +273,10 @@ class CollectionManagerInterfaceTests(TestCase):
             collection, user=self.user, workflow=self.workflow
         )
 
+    @context.disable_permission_checks()
     def test_do_remove_child_types_artifact(self) -> None:
         """Test do_remove_child_types() call do_remove_artifact()."""
-        artifact, _ = self.create_artifact()
+        artifact, _ = self.playground.create_artifact()
         CollectionItem.objects.create(
             name="test",
             artifact=artifact,
