@@ -1,4 +1,4 @@
-# Copyright 2021-2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -24,7 +24,7 @@ from rest_framework.renderers import JSONRenderer
 
 from debusine.db.models import Token, WorkRequest, Worker, Workspace
 from debusine.server.serializers import OnWorkRequestCompleted
-from debusine.server.views.base import (
+from debusine.server.views.rest import (
     IsTokenUserAuthenticated,
     IsWorkerAuthenticated,
 )
@@ -35,14 +35,16 @@ logger = logging.getLogger(__name__)
 class RequestAdaptor:
     """Minimal class used when helper classes expect a Request."""
 
-    def __init__(self, scope: dict[str, Any]):
+    def __init__(self, scope: dict[str, Any]) -> None:
         """Initialize RequestAdaptor."""
-        self.token = scope["token"]
+        self.auth = scope["token"]
 
 
 class ConsumerMixin:
     """Mixin used for different consumer classes."""
 
+    scope: dict[str, Any]
+
     def get_header_value(
         self: AsyncWebsocketConsumer, header_field_name: str
     ) -> str | None:
@@ -57,13 +59,14 @@ class ConsumerMixin:
 
         for name, value in self.scope['headers']:
             if name == encoded_header_field_name:
+                assert isinstance(value, bytes)
                 return value.decode('utf-8')
 
         return None
 
     async def reject_connection(
         self: AsyncWebsocketConsumer,
-        reason,
+        reason: str,
         *,
         reason_code: str | None = None,
     ) -> None:
@@ -83,15 +86,20 @@ class ConsumerMixin:
         self, permission_class: type[BasePermission]
     ) -> bool:
         """Return bool checking if the request has permission to connect."""
-        scope = self.scope  # type: ignore[attr-defined]
-        request_adaptor = RequestAdaptor(scope)
+        request_adaptor = RequestAdaptor(self.scope)
 
-        return await database_sync_to_async(permission_class().has_permission)(
-            request_adaptor, None
-        )
+        has_permission = await database_sync_to_async(
+            permission_class().has_permission
+        )(request_adaptor, None)
+        assert isinstance(has_permission, bool)
+        return has_permission
 
 
-class WorkerConsumer(ConsumerMixin, AsyncWebsocketConsumer):
+# mypy complains that AsyncWebsocketConsumer is untyped, which is true, but
+# we can't fix that here.
+class WorkerConsumer(
+    ConsumerMixin, AsyncWebsocketConsumer  # type: ignore[misc]
+):
     """
     Implement server-side of a Worker.
 
@@ -111,7 +119,7 @@ class WorkerConsumer(ConsumerMixin, AsyncWebsocketConsumer):
 
     async def _channel_layer_group_add(
         self, token_hash: str, channel_name: str
-    ):
+    ) -> None:
         await self.channel_layer.group_add(token_hash, channel_name)
 
     async def connect(self) -> None:
@@ -169,7 +177,7 @@ class WorkerConsumer(ConsumerMixin, AsyncWebsocketConsumer):
         if await database_sync_to_async(self._worker.is_busy)():
             await self._send_work_request_available()
 
-    async def _send_work_request_available(self):
+    async def _send_work_request_available(self) -> None:
         await self.send(
             text_data=json.dumps(
                 {
@@ -179,12 +187,12 @@ class WorkerConsumer(ConsumerMixin, AsyncWebsocketConsumer):
             )
         )
 
-    async def _dynamic_metadata_refresher(self):
+    async def _dynamic_metadata_refresher(self) -> None:
         while True:
             await self._send_dynamic_metadata_request()
             await asyncio.sleep(self.REQUEST_DYNAMIC_METADATA_SECONDS)
 
-    async def _send_dynamic_metadata_request(self):
+    async def _send_dynamic_metadata_request(self) -> None:
         await self.send(
             text_data=json.dumps(
                 {
@@ -194,12 +202,14 @@ class WorkerConsumer(ConsumerMixin, AsyncWebsocketConsumer):
             )
         )
 
-    async def disconnect(self, close_code):
+    async def disconnect(self, close_code: Any) -> None:
         """Worker has disconnected. Cancel tasks, mark as disconnect, etc."""
         if self._request_dynamic_metadata_task:
             self._request_dynamic_metadata_task.cancel()
 
         if self._worker:
+            assert self._worker.token is not None
+
             await self.channel_layer.group_discard(
                 self._worker.token.hash, self.channel_name
             )
@@ -212,8 +222,13 @@ class WorkerConsumer(ConsumerMixin, AsyncWebsocketConsumer):
                 close_code,
             )
 
-    async def worker_disabled(self, event):  # noqa: U100
+    async def worker_disabled(
+        self, event: dict[str, Any]  # noqa: U100
+    ) -> None:
         """Worker has been disabled. Send a connection_closed msg."""
+        assert self._worker is not None
+        assert self._worker.token is not None
+
         logger.info("Worker %s disabled", self._worker.name)
 
         msg = {
@@ -224,21 +239,27 @@ class WorkerConsumer(ConsumerMixin, AsyncWebsocketConsumer):
         }
         await self.send(text_data=json.dumps(msg), close=True)
 
-    async def work_request_assigned(self, event):  # noqa: U100
+    async def work_request_assigned(
+        self, event: dict[str, Any]  # noqa: U100
+    ) -> None:
         """Work Request has been assigned to the worker. Send channel msg."""
         await self._send_work_request_available()
 
 
-class OnWorkRequestCompletedConsumer(ConsumerMixin, AsyncWebsocketConsumer):
+# mypy complains that AsyncWebsocketConsumer is untyped, which is true, but
+# we can't fix that here.
+class OnWorkRequestCompletedConsumer(
+    ConsumerMixin, AsyncWebsocketConsumer  # type: ignore[misc]
+):
     """Send work_request_completed notifications."""
 
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         """Initialize OnWorkRequestCompletedConsumer."""
         super().__init__(*args, **kwargs)
 
-        # If _workspaces is not None: list of the workspaces that are being
+        # If _workspaces is not None: set of the workspaces that are being
         # monitored, otherwise all the workspaces that the client has access to
-        self._workspaces_only: list[int] | None = None
+        self._workspaces_only: set[int] | None = None
 
         self._query_string: dict[str, str] = {}
 
@@ -247,13 +268,13 @@ class OnWorkRequestCompletedConsumer(ConsumerMixin, AsyncWebsocketConsumer):
         # This was fixed in 4.1.0, but work around it.
         self._channel_added_to_group: bool = False
 
-    async def _get_workspaces_from_query_params(self):
-        workspace_names = self._query_string.get("workspaces")
+    async def _get_workspaces_from_query_params(self) -> set[int] | None:
+        workspace_names_str = self._query_string.get("workspaces")
 
-        if workspace_names is None:
-            return
+        if workspace_names_str is None:
+            return None
 
-        workspace_names = workspace_names.split(",")
+        workspace_names = workspace_names_str.split(",")
 
         def get_workspace_ids(workspace_names: list[str]) -> set[int]:
             # TODO: check that the current user has access to each
@@ -274,10 +295,11 @@ class OnWorkRequestCompletedConsumer(ConsumerMixin, AsyncWebsocketConsumer):
         workspace_ids = await database_sync_to_async(get_workspace_ids)(
             workspace_names
         )
+        assert isinstance(workspace_ids, set)
 
         return workspace_ids
 
-    async def connect(self):
+    async def connect(self) -> None:
         """Client is connecting."""
         self._query_string = dict(
             parse_qsl(self.scope["query_string"].decode("utf-8"))
@@ -313,7 +335,7 @@ class OnWorkRequestCompletedConsumer(ConsumerMixin, AsyncWebsocketConsumer):
 
         await self._send_pending_work_requests_since_last_completed()
 
-    async def disconnect(self, close_code):  # noqa: U100
+    async def disconnect(self, close_code: Any) -> None:  # noqa: U100
         """Client is disconnecting, clean-up."""
         if self._channel_added_to_group:
             await self.channel_layer.group_discard(
@@ -366,7 +388,7 @@ class OnWorkRequestCompletedConsumer(ConsumerMixin, AsyncWebsocketConsumer):
                 work_request.result,
             )
 
-    async def work_request_completed(self, event):
+    async def work_request_completed(self, event: dict[str, Any]) -> None:
         """Work Request has completed."""
         work_request_id = event["work_request_id"]
         work_space_id = event["workspace_id"]
