@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -17,11 +17,11 @@ from debusine.artifacts.models import ArtifactCategory, CollectionCategory
 from debusine.db.context import context
 from debusine.db.models import (
     ArtifactRelation,
-    WorkRequest,
     WorkflowTemplate,
     Workspace,
     default_workspace,
 )
+from debusine.db.playground import Playground, scenarios
 from debusine.django.management.tests import call_command
 from debusine.test.django import TestCase
 
@@ -30,17 +30,28 @@ class DeleteWorkspaceCommandTests(TestCase):
     """Tests for the delete_workspace command."""
 
     @context.disable_permission_checks()
-    def create_test_workspace(self):
+    def create_test_workspace(self) -> Workspace:
         """Create a test workspace."""
         scope = self.playground.get_or_create_scope("scope")
         workspace = self.playground.create_workspace(name="Test", scope=scope)
+        self.playground.create_group_role(
+            workspace,
+            Workspace.Roles.CONTRIBUTOR,
+            [self.playground.get_default_user()],
+        )
+        for architecture in ("amd64", "s390x"):
+            self.playground.create_debian_environment(
+                codename="bookworm",
+                architecture=architecture,
+                workspace=workspace,
+            )
         work_request = self.playground.create_work_request(
             workspace=workspace, task_name="noop"
         )
         collection = self.playground.create_collection(
             "test", CollectionCategory.WORKFLOW_INTERNAL, workspace=workspace
         )
-        artifact_hello, _ = self.create_artifact(
+        artifact_hello, _ = self.playground.create_artifact(
             category=ArtifactCategory.SOURCE_PACKAGE,
             data={
                 "name": "hello",
@@ -60,7 +71,7 @@ class DeleteWorkspaceCommandTests(TestCase):
         artifact_hello.created_by_work_request = work_request
         artifact_hello.save()
 
-        artifact_hello_traditional, _ = self.create_artifact(
+        artifact_hello_traditional, _ = self.playground.create_artifact(
             category=ArtifactCategory.SOURCE_PACKAGE,
             data={
                 "name": "hello-traditional",
@@ -77,7 +88,7 @@ class DeleteWorkspaceCommandTests(TestCase):
             create_files=True,
             skip_add_files_in_store=True,
         )
-        self.create_artifact_relation(
+        self.playground.create_artifact_relation(
             artifact_hello_traditional,
             artifact_hello,
             ArtifactRelation.Relations.RELATES_TO,
@@ -110,29 +121,27 @@ class DeleteWorkspaceCommandTests(TestCase):
             workspace=workspace,
         )
 
-        workflow = WorkRequest.objects.create_workflow(
-            template=sbuild_template,
-            data={
+        workflow = self.playground.create_workflow(
+            sbuild_template,
+            task_data={
                 "input": {
                     "source_artifact": udev.pk,
                 },
-                "backend": "schroot",
                 "target_distribution": "debian:bookworm",
                 "architectures": ["all", "amd64", "s390x"],
             },
-            created_by=self.playground.get_default_user(),
         )
         workflow.mark_running()
         workflow.save()
 
-        self.assertIsNotNone(workflow.internal_collection)
+        assert workflow.internal_collection is not None
         self.assertEqual(workflow.internal_collection.workflow, workflow)
 
         # TODO: add more kinds of elements to the workspace, to make
         # sure deletion catches them
         return workspace
 
-    def test_delete_missing_workspace(self):
+    def test_delete_missing_workspace(self) -> None:
         """Workspace does not exist."""
         self.playground.get_or_create_scope("scope")
         with self.assertRaisesRegex(
@@ -169,7 +178,7 @@ class DeleteWorkspaceCommandTests(TestCase):
         )
         self.assertEqual('', stdout)
 
-    def test_delete_workspace(self):
+    def test_delete_workspace(self) -> None:
         """Delete an existing workspace."""
         workspace = self.create_test_workspace()
 
@@ -186,7 +195,28 @@ class DeleteWorkspaceCommandTests(TestCase):
         with self.assertRaises(Workspace.DoesNotExist):
             Workspace.objects.get(name="Test")
 
-    def test_delete_workspace_confirmation(self):
+    def test_delete_workspace_playground_ui_scenario(self) -> None:
+        """Delete a workspace with the playground UI scenario."""
+        playground = Playground(default_workspace_name="Playground")
+        scenario = scenarios.UIPlayground()
+        playground.build_scenario(scenario)
+
+        stdout, stderr, _ = call_command(
+            'delete_workspace',
+            str(scenario.workspace),
+            "--yes",
+        )
+        self.assertEqual('', stdout)
+
+        with self.assertRaises(Workspace.DoesNotExist):
+            Workspace.objects.get(id=scenario.workspace.id)
+
+        with self.assertRaises(Workspace.DoesNotExist):
+            Workspace.objects.get(
+                scope=scenario.scope, name=scenario.workspace.name
+            )
+
+    def test_delete_workspace_confirmation(self) -> None:
         """delete_workspace doesn't delete (user does not confirm)."""
         workspace = self.create_test_workspace()
 
@@ -208,7 +238,7 @@ class DeleteWorkspaceCommandTests(TestCase):
             Workspace.objects.filter(name="Test"), [workspace]
         )
 
-    def test_delete_default_workspace(self):
+    def test_delete_default_workspace(self) -> None:
         """Default workspace does not get deleted."""
         workspace = default_workspace()
         with self.assertRaisesRegex(
@@ -223,7 +253,7 @@ class DeleteWorkspaceCommandTests(TestCase):
             )
         self.assertEqual(exc.exception.returncode, 3)
 
-    def test_delete_workspace_with_scope_not_found(self):
+    def test_delete_workspace_with_scope_not_found(self) -> None:
         """delete_workspace: scope not found."""
         with self.assertRaisesRegex(
             CommandError, 'Scope "nonexistent" not found'
