@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -14,7 +14,7 @@ from typing import ClassVar
 from django.core.management import CommandError
 
 from debusine.db.context import context
-from debusine.db.models import FileStore, Workspace
+from debusine.db.models import Scope, Workspace
 from debusine.django.management.tests import call_command
 from debusine.test.django import TestCase
 
@@ -22,11 +22,12 @@ from debusine.test.django import TestCase
 class ManageWorkspaceCommandTests(TestCase):
     """Tests for manage_workspace management command."""
 
+    scope: ClassVar[Scope]
     workspace: ClassVar[Workspace]
 
     @classmethod
     @context.disable_permission_checks()
-    def setUpTestData(cls):
+    def setUpTestData(cls) -> None:
         """Set up test fixture."""
         super().setUpTestData()
         cls.scope = cls.playground.get_or_create_scope(name="scope")
@@ -34,7 +35,7 @@ class ManageWorkspaceCommandTests(TestCase):
             scope=cls.scope, name="test"
         )
 
-    def test_no_default_reset(self):
+    def test_no_default_reset(self) -> None:
         """Don't confuse no-option with reset-to-default."""
         # Non-default values
         default_expiration_delay = 7
@@ -57,7 +58,7 @@ class ManageWorkspaceCommandTests(TestCase):
             timedelta(default_expiration_delay),
         )
 
-    def test_change_public(self):
+    def test_change_public(self) -> None:
         """manage_workspace changes the public status."""
         for public in (True, False):
             stdout, stderr, exit_code = call_command(
@@ -73,7 +74,7 @@ class ManageWorkspaceCommandTests(TestCase):
             self.workspace.refresh_from_db()
             self.assertEqual(self.workspace.public, public)
 
-    def test_change_public_not_found(self):
+    def test_change_public_not_found(self) -> None:
         """manage_workspace workspace not found."""
         name = "non-existing-workspace"
 
@@ -87,7 +88,7 @@ class ManageWorkspaceCommandTests(TestCase):
 
         self.assertEqual(exc.exception.returncode, 3)
 
-    def test_change_default_expiration_delay(self):
+    def test_change_default_expiration_delay(self) -> None:
         """manage_workspace changes the default expiration delay."""
         default_expiration_delay = 123456
 
@@ -102,7 +103,7 @@ class ManageWorkspaceCommandTests(TestCase):
         stdout, stderr, exit_code = call_command(
             "manage_workspace",
             "--default-expiration-delay",
-            default_expiration_delay,
+            str(default_expiration_delay),
             str(self.workspace),
         )
 
@@ -122,7 +123,7 @@ class ManageWorkspaceCommandTests(TestCase):
         call_command(
             "manage_workspace",
             "--default-expiration-delay",
-            default_expiration_delay,
+            str(default_expiration_delay),
             str(self.workspace),
         )
 
@@ -132,52 +133,14 @@ class ManageWorkspaceCommandTests(TestCase):
             timedelta(default_expiration_delay),
         )
 
-    def test_change_default_file_store(self):
-        """manage_workspace changes the default file store."""
-        default_file_store = FileStore.objects.create(
-            name="test", backend=FileStore.BackendChoices.LOCAL
-        )
-
-        stdout, stderr, exit_code = call_command(
-            "manage_workspace",
-            "--default-file-store",
-            "test",
-            str(self.workspace),
-        )
-
-        self.assertEqual(stdout, "")
-        self.assertEqual(stderr, "")
-        self.assertEqual(exit_code, 0)
-
-        self.workspace.refresh_from_db()
-        self.assertEqual(self.workspace.default_file_store, default_file_store)
-
-    def test_change_default_file_store_not_found(self):
-        """manage_workspace: new default file store not found."""
-        with self.assertRaisesRegex(
-            CommandError, 'File store "nonexistent" not found'
-        ) as exc:
-            call_command(
-                "manage_workspace",
-                "--default-file-store",
-                "nonexistent",
-                str(self.workspace),
-            )
-
-        self.assertEqual(exc.exception.returncode, 3)
-
-    def test_change_all(self):
+    def test_change_all(self) -> None:
         """manage_workspace changes multiple fields at once."""
         default_expiration_delay = 7
-        default_file_store = FileStore.objects.create(
-            name="test", backend=FileStore.BackendChoices.LOCAL
-        )
 
         stdout, stderr, exit_code = call_command(
             "manage_workspace",
             "--public",
             f"--default-expiration-delay={default_expiration_delay}",
-            "--default-file-store=test",
             str(self.workspace),
         )
 
@@ -191,9 +154,8 @@ class ManageWorkspaceCommandTests(TestCase):
             self.workspace.default_expiration_delay,
             timedelta(default_expiration_delay),
         )
-        self.assertEqual(self.workspace.default_file_store, default_file_store)
 
-    def test_delete_workspace_with_scope_not_found(self):
+    def test_delete_workspace_with_scope_not_found(self) -> None:
         """manage_workspace: scope not found."""
         with self.assertRaisesRegex(
             CommandError, 'Scope "nonexistent" not found'
