@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -8,52 +8,20 @@
 # contained in the LICENSE file.
 
 """Middleware for updating Worker.last_seen_at."""
-from collections.abc import Callable
-from typing import Any
+from collections.abc import Awaitable, Callable
+from typing import Any, TYPE_CHECKING
 
-import django
 from channels.db import database_sync_to_async
 from django.utils import timezone
 
 from debusine.db.context import context
 from debusine.db.models import Token
 
+if TYPE_CHECKING:
+    from django.core.handlers.asgi import _ReceiveCallback, _SendCallback
 
-class TokenLastSeenAtMiddleware:
-    """
-    Save the Token in the scope and update Worker.last_seen_at.
-
-    Token available in the request helps to avoid double lookups.
-    """
-
-    def __init__(
-        self,
-        get_response: Callable[
-            [django.http.HttpRequest], django.http.HttpResponse
-        ],
-    ):
-        """Middleware API entry point."""
-        self.get_response = get_response
-
-    def __call__(self, request: django.http.HttpRequest):
-        """Middleware entry point."""
-        token_key = request.headers.get("token")
-
-        if token_key is None:
-            request.token = None  # type: ignore[attr-defined]
-            return self.get_response(request)
-
-        token = Token.objects.get_token_or_none(token_key)
-
-        if token is not None:
-            token.last_seen_at = timezone.now()
-            token.save()
-            if token.enabled and hasattr(token, "worker"):
-                context.set_worker_token(token)
-
-        request.token = token  # type: ignore[attr-defined]
-
-        return self.get_response(request)
+    _ReceiveCallback  # fake usage for vulture
+    _SendCallback  # fake usage for vulture
 
 
 class TokenLastSeenAtMiddlewareChannels:
@@ -63,12 +31,18 @@ class TokenLastSeenAtMiddlewareChannels:
     Token available in the scope helps to avoid double lookups.
     """
 
-    def __init__(self, app):
+    def __init__(
+        self,
+        app: Callable[
+            [dict[str, Any], "_ReceiveCallback", "_SendCallback"],
+            Awaitable[None],
+        ],
+    ) -> None:
         """Middleware API entry point."""
         self.app = app
 
     @staticmethod
-    def get_token_header(headers: list[tuple[str, Any]]) -> str | None:
+    def get_token_header(headers: list[tuple[bytes, bytes]]) -> str | None:
         """Return the token key from the headers or None."""
         for header in headers:
             if header[0].lower() == b"token":
@@ -78,9 +52,18 @@ class TokenLastSeenAtMiddlewareChannels:
 
     async def is_worker_token(self, token: Token) -> bool:
         """Check if a token is a worker token."""
-        return await database_sync_to_async(lambda: hasattr(token, "worker"))()
+        is_worker_token = await database_sync_to_async(
+            lambda: hasattr(token, "worker")
+        )()
+        assert isinstance(is_worker_token, bool)
+        return is_worker_token
 
-    async def __call__(self, scope: dict[str, Any], receive, send):
+    async def __call__(
+        self,
+        scope: dict[str, Any],
+        receive: "_ReceiveCallback",
+        send: "_SendCallback",
+    ) -> None:
         """Middleware entry point."""
         # When middleware is modifying the scope, it should make a copy of the
         # scope object before mutating it.
