@@ -1,4 +1,4 @@
-# Copyright 2021-2022 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -10,9 +10,12 @@
 """Mapping database models to REST Api support."""
 
 from collections.abc import Callable
-from typing import Any, Never
+from copy import copy
+from typing import Any, Never, NoReturn, TYPE_CHECKING, TypeAlias
 
-from django.urls import reverse
+from django.conf import settings
+from django.db.models import QuerySet
+from django.http import HttpRequest
 
 try:
     import pydantic.v1 as pydantic
@@ -20,14 +23,19 @@ except ImportError:
     import pydantic as pydantic  # type: ignore
 
 from rest_framework import serializers
-from rest_framework.exceptions import ValidationError
-from rest_framework.request import Request
+from rest_framework.exceptions import ErrorDetail, ValidationError
 
 from debusine.artifacts.models import ArtifactCategory
-from debusine.client.models import LookupChildType
+from debusine.assets import AssetCategory, asset_data_model
+from debusine.client.models import (
+    LookupChildType,
+    model_to_json_serializable_dict,
+)
+from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
     ArtifactRelation,
+    Asset,
     Collection,
     CollectionItem,
     File,
@@ -35,21 +43,86 @@ from debusine.db.models import (
     WorkRequest,
     WorkflowTemplate,
     Workspace,
-    default_workspace,
 )
 from debusine.server.collections.lookup import LookupResult
+from debusine.server.exceptions import (
+    DebusineAPIException,
+    raise_workspace_not_found,
+)
 from debusine.server.workflows import Workflow
 from debusine.server.workflows.models import WorkRequestManualUnblockAction
-from debusine.tasks.models import EventReactions, WorkerType
+from debusine.tasks.models import EventReactions, OutputData, WorkerType
+
+if TYPE_CHECKING:
+    SerializerMixinBase: TypeAlias = serializers.Serializer[Any]
+    SlugRelatedFieldBase = serializers.SlugRelatedField
+else:
+    SerializerMixinBase = object
+
+    # SlugRelatedField doesn't support generic types at run-time yet.
+    class _SlugRelatedFieldBase:
+        def __class_getitem__(*args):
+            return serializers.SlugRelatedField
+
+    SlugRelatedFieldBase = _SlugRelatedFieldBase
 
 
-class WorkRequestSerializer(serializers.ModelSerializer[WorkRequest]):
+class DebusineSerializerMixin(SerializerMixinBase):
+    """Adjust a serializer with debusine-specific behaviour."""
+
+    validation_error_title: str
+
+    def is_valid(self, *, raise_exception: bool = False) -> bool:
+        """Raise a suitable API exception when deserialization fails."""
+        valid = super().is_valid()
+        if raise_exception and not valid:
+            if (workspace_errors := self.errors.get("workspace", [])) and all(
+                isinstance(error, ErrorDetail)
+                and error.code == "does_not_exist"
+                for error in workspace_errors
+            ):
+                raise_workspace_not_found(self.data["workspace"])
+            else:
+                raise DebusineAPIException(
+                    title=self.validation_error_title,
+                    validation_errors=self.errors,
+                )
+        return valid
+
+
+class WorkspaceInCurrentScopeField(SlugRelatedFieldBase[Workspace]):
+    """A field for workspaces in the current scope."""
+
+    def __init__(self, **kwargs: Any) -> None:
+        """Construct the field."""
+        kwargs.setdefault("default", self._get_default_workspace)
+        super().__init__(**kwargs)
+
+    def get_queryset(self) -> QuerySet[Workspace]:
+        """Limit to the current scope."""
+        return Workspace.objects.all().in_current_scope()
+
+    def _get_default_workspace(self) -> Workspace:
+        """Get the default workspace."""
+        try:
+            return self.get_queryset().get(
+                name=settings.DEBUSINE_DEFAULT_WORKSPACE
+            )
+        except Workspace.DoesNotExist:
+            # The default workspace name might not exist in the current
+            # scope.  If so, the client must provide a workspace.
+            self.fail("required")
+
+
+class WorkRequestSerializer(
+    DebusineSerializerMixin, serializers.ModelSerializer[WorkRequest]
+):
     """Serializer of a WorkRequest."""
 
+    validation_error_title = "Cannot deserialize work request"
+
     artifacts = serializers.SerializerMethodField("artifacts_for_work_request")
-    workspace = serializers.SlugRelatedField(
-        queryset=Workspace.objects.all(), slug_field="name"
-    )
+    workspace = WorkspaceInCurrentScopeField(slug_field="name")
     workflow_data = serializers.JSONField(
         source="workflow_data_json", required=False
     )
@@ -57,7 +130,7 @@ class WorkRequestSerializer(serializers.ModelSerializer[WorkRequest]):
         source='event_reactions_json', required=False
     )
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         """
         Initialize object.
 
@@ -77,7 +150,7 @@ class WorkRequestSerializer(serializers.ModelSerializer[WorkRequest]):
             .values_list("id", flat=True)
         )
 
-    def validate(self, data):
+    def validate(self, data: Any) -> Any:
         """
         Add extra validation in the WorkRequestSerializer. Used by .is_valid().
 
@@ -89,9 +162,8 @@ class WorkRequestSerializer(serializers.ModelSerializer[WorkRequest]):
             wanted_fields = set(self._only_fields)
 
             if unwanted_fields := (fields_in_data - wanted_fields):
-                unwanted_fields = ', '.join(sorted(unwanted_fields))
                 raise serializers.ValidationError(
-                    f'Invalid fields: {unwanted_fields}'
+                    f"Invalid fields: {', '.join(sorted(unwanted_fields))}"
                 )
 
         return data
@@ -129,28 +201,58 @@ class WorkRequestSerializer(serializers.ModelSerializer[WorkRequest]):
         ]
 
 
+class WorkRequestSerializerWithConfiguredTaskData(WorkRequestSerializer):
+    """Specialization of WorkRequestSerializer to feed task data to workers."""
+
+    def update(self, instance: Any, validated_data: dict[str, Any]) -> NoReturn:
+        """Disable update."""
+        raise NotImplementedError('`update()` must be implemented.')
+
+    def create(self, validated_data: dict[str, Any]) -> NoReturn:
+        """Disable create."""
+        raise NotImplementedError('`create()` must be implemented.')
+
+    def to_representation(self, instance: WorkRequest) -> dict[str, Any]:
+        """
+        Use configured_task_data instead of task_data.
+
+        This is intended to feed configured_task_data to workers without
+        breaking the worker API.
+        """
+        res = super().to_representation(instance)
+        if instance.configured_task_data:
+            res["task_data"] = instance.configured_task_data
+        return res
+
+
 class WorkRequestUpdateSerializer(WorkRequestSerializer):
     """Serializer for updating work requests."""
 
+    validation_error_title = "Cannot deserialize work request update"
+
     class Meta:
         model = WorkRequest
         fields = ['priority_adjustment']
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         """Ensure that only expected fields are passed."""
         kwargs["only_fields"] = self.Meta.fields
         super().__init__(*args, **kwargs)
 
 
-class WorkRequestUnblockSerializer(serializers.Serializer[Never]):
+class WorkRequestUnblockSerializer(
+    DebusineSerializerMixin, serializers.Serializer[Never]
+):
     """Serializer for the data used to edit a manual unblock."""
 
+    validation_error_title = "Cannot deserialize work request unblock"
+
     notes = serializers.CharField(max_length=65536, required=False)
     action = serializers.ChoiceField(
         choices=WorkRequestManualUnblockAction, required=False
     )
 
-    def validate(self, data):
+    def validate(self, data: Any) -> Any:
         """Ensure at least one of notes and action is present."""
         if not data.get("notes") and data.get("action") is None:
             raise serializers.ValidationError(
@@ -179,7 +281,11 @@ class WorkRequestExternalDebsignSerializer(WorkRequestSerializer):
                 TaskDatabase(instance)
             )
             data["dynamic_task_data"] = (
-                None if dynamic_task_data is None else dynamic_task_data.dict()
+                None
+                if dynamic_task_data is None
+                else model_to_json_serializable_dict(
+                    dynamic_task_data, exclude_unset=True
+                )
             )
         except Exception as exc:
             raise ValidationError(detail=f"Cannot process task data: {exc}")
@@ -188,10 +294,12 @@ class WorkRequestExternalDebsignSerializer(WorkRequestSerializer):
 
 
 class WorkRequestExternalDebsignRequestSerializer(
-    serializers.Serializer[Never]
+    DebusineSerializerMixin, serializers.Serializer[Never]
 ):
     """Serializer for the data used to complete an `ExternalDebsign` task."""
 
+    validation_error_title = "Cannot deserialize work request"
+
     signed_artifact = serializers.PrimaryKeyRelatedField(
         queryset=Artifact.objects
     )
@@ -205,6 +313,13 @@ class WorkRequestExternalDebsignRequestSerializer(
                     f"{ArtifactCategory.UPLOAD}; got {value.category}"
                 )
             )
+        if value.created_by_work_request is not None:
+            raise ValidationError(
+                detail=(
+                    "Signed artifact must not have been created by a work "
+                    "request"
+                )
+            )
         return value
 
 
@@ -228,10 +343,19 @@ class WorkRequestCompletedSerializer(serializers.ModelSerializer[WorkRequest]):
             if choice != WorkRequest.Results.NONE
         ]
     )
+    output_data = serializers.JSONField(required=False)
+
+    def validate_output_data(self, value: Any) -> Any:
+        """Validate output_data."""
+        try:
+            OutputData.parse_obj(value)
+        except pydantic.ValidationError as e:
+            raise serializers.ValidationError(f"Invalid output_data: {e}")
+        return value
 
     class Meta:
         model = WorkRequest
-        fields = ['result']
+        fields = ["result", "output_data"]
 
 
 class OnWorkRequestCompleted(serializers.Serializer[Never]):
@@ -260,11 +384,15 @@ class FileSerializerResponse(FileSerializer):
     url = serializers.URLField()
 
 
-class ArtifactSerializer(serializers.Serializer[Artifact]):
+class ArtifactSerializer(
+    DebusineSerializerMixin, serializers.Serializer[Artifact]
+):
     """Serializer to deserialize the client request to create an artifact."""
 
+    validation_error_title = "Cannot deserialize artifact"
+
     category = serializers.CharField(max_length=255)
-    workspace = serializers.CharField(max_length=255)
+    workspace = WorkspaceInCurrentScopeField(slug_field="name")
     files = serializers.DictField(child=FileSerializer())
     # Deliberately incompatible with Serializer.data.
     data = serializers.DictField()  # type: ignore[assignment]
@@ -322,9 +450,7 @@ class ArtifactSerializerResponse(serializers.ModelSerializer[Artifact]):
         for file_in_artifact in artifact.fileinartifact_set.all().order_by(
             "path"
         ):
-            if not artifact.workspace.is_file_in_workspace(
-                file_in_artifact.file
-            ):
+            if artifact.workspace.file_needs_upload(file_in_artifact.file):
                 files_to_upload.append(file_in_artifact.path)
 
         return files_to_upload
@@ -356,37 +482,36 @@ class ArtifactSerializerResponse(serializers.ModelSerializer[Artifact]):
 
         return files
 
-    def _download_tar_gz_url(self, obj) -> str:  # noqa: U100
+    def _download_tar_gz_url(self, obj: Artifact) -> str:  # noqa: U100
         """Based on self._base_download_url return URL to download .tar.gz."""
         return self._base_download_url + "?archive=tar.gz"
 
     @staticmethod
-    def _build_absolute_download_url(artifact_id: int, request: Request) -> str:
+    def _build_absolute_download_url(
+        artifact: Artifact, request: HttpRequest
+    ) -> str:
         """Return URL to download the artifact (without the archive format)."""
-        return request.build_absolute_uri(
-            reverse(
-                "artifacts:download",
-                kwargs={"artifact_id": artifact_id},
-            )
-        )
+        return request.build_absolute_uri(artifact.get_absolute_url_download())
 
     @classmethod
     def from_artifact(
-        cls, artifact: Artifact, request: Request
+        cls, artifact: Artifact, request: HttpRequest
     ) -> "ArtifactSerializerResponse":
         """Given an artifact and request return ArtifactSerializerResponse."""
         serialized = ArtifactSerializerResponse(artifact)
         serialized._base_download_url = cls._build_absolute_download_url(
-            artifact.id, request
+            artifact, request
         )
         return serialized
 
 
 class ArtifactRelationResponseSerializer(
-    serializers.ModelSerializer[ArtifactRelation]
+    DebusineSerializerMixin, serializers.ModelSerializer[ArtifactRelation]
 ):
     """Serializer for relations between artifacts."""
 
+    validation_error_title = "Cannot deserialize artifact relation"
+
     media_type = "application/json"
 
     class Meta:
@@ -400,6 +525,74 @@ class ArtifactRelationResponseSerializer(
         validators: list[Callable[[ArtifactRelation], None]] = []
 
 
+class AssetSerializer(DebusineSerializerMixin, serializers.Serializer[Asset]):
+    """Serializer to deserialize the client request to create an asset."""
+
+    validation_error_title = "Cannot deserialize asset"
+
+    id = serializers.IntegerField(required=False, read_only=True)
+    created_at = serializers.DateTimeField(read_only=True)
+    category = serializers.CharField(max_length=255)
+    workspace = WorkspaceInCurrentScopeField(slug_field="name")
+    # Deliberately incompatible with Serializer.data.
+    data = serializers.DictField()  # type: ignore[assignment]
+    work_request = serializers.IntegerField(
+        allow_null=True, required=False, source="created_by_work_request_id"
+    )
+
+    def validate(self, data: dict[str, Any]) -> dict[str, Any]:
+        """Validate category and data."""
+        try:
+            # Python 3.11 doesn't support X in StrEnum
+            AssetCategory(data["category"])
+        except ValueError:
+            raise serializers.ValidationError(
+                {"category": f"{data['category']} is not a known category."}
+            )
+
+        try:
+            asset_data_model(data["category"], data["data"])
+        except ValueError as e:
+            raise serializers.ValidationError(
+                {"data": f"invalid asset data: {e}"}
+            ) from e
+        return data
+
+    def create(self, validated_data: dict[str, Any]) -> Asset:
+        """Create an Asset model from deserialized data."""
+        data = copy(validated_data)
+        if not data.get("created_by_work_request_id"):
+            data["created_by"] = context.require_user()
+        return Asset.objects.create(**data)
+
+
+class AssetPermissionCheckRequest(
+    DebusineSerializerMixin, serializers.Serializer[Never]
+):
+    """Serializer for a permission check on an asset."""
+
+    validation_error_title = "Cannot deserialize permission check request"
+
+    artifact_id = serializers.IntegerField()
+    work_request_id = serializers.IntegerField()
+    workspace = serializers.CharField()
+
+
+class AssetPermissionCheckResponse(
+    DebusineSerializerMixin, serializers.Serializer[Never]
+):
+    """Serializer for a permission check result."""
+
+    validation_error_title = "Cannot serialize permission check result"
+
+    has_permission = serializers.BooleanField()
+    username = serializers.CharField(max_length=150, required=False)
+    user_id = serializers.IntegerField(required=False)
+    resource = serializers.DictField(
+        child=serializers.CharField(max_length=255), required=False
+    )
+
+
 class LookupExpectTypeChoiceField(serializers.ChoiceField):
     """
     A field for `Lookup*.expect_type`.
@@ -410,7 +603,7 @@ class LookupExpectTypeChoiceField(serializers.ChoiceField):
 
     def __init__(self, **kwargs: Any) -> None:
         """Initialize the field."""
-        return super().__init__(choices=LookupChildType)
+        super().__init__(choices=LookupChildType)
 
     def to_internal_value(self, data: Any) -> str:
         """Accept values of `CollectionItemTypes` for compatibility."""
@@ -466,16 +659,16 @@ class LookupResponseSerializer(serializers.Serializer[LookupResult]):
     )
 
 
-class WorkflowTemplateSerializer(serializers.ModelSerializer[WorkflowTemplate]):
+class WorkflowTemplateSerializer(
+    DebusineSerializerMixin, serializers.ModelSerializer[WorkflowTemplate]
+):
     """Serializer for workflow templates."""
 
+    validation_error_title = "Cannot deserialize workflow template"
+
     media_type = "application/json"
 
-    workspace = serializers.SlugRelatedField(
-        queryset=Workspace.objects.all(),
-        slug_field="name",
-        default=default_workspace,
-    )
+    workspace = WorkspaceInCurrentScopeField(slug_field="name")
 
     class Meta:
         model = WorkflowTemplate
@@ -488,7 +681,7 @@ class WorkflowTemplateSerializer(serializers.ModelSerializer[WorkflowTemplate]):
             "priority",
         ]
 
-    def validate_task_name(self, value):
+    def validate_task_name(self, value: str) -> str:
         """
         Ensure that task_name is valid.
 
@@ -503,17 +696,17 @@ class WorkflowTemplateSerializer(serializers.ModelSerializer[WorkflowTemplate]):
         return value
 
 
-class CreateWorkflowRequestSerializer(serializers.ModelSerializer[WorkRequest]):
+class CreateWorkflowRequestSerializer(
+    DebusineSerializerMixin, serializers.ModelSerializer[WorkRequest]
+):
     """Serializer for requests to create workflows."""
 
+    validation_error_title = "Cannot deserialize workflow creation request"
+
     media_type = "application/json"
 
     template_name = serializers.CharField(max_length=255)
-    workspace = serializers.SlugRelatedField(
-        queryset=Workspace.objects.all(),
-        slug_field="name",
-        default=default_workspace,
-    )
+    workspace = WorkspaceInCurrentScopeField(slug_field="name")
 
     class Meta:
         model = WorkRequest
