@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -24,9 +24,11 @@ from typing import Generic, TypeVar
 
 from debian.deb822 import Deb822, Packages, Sources
 from django.db import transaction
+from django_pglocks import advisory_lock
 
 from debusine.artifacts.local_artifact import BinaryPackage, SourcePackage
 from debusine.artifacts.models import ArtifactCategory, CollectionCategory
+from debusine.db.locks import LockError, LockType
 from debusine.db.models import (
     Artifact,
     ArtifactRelation,
@@ -36,7 +38,7 @@ from debusine.db.models import (
 from debusine.server.collections.debian_suite import DebianSuiteManager
 from debusine.server.tasks import BaseServerTask
 from debusine.server.tasks.models import APTMirrorData
-from debusine.tasks import TaskConfigError
+from debusine.tasks import DefaultDynamicData, TaskConfigError
 from debusine.tasks.models import BaseDynamicTaskData
 
 T = TypeVar("T", bound=Deb822)
@@ -89,7 +91,10 @@ def _run_and_log_errors(
         raise
 
 
-class APTMirror(BaseServerTask[APTMirrorData, BaseDynamicTaskData]):
+class APTMirror(
+    BaseServerTask[APTMirrorData, BaseDynamicTaskData],
+    DefaultDynamicData[APTMirrorData],
+):
     """Task that mirrors an external APT suite into a debusine collection."""
 
     TASK_VERSION = 1
@@ -311,7 +316,7 @@ class APTMirror(BaseServerTask[APTMirrorData, BaseDynamicTaskData]):
         name: str,
         binary: Packages,
         component: str,
-    ):
+    ) -> None:
         """Download a binary package and add it to the collection."""
         assert self.work_request is not None
         assert self.workspace is not None
@@ -482,18 +487,36 @@ class APTMirror(BaseServerTask[APTMirrorData, BaseDynamicTaskData]):
 
     def _execute(self) -> bool:
         """Execute the task."""
-        # This task may take a long time, so it commits transactions as it
-        # goes rather than taking a single long transaction.  If processing
-        # an item fails, then the results of earlier processing will remain
-        # visible.
-        with tempfile.TemporaryDirectory(
-            prefix="debusine-aptmirror-"
-        ) as temp_dir:
-            temp_path = Path(temp_dir)
-            self.fetch_indexes(temp_path)
-            self.update_sources(temp_path, self.plan_sources(temp_path))
-            self.update_binaries(temp_path, self.plan_binaries(temp_path))
-        return True
+        with advisory_lock(
+            (
+                LockType.APT_MIRROR,
+                # Only use the bottom 31 bits, in order that this fits into
+                # PostgreSQL's int type.  In the unlikely event that we have
+                # enough mirrored collections for there to be a collision,
+                # then it just means that the colliding collections can't be
+                # mirrored simultaneously.
+                self.collection.id & (2**31 - 1),
+            ),
+            wait=False,
+        ) as acquired:
+            if not acquired:
+                raise LockError(
+                    f"Another APTMirror task for {self.data.collection} is "
+                    f"already running"
+                )
+
+            # This task may take a long time, so it commits transactions as
+            # it goes rather than taking a single long transaction.  If
+            # processing an item fails, then the results of earlier
+            # processing will remain visible.
+            with tempfile.TemporaryDirectory(
+                prefix="debusine-aptmirror-"
+            ) as temp_dir:
+                temp_path = Path(temp_dir)
+                self.fetch_indexes(temp_path)
+                self.update_sources(temp_path, self.plan_sources(temp_path))
+                self.update_binaries(temp_path, self.plan_binaries(temp_path))
+            return True
 
     def get_label(self) -> str:
         """Return the task label."""
