@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -10,16 +10,22 @@
 """Base support for tasks that run on server-side Celery workers."""
 
 from abc import ABCMeta
+from collections.abc import Callable
 from pathlib import Path
 from typing import TypeVar
 
 from debusine.artifacts import WorkRequestDebugLogs
+from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
     ArtifactRelation,
     WorkRequest,
     Workspace,
 )
+from debusine.db.models.permissions import (
+    PermissionUser,
+    format_permission_check_error,
+)
 from debusine.tasks import BaseTask
 from debusine.tasks.models import BaseDynamicTaskData, BaseTaskData, TaskTypes
 from debusine.worker.system_information import host_architecture
@@ -28,6 +34,10 @@ TD = TypeVar("TD", bound=BaseTaskData)
 DTD = TypeVar("DTD", bound=BaseDynamicTaskData)
 
 
+class ServerTaskPermissionDenied(Exception):
+    """Permission predicate checks failed on server tasks."""
+
+
 class BaseServerTask(BaseTask[TD, DTD], metaclass=ABCMeta):
     """Base class for tasks that run on server-side Celery workers."""
 
@@ -53,7 +63,21 @@ class BaseServerTask(BaseTask[TD, DTD], metaclass=ABCMeta):
         self.workspace_name = work_request.workspace.name
         self.worker_host_architecture = host_architecture()
 
-    def _upload_work_request_debug_logs(self):
+    def execute(self) -> bool:
+        """Execute task, setting up a suitable context."""
+        assert self.work_request is not None
+        assert self.workspace is not None
+
+        context.reset()
+        context.set_scope(self.workspace.scope)
+        context.set_user(self.work_request.created_by)
+        self.workspace.set_current()
+        try:
+            return super().execute()
+        finally:
+            context.reset()
+
+    def _upload_work_request_debug_logs(self) -> None:
         """
         Create a WorkRequestDebugLogs artifact and upload the logs.
 
@@ -70,6 +94,7 @@ class BaseServerTask(BaseTask[TD, DTD], metaclass=ABCMeta):
             files=Path(self._debug_log_files_directory.name).glob("*")
         )
 
+        assert self.workspace is not None
         artifact = Artifact.objects.create_from_local_artifact(
             work_request_debug_logs,
             self.workspace,
@@ -85,3 +110,14 @@ class BaseServerTask(BaseTask[TD, DTD], metaclass=ABCMeta):
 
         self._debug_log_files_directory.cleanup()
         self._debug_log_files_directory = None
+
+    def enforce(self, predicate: Callable[[PermissionUser], bool]) -> None:
+        """Enforce a permission predicate."""
+        assert self.work_request is not None
+
+        if predicate(self.work_request.created_by):
+            return
+
+        raise ServerTaskPermissionDenied(
+            format_permission_check_error(predicate, context.user)
+        )
