@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -13,6 +13,7 @@ import re
 from typing import ClassVar, cast
 
 from debusine.artifacts.models import ArtifactCategory, CollectionCategory
+from debusine.db.context import context
 from debusine.db.models import (
     Collection,
     CollectionItem,
@@ -30,11 +31,12 @@ from debusine.server.tasks.update_derived_collection import (
     DerivedCollectionChange,
     DerivedCollectionChangeKind,
 )
+from debusine.tasks.models import LintianData, LintianInput, LookupMultiple
 from debusine.test.django import TestCase
 
 
 class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
-    """Test the `updatesuitelintiancollection` task."""
+    """Test the `UpdateSuiteLintianCollection` task."""
 
     user: ClassVar[User]
     workspace: ClassVar[Workspace]
@@ -94,6 +96,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
         del self.task.derived_collection
         self.assertEqual(self.task.derived_collection, self.derived_collection)
 
+    @context.disable_permission_checks()
     def test_find_relevant_base_items(self) -> None:
         """Only active items in the correct base collection are relevant."""
         collection_items = [
@@ -117,6 +120,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
             ordered=False,
         )
 
+    @context.disable_permission_checks()
     def test_find_relevant_derived_items(self) -> None:
         """Only active items in the correct derived collection are relevant."""
         source_package_items = [
@@ -147,6 +151,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
             ordered=False,
         )
 
+    @context.disable_permission_checks()
     def test_find_expected_derived_items(self) -> None:
         """
         Derived item names are mapped to sets of base item IDs.
@@ -195,6 +200,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
             },
         )
 
+    @context.disable_permission_checks()
     def test_categorize_derived_items(self) -> None:
         """Items are categorized into add/replace/remove."""
         source_package_items = [
@@ -243,6 +249,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
             ],
         )
 
+    @context.disable_permission_checks()
     def test_make_child_work_request_source_only(self) -> None:
         """Make a child work request with no binary packages."""
         source_package_item = self.create_source_package_item(
@@ -264,7 +271,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
             {
                 "input": {
                     "source_artifact": source_package_item.artifact_id,
-                    "binary_artifacts": (),
+                    "binary_artifacts": [],
                 }
             },
         )
@@ -284,6 +291,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
             ],
         )
 
+    @context.disable_permission_checks()
     def test_make_child_work_request_with_binaries(self) -> None:
         """Make a child work request with binary packages."""
         source_package_item = self.create_source_package_item(
@@ -313,9 +321,9 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
             {
                 "input": {
                     "source_artifact": source_package_item.artifact_id,
-                    "binary_artifacts": tuple(
-                        [item.artifact_id for item in binary_package_items]
-                    ),
+                    "binary_artifacts": [
+                        item.artifact_id for item in binary_package_items
+                    ],
                 }
             },
         )
@@ -335,6 +343,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
             ],
         )
 
+    @context.disable_permission_checks()
     def test_make_child_work_request_with_only_binaries(self) -> None:
         """Make a child work request with binary packages but no sources."""
         binary_package_items = [
@@ -358,9 +367,9 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
             child_work_request.task_data,
             {
                 "input": {
-                    "binary_artifacts": tuple(
-                        [item.artifact_id for item in binary_package_items]
-                    )
+                    "binary_artifacts": [
+                        item.artifact_id for item in binary_package_items
+                    ]
                 }
             },
         )
@@ -394,6 +403,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
                 base_item_ids=set(), child_task_data=None, force=False
             )
 
+    @context.disable_permission_checks()
     def test_make_child_work_request_with_multiple_sources(self) -> None:
         """`make_child_work_request` raises ValueError: multiple sources."""
         source_package_items = [
@@ -417,6 +427,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
                 force=False,
             )
 
+    @context.disable_permission_checks()
     def test_make_child_work_request_extra_task_data(self) -> None:
         """A child work request can be supplied with extra task data."""
         source_package_item = self.create_source_package_item(
@@ -440,7 +451,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
             {
                 "input": {
                     "source_artifact": source_package_item.artifact_id,
-                    "binary_artifacts": (),
+                    "binary_artifacts": [],
                 },
                 "environment": (
                     "debian/match:codename=bookworm:architecture=amd64"
@@ -457,6 +468,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
             },
         )
 
+    @context.disable_permission_checks()
     def test_make_child_work_request_force(self) -> None:
         """`force=True` is needed if a matching work request already exists."""
         source_package_item = self.create_source_package_item(
@@ -466,12 +478,12 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
         self.task.set_work_request(work_request)
         self.playground.create_work_request(
             task_name="lintian",
-            task_data={
-                "input": {
-                    "source_artifact": source_package_item.artifact_id,
-                    "binary_artifacts": (),
-                }
-            },
+            task_data=LintianData(
+                input=LintianInput(
+                    source_artifact=source_package_item.artifact_id,
+                    binary_artifacts=LookupMultiple.parse_obj([]),
+                )
+            ),
             workspace=self.workspace,
             created_by=work_request.created_by,
             parent=work_request,
@@ -505,6 +517,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
             )
         )
 
+    @context.disable_permission_checks()
     def test_execute(self) -> None:
         """The task plans and executes the required changes."""
         source_package_items = [
