@@ -1,4 +1,4 @@
-# Copyright 2021-2022 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -11,7 +11,7 @@
 
 import asyncio
 from datetime import timedelta
-from typing import cast
+from typing import Any, cast
 from unittest import mock
 from unittest.mock import PropertyMock
 from urllib.parse import urlencode
@@ -19,36 +19,35 @@ from urllib.parse import urlencode
 from channels.db import database_sync_to_async
 from channels.routing import URLRouter
 from channels.testing import WebsocketCommunicator
+from django.conf import settings
 from django.utils import timezone
 from rest_framework.fields import DateTimeField
 
 from debusine.db.context import context
-from debusine.db.models import (
-    DEFAULT_WORKSPACE_NAME,
-    Token,
-    WorkRequest,
-    Worker,
-    Workspace,
-)
+from debusine.db.models import Token, WorkRequest, Worker, Workspace
 from debusine.server import consumers
 from debusine.server.consumers import WorkerConsumer
 from debusine.server.middlewares.token_last_seen_at import (
     TokenLastSeenAtMiddlewareChannels,
 )
 from debusine.server.routing import websocket_urlpatterns
-from debusine.test.django import TestCase, TransactionTestCase
-from debusine.test.utils import date_time_to_isoformat_rest_framework
+from debusine.test.django import (
+    BaseDjangoTestCase,
+    TestCase,
+    TransactionTestCase,
+)
+from debusine.test.test_utils import date_time_to_isoformat_rest_framework
 
 
 class WebSocketURLTests(TestCase):
     """Tests for websocket_urlpatterns."""
 
-    def test_websocket_urlpatterns(self):
+    def test_websocket_urlpatterns(self) -> None:
         """Ensure websocket_urlpatterns provides a valid URLRouter config."""
         URLRouter(websocket_urlpatterns)
 
 
-class WorkerConsumerMixin:
+class WorkerConsumerMixin(BaseDjangoTestCase):
     """Methods used by different WorkerConsumer tests classes."""
 
     CONNECTED_PAYLOAD = {
@@ -66,26 +65,26 @@ class WorkerConsumerMixin:
         "text": "work_request_available",
     }
 
-    @database_sync_to_async
-    def acreate_worker(self):
+    # mypy complains that database_sync_to_async is untyped, which is true,
+    # but we can't fix that here.
+    @database_sync_to_async  # type: ignore[misc]
+    def acreate_worker(self) -> Worker:
         """Async version to return a new Worker."""
         return self._create_worker()
 
-    def _create_worker(self):
+    def _create_worker(self) -> Worker:
         """Return a new Worker."""
-        token = self.playground.create_token_enabled()
-
-        worker = Worker.objects.create_with_fqdn("computer.lan", token)
-
+        worker = Worker.objects.create_with_fqdn(
+            "computer.lan",
+            token=cast(TestCase, self).playground.create_bare_token(),
+        )
         worker.set_dynamic_metadata(
             {
                 "cpu_cores": 4,
                 "sbuild:version": 1,
                 "sbuild:available": True,
-                "sbuild:chroots": "bullseye-amd64",
             }
         )
-
         return worker
 
     async def connect(
@@ -93,7 +92,7 @@ class WorkerConsumerMixin:
         *,
         token_key: str | None = None,
         wait_for_fully_connected: bool = True,
-    ):
+    ) -> WebsocketCommunicator:
         """Return WebsocketCommunicator to api/ws/1.0/worker/connect/."""
         headers = [(b'User-Agent', b'A user agent')]
 
@@ -109,17 +108,19 @@ class WorkerConsumerMixin:
         )
 
         connected, subprotocol = await communicator.connect()
-        cast(TestCase, self).assertTrue(connected)
+        self.assertTrue(connected)
 
         if wait_for_fully_connected:
-            cast(TestCase, self).assertEqual(
+            self.assertEqual(
                 await communicator.receive_json_from(timeout=3),
                 self.CONNECTED_PAYLOAD,
             )
 
         return communicator
 
-    async def connect_to_new_worker(self):
+    async def connect_to_new_worker(
+        self,
+    ) -> tuple[WebsocketCommunicator, Worker]:
         """Return a communicator connected to a new worker."""
         worker = await self.acreate_worker()
         communicator = await self.connect(token_key=worker.token.key)
@@ -133,7 +134,7 @@ class WorkerConsumerMixin:
 
         return communicator, worker
 
-    def assertTaskExists(self, task_name):  # pragma: no cover
+    def assertTaskExists(self, task_name: str) -> None:  # pragma: no cover
         """Fail if task_name is not in asyncio.all_tasks()."""
         for task in asyncio.all_tasks():
             if task.get_name() == task_name:
@@ -141,20 +142,24 @@ class WorkerConsumerMixin:
 
         self.fail(f"Asynchronous task '{task_name}' does not exist")
 
-    def assertTaskNotExists(self, task_name):  # pragma: no cover
+    def assertTaskNotExists(self, task_name: str) -> None:  # pragma: no cover
         """Fail if task_name is in asyncio.all_tasks()."""
         for task in asyncio.all_tasks():
             if task.get_name() == task_name:
                 self.fail(f"Asynchronous task '{task_name}' does exist")
 
-    async def assertRequestDynamicMetadata(self, communicator):
+    async def assertRequestDynamicMetadata(
+        self, communicator: WebsocketCommunicator
+    ) -> None:
         """Assert the next communicator msg is request for dynamic metadata."""
         self.assertEqual(
             await communicator.receive_json_from(),
             self.REQUEST_DYNAMIC_METADATA_PAYLOAD,
         )
 
-    def patch_workerconsumer_REQUEST_DYNAMIC_METADATA_SECONDS_fast(self):
+    def patch_workerconsumer_REQUEST_DYNAMIC_METADATA_SECONDS_fast(
+        self,
+    ) -> mock.MagicMock:
         """Patch WorkerConsumer.REQUEST_DYNAMIC_METADATA_SECONDS to be fast."""
         patcher = mock.patch(
             'debusine.server.consumers.'
@@ -173,7 +178,7 @@ class WorkerConsumerMixin:
         token_key: str | None,
         reason: str,
         reason_code: str | None = None,
-    ):
+    ) -> None:
         """Assert that the client is rejected with a reason."""
         communicator = await self.connect(
             token_key=token_key, wait_for_fully_connected=False
@@ -186,11 +191,9 @@ class WorkerConsumerMixin:
         if reason_code is not None:
             msg["reason_code"] = reason_code
 
-        cast(TestCase, self).assertEqual(
-            await communicator.receive_json_from(), msg
-        )
+        self.assertEqual(await communicator.receive_json_from(), msg)
 
-        cast(TestCase, self).assertEqual(
+        self.assertEqual(
             await communicator.receive_output(), {'type': 'websocket.close'}
         )
 
@@ -206,7 +209,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
     https://github.com/django/channels/issues/1091#issuecomment-701361358
     """
 
-    async def test_disconnect(self):
+    async def test_disconnect(self) -> None:
         """Disconnect marks the Worker as disconnected."""
         communicator, worker = await self.connect_to_new_worker()
 
@@ -216,7 +219,9 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
 
         self.assertFalse(worker.connected())
 
-    async def assert_messages_send_on_connect(self, token_key, expected_msgs):
+    async def assert_messages_send_on_connect(
+        self, token_key: str, expected_msgs: list[dict[str, str]]
+    ) -> None:
         """
         Connect authenticating with token_key and expects expected_msgs.
 
@@ -238,9 +243,15 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
         for message in expected_msgs:
             self.assertIn(message, received_messages)
 
-        await communicator.disconnect()
+        try:
+            await communicator.disconnect()
+        except asyncio.exceptions.CancelledError:  # pragma: no cover
+            # asgiref < 3.9.0 swallowed this exception; asgiref 3.9.0
+            # re-raises it.  See
+            # https://github.com/django/asgiref/issues/518.
+            pass
 
-    async def test_connect_valid_token(self):
+    async def test_connect_valid_token(self) -> None:
         """Connect succeeds and a request for dynamic metadata is received."""
         worker = await self.acreate_worker()
 
@@ -248,7 +259,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
             worker.token.key, [self.REQUEST_DYNAMIC_METADATA_PAYLOAD]
         )
 
-    async def test_connected_send_work_request_available_running(self):
+    async def test_connected_send_work_request_available_running(self) -> None:
         """Connect succeeds and consumer send expected two messages."""
         worker = await self.acreate_worker()
 
@@ -265,7 +276,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
             ],
         )
 
-    async def test_connected_send_work_request_available_pending(self):
+    async def test_connected_send_work_request_available_pending(self) -> None:
         """Connect succeeds and consumer send expected two messages."""
         worker = await self.acreate_worker()
 
@@ -282,7 +293,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
             ],
         )
 
-    async def test_connected_token_without_worker(self):
+    async def test_connected_token_without_worker(self) -> None:
         """
         Connect succeeds and an error message is returned.
 
@@ -299,7 +310,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
             reason_code="TOKEN_DISABLED",
         )
 
-    async def test_connected_token_not_enabled(self):
+    async def test_connected_token_not_enabled(self) -> None:
         """
         Connect succeeds and an error message is returned.
 
@@ -314,7 +325,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
             reason_code="TOKEN_DISABLED",
         )
 
-    async def test_disconnect_cancel_request_dynamic_metadata(self):
+    async def test_disconnect_cancel_request_dynamic_metadata(self) -> None:
         """Disconnect cancels the dynamic metadata refresher."""
         self.assertTaskNotExists('dynamic_metadata_refresher')
 
@@ -333,7 +344,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
         with context.disable_permission_checks():
             work_request: WorkRequest = await database_sync_to_async(
                 self.playground.create_work_request
-            )(task_name="sbuild")
+            )()
 
         await database_sync_to_async(work_request.assign_worker)(worker)
         await database_sync_to_async(work_request.mark_running)()
@@ -388,7 +399,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
         )
         self.assertFalse(worker.connected())
 
-    async def test_request_dynamic_metadata_after_connect(self):
+    async def test_request_dynamic_metadata_after_connect(self) -> None:
         """Debusine sends request_dynamic_metadata_updated after connect."""
         worker = await self.acreate_worker()
 
@@ -412,7 +423,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
 
         await communicator.disconnect()
 
-    async def test_connect_redis_not_available(self):
+    async def test_connect_redis_not_available(self) -> None:
         """
         Connect succeeds and an error message is returned.
 
@@ -428,20 +439,27 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
         self.addCleanup(patcher.stop)
 
         token = await database_sync_to_async(
-            self.playground.create_token_enabled
+            self.playground.create_bare_token
         )()
-        await Worker.objects.acreate(registered_at=timezone.now(), token=token)
+        await Worker.objects.acreate(
+            registered_at=timezone.now(),
+            token=token,
+        )
 
         await self.assert_client_rejected(token.key, 'Service unavailable')
         mocked.assert_called()
 
-    async def test_worker_disabled(self):
+    async def test_worker_disabled(self) -> None:
         """Worker is disabled and msg disconnection msg sent to the worker."""
         communicator, worker = await self.connect_to_new_worker()
 
         await communicator.send_input({"type": "worker.disabled"})
 
-        hash_ = await database_sync_to_async(lambda: worker.token.hash)()
+        def get_token_hash() -> str:
+            assert worker.token is not None
+            return worker.token.hash
+
+        hash_ = await database_sync_to_async(get_token_hash)()
 
         self.assertEqual(
             await communicator.receive_json_from(),
@@ -454,14 +472,12 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
 
         await communicator.disconnect()
 
-    async def test_work_request_assigned(self):
+    async def test_work_request_assigned(self) -> None:
         """Assert it sends message to the worker."""
         communicator, worker = await self.connect_to_new_worker()
 
-        def assign_work_request(worker):
-            work_request = self.playground.create_work_request(
-                task_name="sbuild"
-            )
+        def assign_work_request(worker: Worker) -> None:
+            work_request = self.playground.create_work_request()
             work_request.assign_worker(worker)
 
         with context.disable_permission_checks():
@@ -474,7 +490,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
 
         await communicator.disconnect()
 
-    async def test_connect_invalid_token(self):
+    async def test_connect_invalid_token(self) -> None:
         """
         Connect succeeds and an error message is returned.
 
@@ -490,7 +506,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
 class WorkerConsumerTests(WorkerConsumerMixin, TestCase):
     """Tests for the WorkerConsumer class."""
 
-    async def test_connect_without_token(self):
+    async def test_connect_without_token(self) -> None:
         """
         Connect succeeds and an error message is returned.
 
@@ -502,7 +518,7 @@ class WorkerConsumerTests(WorkerConsumerMixin, TestCase):
             reason_code="TOKEN_DISABLED",
         )
 
-    def test_get_header_value_key_found(self):
+    def test_get_header_value_key_found(self) -> None:
         """WorkerConsumer.get_header_value return the key (str)."""
         worker = WorkerConsumer()
 
@@ -512,7 +528,7 @@ class WorkerConsumerTests(WorkerConsumerMixin, TestCase):
 
         self.assertEqual(worker.get_header_value('token'), token)
 
-    def test_get_header_value_key_not_found(self):
+    def test_get_header_value_key_not_found(self) -> None:
         """WorkerConsumer.get_header_value return None when key not found."""
         worker = WorkerConsumer()
 
@@ -520,7 +536,7 @@ class WorkerConsumerTests(WorkerConsumerMixin, TestCase):
 
         self.assertIsNone(worker.get_header_value('token'))
 
-    async def test_disconnect_while_not_connected(self):
+    async def test_disconnect_while_not_connected(self) -> None:
         """
         WorkerConsumer.disconnect returns without exceptions.
 
@@ -539,18 +555,18 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
         "text": "connected",
     }
 
-    def setUp(self):
+    def setUp(self) -> None:
         """Initialize test."""
         super().setUp()
-        self.token = self.playground.create_token_enabled(with_user=True)
+        self.token = self.playground.create_user_token()
 
     async def connect(
         self,
         *,
         token_key: str | None = None,
-        query_params: dict[str, str] = {},
+        query_params: dict[str, Any] = {},
         wait_for_fully_connected: bool = True,
-    ):
+    ) -> WebsocketCommunicator:
         """Connect to the WorkRequestCompleted endpoint."""
         headers = [(b"User-Agent", b"A user agent")]
 
@@ -578,7 +594,7 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
 
         return communicator
 
-    async def test_connect(self):
+    async def test_connect(self) -> None:
         """Connect to receive notifications that a Work Request completed."""
         communicator = await self.connect(token_key=self.token.key)
 
@@ -586,7 +602,7 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
 
         await communicator.disconnect()
 
-    async def test_disconnect(self):
+    async def test_disconnect(self) -> None:
         """Ensure we clean-up, cf. #225."""
         consumer = consumers.OnWorkRequestCompletedConsumer()
 
@@ -599,7 +615,7 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
             "work_request_completed", consumer.channel_name
         )
 
-    async def test_connect_rejected_no_token_header(self):
+    async def test_connect_rejected_no_token_header(self) -> None:
         """Connect is rejected: no token was provided."""
         communicator = await self.connect(wait_for_fully_connected=False)
 
@@ -614,7 +630,7 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
 
         await communicator.disconnect()
 
-    async def test_connect_rejected_workspace_not_found(self):
+    async def test_connect_rejected_workspace_not_found(self) -> None:
         """Connect is rejected: workspace is not found."""
         communicator = await self.connect(
             token_key=self.token.key,
@@ -632,7 +648,7 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
 
         await communicator.disconnect()
 
-    async def test_connect_receive_greater_than_last_modified(self):
+    async def test_connect_receive_greater_than_last_modified(self) -> None:
         """
         Consumer filter out WorkRequest that completed_at < completed_at_since.
 
@@ -651,7 +667,6 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
             work_request_notified = await database_sync_to_async(
                 self.playground.create_work_request
             )(
-                task_name="sbuild",
                 assign_new_worker=True,
                 mark_running=True,
                 result=result,
@@ -663,7 +678,6 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
         work_request_to_notify = await database_sync_to_async(
             self.playground.create_work_request
         )(
-            task_name="sbuild",
             assign_new_worker=True,
             mark_running=True,
             result=result,
@@ -693,7 +707,7 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
 
         await communicator.disconnect()
 
-    async def test_connect_receive_no_include_last_modified(self):
+    async def test_connect_receive_no_include_last_modified(self) -> None:
         """
         Consumer filter out WorkRequest that completed_at == completed_at_since.
 
@@ -706,7 +720,6 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
 
         with context.disable_permission_checks():
             await database_sync_to_async(self.playground.create_work_request)(
-                task_name="sbuild",
                 assign_new_worker=True,
                 mark_running=True,
                 result=result,
@@ -721,7 +734,7 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
 
         await communicator.disconnect()
 
-    async def test_connect_receive_include_last_modified(self):
+    async def test_connect_receive_include_last_modified(self) -> None:
         """
         Consumer return 2 WorkRequests with completed_at == completed_at_since.
 
@@ -741,7 +754,6 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
             work_request_last_created_1 = await database_sync_to_async(
                 self.playground.create_work_request
             )(
-                task_name="sbuild",
                 assign_new_worker=True,
                 mark_running=True,
                 result=result,
@@ -753,7 +765,6 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
         work_request_last_created_2 = await database_sync_to_async(
             self.playground.create_work_request
         )(
-            task_name="sbuild",
             assign_new_worker=True,
             mark_running=True,
             result=result,
@@ -797,28 +808,20 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
 
         await communicator.disconnect()
 
-    async def create_sbuild_work_request(self, workspace: Workspace):
-        """."""
-        return await database_sync_to_async(
+    async def create_work_request(self, workspace: Workspace) -> WorkRequest:
+        """Create a work request."""
+        work_request = await database_sync_to_async(
             self.playground.create_work_request
         )(
-            task_name="sbuild",
             workspace=workspace,
             assign_new_worker=True,
             mark_running=True,
             result=WorkRequest.Results.SUCCESS,
-            task_data={
-                "input": {
-                    "source_artifact": 5,
-                },
-                "distribution": "stable",
-                "backend": "schroot",
-                "host_architecture": "amd64",
-                "build_components": ["any", "all"],
-            },
         )
+        assert isinstance(work_request, WorkRequest)
+        return work_request
 
-    async def test_work_request_completed_selected_workspace(self):
+    async def test_work_request_completed_selected_workspace(self) -> None:
         """Notification is received: from a monitored workspace."""
         with context.disable_permission_checks():
             workspace_name = "lts"
@@ -836,7 +839,8 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
         # the work request
         self.assertTrue(await communicator.receive_nothing())
 
-        work_request = await self.create_sbuild_work_request(workspace)
+        work_request = await self.create_work_request(workspace)
+        assert work_request.completed_at is not None
 
         actual = await communicator.receive_json_from()
         self.assertEqual(
@@ -854,12 +858,12 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
 
         await communicator.disconnect()
 
-    async def test_work_request_completed_workspace_not_monitored(self):
+    async def test_work_request_completed_workspace_not_monitored(self) -> None:
         """Notification not received: from a non-monitored workspace."""
         # Need to exist when connecting
         with context.disable_permission_checks():
             await database_sync_to_async(self.playground.create_workspace)(
-                name=DEFAULT_WORKSPACE_NAME
+                name=settings.DEBUSINE_DEFAULT_WORKSPACE
             )
 
             workspace = await database_sync_to_async(
@@ -868,7 +872,7 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
 
         communicator = await self.connect(
             token_key=self.token.key,
-            query_params={"workspaces": DEFAULT_WORKSPACE_NAME},
+            query_params={"workspaces": settings.DEBUSINE_DEFAULT_WORKSPACE},
         )
 
         # Wait that the communicator is fully connected before creating
@@ -878,7 +882,6 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
         result = WorkRequest.Results.SUCCESS
 
         await database_sync_to_async(self.playground.create_work_request)(
-            task_name="sbuild",
             workspace=workspace,
             assign_new_worker=True,
             mark_running=True,
@@ -888,7 +891,7 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
 
         await communicator.disconnect()
 
-    async def test_work_request_completed_no_workspace_specified(self):
+    async def test_work_request_completed_no_workspace_specified(self) -> None:
         """Notification is received: all workspaces are monitored."""
         communicator = await self.connect(token_key=self.token.key)
 
@@ -899,7 +902,8 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
                 self.playground.create_workspace
             )()
 
-        work_request = await self.create_sbuild_work_request(workspace)
+        work_request = await self.create_work_request(workspace)
+        assert work_request.completed_at is not None
 
         actual = await communicator.receive_json_from()
         self.assertEqual(
