@@ -777,6 +777,11 @@ class WorkflowViewTests(TestCase):
 
     def test_create_workflow_success(self) -> None:
         """Create a workflow."""
+        self.playground.create_group_role(
+            self.scenario.workspace,
+            Workspace.Roles.CONTRIBUTOR,
+            [self.scenario.user],
+        )
         artifact = self.playground.create_source_artifact()
         for architecture in ("amd64", "arm64"):
             self.playground.create_debian_environment(
@@ -824,6 +829,11 @@ class WorkflowViewTests(TestCase):
         workspace = self.playground.create_workspace(
             name="test-workspace", public=True
         )
+        self.playground.create_group_role(
+            workspace,
+            Workspace.Roles.CONTRIBUTOR,
+            [self.scenario.user],
+        )
         artifact = self.playground.create_source_artifact(workspace=workspace)
         for architecture in ("amd64", "arm64"):
             self.playground.create_debian_environment(
@@ -856,9 +866,10 @@ class WorkflowViewTests(TestCase):
 
     def test_create_workflow_nonexistent_template(self) -> None:
         """The view returns HTTP 404 if the workflow template does not exist."""
-        response = self.post_workflow(
-            {"template_name": "test", "task_data": {}}
-        )
+        with self.assert_model_count_unchanged(WorkRequest):
+            response = self.post_workflow(
+                {"template_name": "test", "task_data": {}}
+            )
 
         self.assertResponseProblem(
             response,
@@ -868,11 +879,17 @@ class WorkflowViewTests(TestCase):
 
     def test_create_workflow_invalid_task_data(self) -> None:
         """Creating a workflow validates task data."""
+        self.playground.create_group_role(
+            self.scenario.workspace,
+            Workspace.Roles.CONTRIBUTOR,
+            [self.scenario.user],
+        )
         self.playground.create_workflow_template(name="noop", task_name="noop")
 
-        response = self.post_workflow(
-            {"template_name": "noop", "task_data": {"nonexistent": True}}
-        )
+        with self.assert_model_count_unchanged(WorkRequest):
+            response = self.post_workflow(
+                {"template_name": "noop", "task_data": {"nonexistent": True}}
+            )
 
         self.assertResponseProblem(
             response,
@@ -884,6 +901,11 @@ class WorkflowViewTests(TestCase):
 
     def test_create_workflow_thorough_input_validation(self) -> None:
         """Creating a workflow does thorough validation of input data."""
+        self.playground.create_group_role(
+            self.scenario.workspace,
+            Workspace.Roles.CONTRIBUTOR,
+            [self.scenario.user],
+        )
         self.playground.create_workflow_template(
             name="sbuild", task_name="sbuild"
         )
@@ -891,16 +913,17 @@ class WorkflowViewTests(TestCase):
             category=ArtifactCategory.TEST
         )
 
-        response = self.post_workflow(
-            {
-                "template_name": "sbuild",
-                "task_data": {
-                    "input": {"source_artifact": artifact.id},
-                    "target_distribution": "debian:bookworm",
-                    "architectures": ["amd64"],
-                },
-            }
-        )
+        with self.assert_model_count_unchanged(WorkRequest):
+            response = self.post_workflow(
+                {
+                    "template_name": "sbuild",
+                    "task_data": {
+                        "input": {"source_artifact": artifact.id},
+                        "target_distribution": "debian:bookworm",
+                        "architectures": ["amd64"],
+                    },
+                }
+            )
 
         self.assertResponseProblem(
             response,
@@ -949,6 +972,12 @@ class WorkflowViewTests(TestCase):
             (workspace1, ["amd64"]),
             (workspace2, ["i386"]),
         ):
+            self.playground.create_group_role(
+                workspace,
+                Workspace.Roles.CONTRIBUTOR,
+                [self.scenario.user],
+            )
+
             response = self.post_workflow(
                 {
                     "template_name": "test",
@@ -966,14 +995,15 @@ class WorkflowViewTests(TestCase):
             self.assertEqual(workflow.workspace, workspace)
             self.assertEqual(workflow.task_data["architectures"], architectures)
 
-        response = self.post_workflow(
-            {
-                "template_name": "test",
-                "workspace": "common-name",
-                "task_data": task_data,
-            },
-            scope=scope3.name,
-        )
+        with self.assert_model_count_unchanged(WorkRequest):
+            response = self.post_workflow(
+                {
+                    "template_name": "test",
+                    "workspace": "common-name",
+                    "task_data": task_data,
+                },
+                scope=scope3.name,
+            )
         self.assertResponseProblem(
             response,
             "Workspace not found",
@@ -985,9 +1015,10 @@ class WorkflowViewTests(TestCase):
         """POST with no workspace in a scope without a default workspace."""
         scope = self.playground.get_or_create_scope("empty-scope")
 
-        response = self.post_workflow(
-            {"template_name": "test"}, scope=scope.name
-        )
+        with self.assert_model_count_unchanged(WorkRequest):
+            response = self.post_workflow(
+                {"template_name": "test"}, scope=scope.name
+            )
 
         self.assertResponseProblem(
             response,
@@ -1004,10 +1035,11 @@ class WorkflowViewTests(TestCase):
             name="test", task_name="noop", workspace=private_workspace
         )
 
-        response = self.post_workflow(
-            {"template_name": "test", "workspace": private_workspace.name},
-            scope=private_workspace.scope.name,
-        )
+        with self.assert_model_count_unchanged(WorkRequest):
+            response = self.post_workflow(
+                {"template_name": "test", "workspace": private_workspace.name},
+                scope=private_workspace.scope.name,
+            )
 
         self.assertResponseProblem(
             response,
