@@ -18,6 +18,7 @@ except ImportError:
 
 from debusine.assets import KeyPurpose
 from debusine.client.models import LookupChildType
+from debusine.db.models import WorkRequest
 from debusine.server.collections.lookup import lookup_single
 from debusine.server.tasks.models import PackageUploadTarget
 from debusine.server.workflows import (
@@ -53,6 +54,20 @@ class DebianPipelineWorkflow(
 
     TASK_NAME = "debian_pipeline"
 
+    def validate_input(self) -> None:
+        """Thorough validation of input data."""
+        if self.data.reverse_dependencies_autopkgtest_suite is not None:
+            try:
+                lookup_single(
+                    self.data.reverse_dependencies_autopkgtest_suite,
+                    self.workspace,
+                    user=self.work_request.created_by,
+                    workflow_root=self.work_request.get_workflow_root(),
+                    expect_type=LookupChildType.COLLECTION,
+                ).collection
+            except LookupError as e:
+                raise WorkflowValidationError(str(e)) from e
+
     def build_dynamic_data(
         self, task_database: TaskDatabaseInterface  # noqa: U100
     ) -> BaseDynamicTaskData:
@@ -108,6 +123,8 @@ class DebianPipelineWorkflow(
             }
         )
 
+        to_upload = [(self.data.source_artifact, binary_artifacts, False)]
+
         if (
             self.data.enable_autopkgtest
             or self.data.enable_reverse_dependencies_autopkgtest
@@ -134,6 +151,7 @@ class DebianPipelineWorkflow(
                 lintian_fail_on_severity=self.data.lintian_fail_on_severity,
                 enable_piuparts=self.data.enable_piuparts,
                 piuparts_backend=self.data.piuparts_backend,
+                piuparts_environment=self.data.piuparts_environment,
             )
 
         if (
@@ -152,7 +170,10 @@ class DebianPipelineWorkflow(
                     'signing the source'
                 )
 
-            self._populate_make_signed_source(
+            for (
+                upload_source_artifact,
+                upload_binary_artifacts,
+            ) in self._populate_make_signed_source(
                 binary_artifacts=binary_artifacts,
                 vendor=self.data.vendor,
                 codename=self.data.codename,
@@ -161,23 +182,32 @@ class DebianPipelineWorkflow(
                 key=self.data.make_signed_source_key,
                 sbuild_backend=self.data.sbuild_backend,
                 signing_template_names=self.data.signing_template_names,
-            )
+            ):
+                to_upload.append(
+                    (upload_source_artifact, upload_binary_artifacts, True)
+                )
 
         if self.data.enable_upload:
-            self._populate_package_upload(
-                source_artifact=self.data.source_artifact,
-                binary_artifacts=binary_artifacts,
-                include_source=self.data.upload_include_source,
-                include_binaries=self.data.upload_include_binaries,
-                merge_uploads=self.data.upload_merge_uploads,
-                since_version=self.data.upload_since_version,
-                key=self.data.upload_key,
-                require_signature=self.data.upload_require_signature,
-                target=self.data.upload_target,
-                delayed_days=self.data.upload_delayed_days,
-                vendor=self.data.vendor,
-                codename=self.data.codename,
-            )
+            for (
+                upload_source_artifact,
+                upload_binary_artifacts,
+                signed_source,
+            ) in to_upload:
+                self._populate_package_upload(
+                    source_artifact=upload_source_artifact,
+                    binary_artifacts=upload_binary_artifacts,
+                    include_source=self.data.upload_include_source,
+                    include_binaries=self.data.upload_include_binaries,
+                    signed_source=signed_source,
+                    merge_uploads=self.data.upload_merge_uploads,
+                    since_version=self.data.upload_since_version,
+                    key=self.data.upload_key,
+                    require_signature=self.data.upload_require_signature,
+                    target=self.data.upload_target,
+                    delayed_days=self.data.upload_delayed_days,
+                    vendor=self.data.vendor,
+                    codename=self.data.codename,
+                )
 
     def _populate_sbuild(
         self,
@@ -229,33 +259,37 @@ class DebianPipelineWorkflow(
         lintian_fail_on_severity: LintianFailOnSeverity,
         enable_piuparts: bool,
         piuparts_backend: BackendType,
+        piuparts_environment: LookupSingle | None,
     ) -> None:
         """Create work request for qa workflow."""
+        data = QAWorkflowData(
+            source_artifact=source_artifact,
+            binary_artifacts=binary_artifacts,
+            vendor=vendor,
+            codename=codename,
+            architectures=architectures,
+            arch_all_host_architecture=arch_all_host_architecture,
+            extra_repositories=self.data.extra_repositories,
+            enable_autopkgtest=enable_autopkgtest,
+            autopkgtest_backend=autopkgtest_backend,
+            enable_reverse_dependencies_autopkgtest=(
+                enable_reverse_dependencies_autopkgtest
+            ),
+            reverse_dependencies_autopkgtest_suite=(
+                reverse_dependencies_autopkgtest_suite
+            ),
+            enable_lintian=enable_lintian,
+            lintian_backend=lintian_backend,
+            lintian_fail_on_severity=lintian_fail_on_severity,
+            enable_piuparts=enable_piuparts,
+            piuparts_backend=piuparts_backend,
+        )
+        if piuparts_environment:
+            data.piuparts_environment = piuparts_environment
         wr = self.work_request_ensure_child(
             task_name="qa",
             task_type=TaskTypes.WORKFLOW,
-            task_data=QAWorkflowData(
-                source_artifact=source_artifact,
-                binary_artifacts=binary_artifacts,
-                vendor=vendor,
-                codename=codename,
-                architectures=architectures,
-                arch_all_host_architecture=arch_all_host_architecture,
-                extra_repositories=self.data.extra_repositories,
-                enable_autopkgtest=enable_autopkgtest,
-                autopkgtest_backend=autopkgtest_backend,
-                enable_reverse_dependencies_autopkgtest=(
-                    enable_reverse_dependencies_autopkgtest
-                ),
-                reverse_dependencies_autopkgtest_suite=(
-                    reverse_dependencies_autopkgtest_suite
-                ),
-                enable_lintian=enable_lintian,
-                lintian_backend=lintian_backend,
-                lintian_fail_on_severity=lintian_fail_on_severity,
-                enable_piuparts=enable_piuparts,
-                piuparts_backend=piuparts_backend,
-            ),
+            task_data=data,
             workflow_data=WorkRequestWorkflowData(
                 display_name="QA",
                 step="qa",
@@ -278,9 +312,10 @@ class DebianPipelineWorkflow(
         key: str,
         sbuild_backend: BackendType,
         signing_template_names: dict[str, list[str]],
-    ) -> None:
+    ) -> list[tuple[LookupSingle, LookupMultiple]]:
         """Create work request for make signed source workflow."""
         signing_artifacts: list[LookupSingle] = []
+        to_upload: list[tuple[LookupSingle, LookupMultiple]] = []
         for arch, names in signing_template_names.items():
             for name in names:
                 # The promises for the signing_artifacts are created
@@ -297,6 +332,23 @@ class DebianPipelineWorkflow(
                 )
 
                 signing_artifacts.append(signing_artifact)
+                to_upload.append(
+                    (
+                        f"internal@collections/name:"
+                        f"signed-source-{arch}-{name}",
+                        LookupMultiple.parse_obj(
+                            {
+                                "collection": "internal@collections",
+                                "child_type": (
+                                    LookupChildType.ARTIFACT_OR_PROMISE
+                                ),
+                                "name__startswith": (
+                                    f"signed-source-{arch}-{name}|build-"
+                                ),
+                            }
+                        ),
+                    )
+                )
 
         signing_template_artifacts = LookupMultiple.parse_obj(signing_artifacts)
         wr = self.work_request_ensure_child(
@@ -323,13 +375,16 @@ class DebianPipelineWorkflow(
         wr.mark_running()
         orchestrate_workflow(wr)
 
+        return to_upload
+
     def _populate_package_upload(
         self,
         *,
         binary_artifacts: LookupMultiple,
-        source_artifact: LookupSingle | None,
+        source_artifact: LookupSingle,
         include_source: bool,
         include_binaries: bool,
+        signed_source: bool,
         merge_uploads: bool,
         since_version: str | None,
         key: str | None,
@@ -340,20 +395,18 @@ class DebianPipelineWorkflow(
         codename: str,
     ) -> None:
         """Create work request for package upload workflow."""
-        if include_source:
-            source_artifact = source_artifact
-        else:
-            source_artifact = None
-
-        if not include_binaries:
-            binary_artifacts = LookupMultiple.parse_obj(())
-
         wr = self.work_request_ensure_child(
             task_name="package_upload",
             task_type=TaskTypes.WORKFLOW,
             task_data=PackageUploadWorkflowData(
-                source_artifact=source_artifact,
-                binary_artifacts=binary_artifacts,
+                source_artifact=(
+                    source_artifact if include_source or signed_source else None
+                ),
+                binary_artifacts=(
+                    binary_artifacts
+                    if include_binaries
+                    else LookupMultiple.parse_obj(())
+                ),
                 merge_uploads=merge_uploads,
                 since_version=since_version,
                 target_distribution=self.data.upload_target_distribution,
@@ -368,12 +421,20 @@ class DebianPipelineWorkflow(
                 display_name="package upload", step="package_upload"
             ),
         )
+
         # The package_upload workflow's children will have dependencies on
-        # the work requests creating binary_artifacts, but the
-        # package_upload workflow itself doesn't need that in order to
-        # populate itself.
-        wr.mark_running()
-        orchestrate_workflow(wr)
+        # the work requests creating source_artifact and binary_artifacts,
+        # but the package_upload workflow itself doesn't normally need that
+        # in order to populate itself.  An exception is if we're uploading
+        # signed source with binaries, in which case the make_signed_source
+        # workflow will only be able to create even the necessary promises
+        # once the signed source package has been assembled.
+        if signed_source and include_binaries and binary_artifacts:
+            self.requires_artifact(wr, source_artifact)
+
+        if wr.status == WorkRequest.Statuses.PENDING:
+            wr.mark_running()
+            orchestrate_workflow(wr)
 
     def get_label(self) -> str:
         """Return the task label."""
