@@ -10,7 +10,7 @@
 """Pydantic models used by debusine workflows."""
 
 import re
-from datetime import datetime, timedelta
+from datetime import datetime
 from enum import StrEnum
 from typing import Any
 
@@ -110,6 +110,7 @@ class WorkRequestWorkflowData(pydantic.BaseModel):
     #: Whether this task requires user input.  Only relevant for WAIT tasks.
     needs_input: bool | None = None
 
+    #: The number of times this task has been retried automatically.
     retry_count: int = pydantic.Field(default=0)
 
 
@@ -216,6 +217,7 @@ class PiupartsWorkflowData(BaseWorkflowData):
     codename: str
     architectures: list[str] | None = None
     backend: BackendType = BackendType.AUTO
+    environment: LookupSingle | None = None
     arch_all_host_architecture: str = "amd64"
     extra_repositories: list[ExtraRepository] | None = None
 
@@ -317,6 +319,7 @@ class QAWorkflowData(BaseWorkflowData):
 
     enable_piuparts: bool = True
     piuparts_backend: BackendType = BackendType.AUTO
+    piuparts_environment: LookupSingle | None = None
 
     @pydantic.root_validator(allow_reuse=True)
     @classmethod
@@ -366,6 +369,7 @@ class DebianPipelineWorkflowData(BaseWorkflowData):
 
     enable_piuparts: bool = True
     piuparts_backend: BackendType = BackendType.AUTO
+    piuparts_environment: LookupSingle | None = None
 
     enable_make_signed_source: bool = False
     make_signed_source_purpose: KeyPurpose | None = None
@@ -437,7 +441,7 @@ class ExperimentWorkspaceData(BaseWorkflowData):
     public: bool = True
     owner_group: str | None = None
     workflow_template_names: list[str] = []
-    expiration_delay: timedelta | None = timedelta(days=60)
+    expiration_delay: int | None = 60
 
     @pydantic.validator("experiment_name")
     @classmethod
