@@ -8,6 +8,9 @@
 # contained in the LICENSE file.
 
 """Package upload workflow."""
+
+from functools import cached_property
+
 try:
     import pydantic.v1 as pydantic
 except ImportError:
@@ -64,6 +67,22 @@ class PackageUploadWorkflow(
 
         self._artifacts_pending_upload: list[LookupSingle] = []
 
+    @cached_property
+    def source_category(self) -> str | None:
+        """Category of source artifact or promise."""
+        if self.data.source_artifact is None:
+            return None
+
+        return workflow_utils.lookup_result_artifact_category(
+            lookup_single(
+                self.data.source_artifact,
+                self.workspace,
+                user=self.work_request.created_by,
+                workflow_root=self.work_request.get_workflow_root(),
+                expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
+            )
+        )
+
     def validate_input(self) -> None:
         """Raise WorkflowValidationError if needed."""
         if (
@@ -74,47 +93,28 @@ class PackageUploadWorkflow(
                 '"source_artifact" or "binary_artifacts" must be set'
             )
 
-        if self.data.source_artifact is not None:
-            source_artifact = lookup_single(
-                self.data.source_artifact,
-                self.workspace,
-                user=self.work_request.created_by,
-                workflow_root=self.work_request.get_workflow_root(),
-                expect_type=LookupChildType.ARTIFACT,
-            ).artifact
-
-            if (
-                source_artifact.category == ArtifactCategory.SOURCE_PACKAGE
-                and (self.data.vendor is None or self.data.codename is None)
-            ):
-                # MakeSourcePackageUpload will be used but
-                # self.data.vendor or self.data.codename are not set
-                raise WorkflowValidationError(
-                    f'"vendor" and "codename" are required when source '
-                    f'artifact category is {ArtifactCategory.SOURCE_PACKAGE}'
-                )
+        if self.source_category == ArtifactCategory.SOURCE_PACKAGE and (
+            self.data.vendor is None or self.data.codename is None
+        ):
+            # MakeSourcePackageUpload will be used, but self.data.vendor or
+            # self.data.codename are not set.
+            raise WorkflowValidationError(
+                f'"vendor" and "codename" are required when source '
+                f'artifact category is {ArtifactCategory.SOURCE_PACKAGE}'
+            )
 
     def populate(self) -> None:
         """Create package upload work requests."""
         workflow_root = self.work_request.get_workflow_root()
 
         if (source_artifact := self.data.source_artifact) is not None:
-            artifact = lookup_single(
-                source_artifact,
-                self.workspace,
-                user=self.work_request.created_by,
-                workflow_root=workflow_root,
-                expect_type=LookupChildType.ARTIFACT,
-            ).artifact
             since_version = self.data.since_version
             target_distribution = self.data.target_distribution
 
-            assert self.data.source_artifact
-
-            if artifact.category == ArtifactCategory.SOURCE_PACKAGE:
+            if self.source_category == ArtifactCategory.SOURCE_PACKAGE:
                 package_upload_source = (
                     self._populate_make_source_package_upload(
-                        self.data.source_artifact,
+                        source_artifact,
                         since_version=since_version,
                         target_distribution=target_distribution,
                     )
@@ -122,7 +122,7 @@ class PackageUploadWorkflow(
                 self._artifacts_pending_upload.append(package_upload_source)
 
             else:
-                self._artifacts_pending_upload.append(self.data.source_artifact)
+                self._artifacts_pending_upload.append(source_artifact)
 
         # Upload binary_artifacts
         results = lookup_multiple(
