@@ -248,6 +248,17 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
         assert source_artifact.category == ArtifactCategory.SOURCE_PACKAGE
         debian_source_package = DebianSourcePackage(**source_artifact.data)
 
+        vendor, codename = self.data.target_distribution.split(":", 1)
+        extra_repositories = workflow_utils.configure_for_overlay_suite(
+            self,
+            extra_repositories=self.data.extra_repositories,
+            vendor=vendor,
+            codename=codename,
+            environment=environment,
+            backend=self.data.backend,
+            architecture=architecture,
+        )
+
         task_data = SbuildData(
             input=input_,
             host_architecture=host_architecture,
@@ -261,7 +272,7 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
                 )
             ],
             build_profiles=self.data.build_profiles,
-            extra_repositories=self.data.extra_repositories,
+            extra_repositories=extra_repositories,
             binnmu=self.data.binnmu,
         )
         wr = self.work_request_ensure_child(
@@ -294,8 +305,6 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
             build_logs_collection_id = None
 
         if build_logs_collection_id is not None:
-            vendor, codename = self.data.target_distribution.split(":", 1)
-
             variables = {
                 "work_request_id": wr.id,
                 "vendor": vendor,
@@ -339,7 +348,10 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
                 ArtifactCategory.BINARY_PACKAGE,
                 f"{self.data.prefix}signing-template-{architecture}-"
                 f"{binary_package_name}",
-                data={"architecture": architecture},
+                data={
+                    "architecture": architecture,
+                    "binary_package_name": binary_package_name,
+                },
                 artifact_filters={
                     "data__deb_fields__Package": binary_package_name
                 },
