@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -14,16 +14,15 @@ from unittest import mock
 
 from debusine.db.models import WorkRequest
 from debusine.server.workflows.base import (
-    Workflow,
     WorkflowRunError,
     orchestrate_workflow,
 )
 from debusine.server.workflows.models import BaseWorkflowData
+from debusine.server.workflows.tests.helpers import TestWorkflow
 from debusine.tasks.models import BaseDynamicTaskData, TaskTypes
 from debusine.tasks.server import TaskDatabaseInterface
-from debusine.tasks.tests.helper_mixin import TestTaskMixin
 from debusine.test.django import TestCase
-from debusine.test.utils import preserve_task_registry
+from debusine.test.test_utils import preserve_task_registry
 
 
 class OrchestrateWorkflowTests(TestCase):
@@ -55,7 +54,7 @@ class OrchestrateWorkflowTests(TestCase):
             dynamic: str
 
         class ExampleWorkflow(
-            TestTaskMixin, Workflow[BaseWorkflowData, ExampleDynamicData]
+            TestWorkflow[BaseWorkflowData, ExampleDynamicData]
         ):
             TASK_NAME = "example"
 
@@ -92,7 +91,7 @@ class OrchestrateWorkflowTests(TestCase):
             dynamic: str
 
         class ExampleWorkflow(
-            TestTaskMixin, Workflow[BaseWorkflowData, ExampleDynamicData]
+            TestWorkflow[BaseWorkflowData, ExampleDynamicData]
         ):
             TASK_NAME = "example"
 
@@ -142,7 +141,9 @@ class OrchestrateWorkflowTests(TestCase):
         ):
             orchestrate_workflow(wr)
 
-        self.assertEqual(raised.exception.args, (wr, expected_message))
+        self.assertEqual(raised.exception.work_request, wr)
+        self.assertEqual(raised.exception.message, expected_message)
+        self.assertEqual(raised.exception.code, "configure-failed")
         wr.refresh_from_db()
         self.assertEqual(wr.status, WorkRequest.Statuses.COMPLETED)
         self.assertEqual(wr.result, WorkRequest.Results.ERROR)
@@ -150,12 +151,12 @@ class OrchestrateWorkflowTests(TestCase):
     def test_workflow_callback_bad_task_data(self) -> None:
         """A workflow callback with bad task data is skipped."""
         parent = self.playground.create_workflow(
-            task_name="noop", task_data={"nonsense": ""}
+            task_name="noop", task_data={"nonsense": ""}, validate=False
         )
         wr = WorkRequest.objects.create_workflow_callback(
             parent=parent, step="test"
         )
-        expected_message = "failed to configure: 1 validation error"
+        expected_message = "Failed to configure: 1 validation error"
 
         with (
             mock.patch(
@@ -172,9 +173,9 @@ class OrchestrateWorkflowTests(TestCase):
             orchestrate_workflow(wr)
 
         mock_noop_callback.assert_not_called()
-        self.assertEqual(len(raised.exception.args), 2)
-        self.assertEqual(raised.exception.args[0], wr)
-        self.assertTrue(raised.exception.args[1].startswith(expected_message))
+        self.assertEqual(raised.exception.work_request, wr)
+        self.assertTrue(raised.exception.message.startswith(expected_message))
+        self.assertEqual(raised.exception.code, "configure-failed")
         wr.refresh_from_db()
         self.assertEqual(wr.status, WorkRequest.Statuses.COMPLETED)
         self.assertEqual(wr.result, WorkRequest.Results.ERROR)
@@ -185,6 +186,10 @@ class OrchestrateWorkflowTests(TestCase):
         wr = WorkRequest.objects.create_workflow_callback(
             parent=parent, step="test"
         )
+        expected_message = (
+            f"Failed to compute dynamic data for Workflow/noop ({parent.id}): "
+            f"Boom"
+        )
 
         with (
             mock.patch(
@@ -195,25 +200,20 @@ class OrchestrateWorkflowTests(TestCase):
             mock.patch(
                 "debusine.server.workflows.noop.NoopWorkflow.callback"
             ) as mock_noop_callback,
-            self.assertLogs("debusine", level=logging.WARNING) as log,
+            self.assertLogsContains(
+                f"Error running work request Internal/workflow ({wr.id}): "
+                f"{expected_message}",
+                logger="debusine.server.workflows.base",
+                level=logging.WARNING,
+            ),
             self.assertRaises(WorkflowRunError) as raised,
         ):
             orchestrate_workflow(wr)
 
         mock_noop_callback.assert_not_called()
-        self.assertIn(
-            f"Workflow orchestrator failed to pre-process task data for "
-            f"WorkRequest {parent.id}. "
-            f"Task data: {parent.task_data} Error: Boom",
-            "\n".join(log.output),
-        )
-        expected_message = "failed to compute dynamic data"
-        self.assertIn(
-            f"Error running work request Internal/workflow ({wr.id}): "
-            f"{expected_message}",
-            "\n".join(log.output),
-        )
-        self.assertEqual(raised.exception.args, (wr, expected_message))
+        self.assertEqual(raised.exception.work_request, wr)
+        self.assertEqual(raised.exception.message, expected_message)
+        self.assertEqual(raised.exception.code, "dynamic-data-failed")
         wr.refresh_from_db()
         self.assertEqual(wr.status, WorkRequest.Statuses.COMPLETED)
         self.assertEqual(wr.result, WorkRequest.Results.ERROR)
@@ -224,7 +224,7 @@ class OrchestrateWorkflowTests(TestCase):
         wr = WorkRequest.objects.create_workflow_callback(
             parent=parent, step="test"
         )
-        expected_message = "orchestrator failed: Boom"
+        expected_message = "Orchestrator failed: Boom"
 
         with (
             mock.patch(
@@ -242,7 +242,9 @@ class OrchestrateWorkflowTests(TestCase):
             orchestrate_workflow(wr)
 
         mock_noop_callback.assert_called_once_with(wr)
-        self.assertEqual(raised.exception.args, (wr, expected_message))
+        self.assertEqual(raised.exception.work_request, wr)
+        self.assertEqual(raised.exception.message, expected_message)
+        self.assertEqual(raised.exception.code, "orchestrator-failed")
         wr.refresh_from_db()
         self.assertEqual(wr.status, WorkRequest.Statuses.COMPLETED)
         self.assertEqual(wr.result, WorkRequest.Results.ERROR)
@@ -252,6 +254,25 @@ class OrchestrateWorkflowTests(TestCase):
         wr = self.playground.create_workflow(task_name="noop")
         wr.mark_running()
 
+        def populate() -> None:
+            wr.create_child("noop")
+
+        with mock.patch(
+            "debusine.server.workflows.noop.NoopWorkflow.populate",
+            side_effect=populate,
+        ) as mock_noop_populate:
+            orchestrate_workflow(wr)
+
+        mock_noop_populate.assert_called_once_with()
+        wr.refresh_from_db()
+        self.assertEqual(wr.status, WorkRequest.Statuses.RUNNING)
+        self.assertEqual(wr.result, WorkRequest.Results.NONE)
+
+    def test_workflow_empty(self) -> None:
+        """An empty workflow is populated and marked as completed."""
+        wr = self.playground.create_workflow(task_name="noop")
+        wr.mark_running()
+
         with mock.patch(
             "debusine.server.workflows.noop.NoopWorkflow.populate"
         ) as mock_noop_populate:
@@ -259,8 +280,8 @@ class OrchestrateWorkflowTests(TestCase):
 
         mock_noop_populate.assert_called_once_with()
         wr.refresh_from_db()
-        self.assertEqual(wr.status, WorkRequest.Statuses.RUNNING)
-        self.assertEqual(wr.result, WorkRequest.Results.NONE)
+        self.assertEqual(wr.status, WorkRequest.Statuses.COMPLETED)
+        self.assertEqual(wr.result, WorkRequest.Results.SUCCESS)
 
     @preserve_task_registry()
     def test_workflow_computes_dynamic_data(self) -> None:
@@ -270,7 +291,7 @@ class OrchestrateWorkflowTests(TestCase):
             dynamic: str
 
         class ExampleWorkflow(
-            TestTaskMixin, Workflow[BaseWorkflowData, ExampleDynamicData]
+            TestWorkflow[BaseWorkflowData, ExampleDynamicData]
         ):
             TASK_NAME = "example"
 
@@ -280,16 +301,13 @@ class OrchestrateWorkflowTests(TestCase):
                 return ExampleDynamicData(dynamic="foo")
 
             def populate(self) -> None:
-                """Unused abstract method from Workflow."""
-                raise NotImplementedError()
+                self.work_request.create_child("noop")
 
         wr = self.playground.create_workflow(task_name="example")
         wr.mark_running()
 
-        with mock.patch.object(ExampleWorkflow, "populate") as mock_populate:
-            orchestrate_workflow(wr)
+        orchestrate_workflow(wr)
 
-        mock_populate.assert_called_once_with()
         wr.refresh_from_db()
         self.assertEqual(wr.dynamic_task_data, {"dynamic": "foo"})
         self.assertEqual(wr.status, WorkRequest.Statuses.RUNNING)
@@ -306,7 +324,7 @@ class OrchestrateWorkflowTests(TestCase):
         wr.task_data = {"nonsense": ""}
         wr.save()
         wr.mark_running()
-        expected_message = "failed to configure: 1 validation error"
+        expected_message = "Failed to configure: 1 validation error"
 
         with (
             mock.patch(
@@ -323,9 +341,9 @@ class OrchestrateWorkflowTests(TestCase):
             orchestrate_workflow(wr)
 
         mock_noop_populate.assert_not_called()
-        self.assertEqual(len(raised.exception.args), 2)
-        self.assertEqual(raised.exception.args[0], wr)
-        self.assertTrue(raised.exception.args[1].startswith(expected_message))
+        self.assertEqual(raised.exception.work_request, wr)
+        self.assertTrue(raised.exception.message.startswith(expected_message))
+        self.assertEqual(raised.exception.code, "configure-failed")
         wr.refresh_from_db()
         self.assertEqual(wr.status, WorkRequest.Statuses.COMPLETED)
         self.assertEqual(wr.result, WorkRequest.Results.ERROR)
@@ -334,6 +352,7 @@ class OrchestrateWorkflowTests(TestCase):
         """A workflow where computing dynamic data fails is skipped."""
         wr = self.playground.create_workflow(task_name="noop")
         wr.mark_running()
+        expected_message = "Failed to compute dynamic data: Boom"
 
         with (
             mock.patch(
@@ -344,25 +363,20 @@ class OrchestrateWorkflowTests(TestCase):
             mock.patch(
                 "debusine.server.workflows.noop.NoopWorkflow.populate"
             ) as mock_noop_populate,
-            self.assertLogs("debusine", level=logging.WARNING) as log,
+            self.assertLogsContains(
+                f"Error running work request Workflow/noop ({wr.id}): "
+                f"{expected_message}",
+                logger="debusine.server.workflows.base",
+                level=logging.WARNING,
+            ),
             self.assertRaises(WorkflowRunError) as raised,
         ):
             orchestrate_workflow(wr)
 
         mock_noop_populate.assert_not_called()
-        self.assertIn(
-            f"Workflow orchestrator failed to pre-process task data for "
-            f"WorkRequest {wr.id}. "
-            f"Task data: {wr.task_data} Error: Boom",
-            "\n".join(log.output),
-        )
-        expected_message = "failed to compute dynamic data"
-        self.assertIn(
-            f"Error running work request Workflow/noop ({wr.id}): "
-            f"{expected_message}",
-            "\n".join(log.output),
-        )
-        self.assertEqual(raised.exception.args, (wr, expected_message))
+        self.assertEqual(raised.exception.work_request, wr)
+        self.assertEqual(raised.exception.message, expected_message)
+        self.assertEqual(raised.exception.code, "dynamic-data-failed")
         wr.refresh_from_db()
         self.assertEqual(wr.status, WorkRequest.Statuses.COMPLETED)
         self.assertEqual(wr.result, WorkRequest.Results.ERROR)
@@ -371,7 +385,7 @@ class OrchestrateWorkflowTests(TestCase):
         """A workflow that fails is logged."""
         wr = self.playground.create_workflow(task_name="noop")
         wr.mark_running()
-        expected_message = "orchestrator failed: Boom"
+        expected_message = "Orchestrator failed: Boom"
 
         with (
             mock.patch(
@@ -389,7 +403,9 @@ class OrchestrateWorkflowTests(TestCase):
             orchestrate_workflow(wr)
 
         mock_noop_populate.assert_called_once_with()
-        self.assertEqual(raised.exception.args, (wr, expected_message))
+        self.assertEqual(raised.exception.work_request, wr)
+        self.assertEqual(raised.exception.message, expected_message)
+        self.assertEqual(raised.exception.code, "orchestrator-failed")
         wr.refresh_from_db()
         self.assertEqual(wr.status, WorkRequest.Statuses.COMPLETED)
         self.assertEqual(wr.result, WorkRequest.Results.ERROR)
@@ -399,7 +415,7 @@ class OrchestrateWorkflowTests(TestCase):
         wr = self.playground.create_workflow(task_name="noop")
         wr.mark_running()
 
-        def populate():
+        def populate() -> None:
             children = [wr.create_child("noop") for _ in range(2)]
             children[1].add_dependency(children[0])
 
@@ -421,7 +437,7 @@ class OrchestrateWorkflowTests(TestCase):
     def test_wrong_task_type(self) -> None:
         """Attempts to orchestrate non-workflows are logged."""
         wr = self.playground.create_work_request()
-        expected_message = "does not have a workflow orchestrator"
+        expected_message = "Does not have a workflow orchestrator"
 
         with (
             self.assertLogsContains(
@@ -434,7 +450,9 @@ class OrchestrateWorkflowTests(TestCase):
         ):
             orchestrate_workflow(wr)
 
-        self.assertEqual(raised.exception.args, (wr, expected_message))
+        self.assertEqual(raised.exception.work_request, wr)
+        self.assertEqual(raised.exception.message, expected_message)
+        self.assertEqual(raised.exception.code, "configure-failed")
         wr.refresh_from_db()
         self.assertEqual(wr.status, WorkRequest.Statuses.COMPLETED)
         self.assertEqual(wr.result, WorkRequest.Results.ERROR)
