@@ -15,7 +15,6 @@ from django.test import override_settings
 
 from debusine.artifacts.models import (
     ArtifactCategory,
-    DebianSourcePackage,
     DebusineSigningInput,
     DebusineSigningOutput,
 )
@@ -134,7 +133,7 @@ class MakeSignedSourceWorkflowTests(TestCase):
                             target_distribution="debian:bookworm",
                             architectures=sbuild_architectures,
                             signing_template_names={
-                                arch: [f"hello-{arch}-signed"]
+                                arch: [f"hello-{arch}-signed-template"]
                                 for arch in sbuild_architectures
                             },
                         ),
@@ -251,10 +250,10 @@ class MakeSignedSourceWorkflowTests(TestCase):
         )
         self.assertEqual(signs.count(), len(architectures))
 
-        assemble_sign_sources = make_signed_source.children.filter(
+        assemble_signed_sources = make_signed_source.children.filter(
             task_name="assemblesignedsource"
         )
-        self.assertEqual(assemble_sign_sources.count(), len(architectures))
+        self.assertEqual(assemble_signed_sources.count(), len(architectures))
 
         sbuild_workflows = make_signed_source.children.filter(
             task_type=TaskTypes.WORKFLOW, task_name="sbuild"
@@ -262,6 +261,7 @@ class MakeSignedSourceWorkflowTests(TestCase):
         self.assertEqual(sbuild_workflows.count(), len(architectures))
 
         for arch in architectures:
+            template_name = f"hello-{arch}-signed-template"
             extract_for_signing = extract_for_signings.get(
                 task_data__input__template_artifact__startswith=(
                     f"internal@collections/name:signing-template-{arch}-"
@@ -277,7 +277,7 @@ class MakeSignedSourceWorkflowTests(TestCase):
                         ],
                         "template_artifact": (
                             f"internal@collections/"
-                            f"name:signing-template-{arch}-hello-{arch}-signed"
+                            f"name:signing-template-{arch}-{template_name}"
                         ),
                     },
                 },
@@ -285,8 +285,10 @@ class MakeSignedSourceWorkflowTests(TestCase):
             self.assertEqual(
                 extract_for_signing.workflow_data_json,
                 {
-                    "display_name": f"Extract for signing {arch}",
-                    "step": f"extract-for-signing-{arch}",
+                    "display_name": (
+                        f"Extract for signing ({template_name}/{arch})"
+                    ),
+                    "step": f"extract-for-signing-{arch}-{template_name}",
                 },
             )
             self.assertEqual(
@@ -339,7 +341,10 @@ class MakeSignedSourceWorkflowTests(TestCase):
             )
             self.assertEqual(
                 sign.workflow_data_json,
-                {"display_name": f"Sign {arch}", "step": f"sign-{arch}"},
+                {
+                    "display_name": f"Sign ({template_name}/{arch})",
+                    "step": f"sign-{arch}-{template_name}",
+                },
             )
 
             self.assertEqual(
@@ -355,11 +360,13 @@ class MakeSignedSourceWorkflowTests(TestCase):
                             },
                             "collection": "internal@collections",
                             "name_template": (
-                                "signed-{architecture}-{binary_package_name}"
+                                "signed-{architecture}-{template_name}-"
+                                "{binary_package_name}"
                             ),
                             "variables": {
                                 "$binary_package_name": "binary_package_name",
                                 "architecture": arch,
+                                "template_name": template_name,
                             },
                         }
                     ],
@@ -372,48 +379,50 @@ class MakeSignedSourceWorkflowTests(TestCase):
                     "collection": "internal@collections",
                     "child_type": LookupChildType.ARTIFACT,
                     "category": ArtifactCategory.SIGNING_OUTPUT,
-                    "name__startswith": "signed-",
+                    "name__startswith": f"signed-{arch}-{template_name}-",
                     "data__architecture": arch,
                 }
             )
-            assemble_sign_source = assemble_sign_sources.get(
+            assemble_signed_source = assemble_signed_sources.get(
                 task_data__signed=signed.dict(exclude_unset=True)["__root__"]
             )
 
             self.assertEqual(
                 {
                     key: value
-                    for key, value in assemble_sign_source.task_data.items()
+                    for key, value in assemble_signed_source.task_data.items()
                     if key != "signed"
                 },
                 {
                     "environment": "debian/match:codename=bookworm",
                     "template": (
                         f"internal@collections/"
-                        f"name:signing-template-{arch}-hello-{arch}-signed"
+                        f"name:signing-template-{arch}-{template_name}"
                     ),
                 },
             )
             self.assertEqual(
-                assemble_sign_source.workflow_data_json,
+                assemble_signed_source.workflow_data_json,
                 {
-                    "display_name": f"Assemble signed source {arch}",
-                    "step": f"assemble-signed-source-{arch}",
+                    "display_name": (
+                        f"Assemble signed source ({template_name}/{arch})"
+                    ),
+                    "step": f"assemble-signed-source-{arch}-{template_name}",
                 },
             )
             self.assertEqual(
-                assemble_sign_source.event_reactions_json,
+                assemble_signed_source.event_reactions_json,
                 {
                     "on_creation": [],
                     "on_failure": [],
                     "on_success": [
                         {
                             "action": "update-collection-with-artifacts",
-                            "artifact_filters": {
-                                "category": "debian:source-package"
-                            },
+                            "artifact_filters": {"category": "debian:upload"},
                             "collection": "internal@collections",
-                            "name_template": f"signed-source-{arch}",
+                            "name_template": (
+                                f"signed-source-{arch}-{template_name}"
+                            ),
                             "variables": None,
                         }
                     ],
@@ -423,7 +432,8 @@ class MakeSignedSourceWorkflowTests(TestCase):
 
             sbuild_workflow = sbuild_workflows.get(
                 task_data__input__source_artifact=(
-                    f"internal@collections/name:signed-source-{arch}"
+                    f"internal@collections/"
+                    f"name:signed-source-{arch}-{template_name}"
                 )
             )
             self.assertEqual(
@@ -433,12 +443,13 @@ class MakeSignedSourceWorkflowTests(TestCase):
             self.assertEqual(
                 sbuild_workflow.task_data,
                 {
-                    "prefix": "signed-source|",
+                    "prefix": f"signed-source-{arch}-{template_name}|",
                     "architectures": ["all", "amd64", "i386"],
                     "backend": "auto",
                     "input": {
                         "source_artifact": (
-                            f"internal@collections/name:signed-source-{arch}"
+                            f"internal@collections/"
+                            f"name:signed-source-{arch}-{template_name}"
                         ),
                         "extra_binary_artifacts": [
                             f"internal@collections/name:build-{arch}"
@@ -449,7 +460,10 @@ class MakeSignedSourceWorkflowTests(TestCase):
             )
             self.assertEqual(
                 sbuild_workflow.workflow_data_json,
-                {"display_name": f"Sbuild {arch}", "step": f"sbuild-{arch}"},
+                {
+                    "display_name": f"Sbuild ({template_name}/{arch})",
+                    "step": f"sbuild-{arch}-{template_name}",
+                },
             )
             self.assertEqual(sbuild_workflow.event_reactions_json, {})
 
@@ -496,10 +510,10 @@ class MakeSignedSourceWorkflowTests(TestCase):
                     result.artifact
                     for result in lookup_multiple(
                         LookupMultiple.parse_obj(
-                            assemble_sign_source.task_data["signed"]
+                            assemble_signed_source.task_data["signed"]
                         ),
-                        assemble_sign_source.workspace,
-                        user=assemble_sign_source.created_by,
+                        assemble_signed_source.workspace,
+                        user=assemble_signed_source.created_by,
                         workflow_root=root,
                         expect_type=LookupChildType.ARTIFACT,
                     )
@@ -507,19 +521,16 @@ class MakeSignedSourceWorkflowTests(TestCase):
                 [signing_output],
             )
 
-            source_package, _ = self.playground.create_artifact(
-                category=ArtifactCategory.SOURCE_PACKAGE,
-                data=DebianSourcePackage(
-                    name=f"hello-{arch}-signed",
-                    version="1.0.0",
-                    type="dpkg",
-                    dsc_fields={"Architecture": arch},
-                ),
-                workspace=assemble_sign_source.workspace,
-                work_request=assemble_sign_source,
+            upload_artifacts = self.playground.create_upload_artifacts(
+                src_name=f"hello-{arch}-signed",
+                version="1.0.0",
+                source=True,
+                binary=False,
+                workspace=assemble_signed_source.workspace,
+                work_request=assemble_signed_source,
             )
-            assemble_sign_source.refresh_from_db()
-            assemble_sign_source.mark_completed(WorkRequest.Results.SUCCESS)
+            assemble_signed_source.refresh_from_db()
+            assemble_signed_source.mark_completed(WorkRequest.Results.SUCCESS)
             self.assertEqual(
                 lookup_single(
                     sbuild_workflow.task_data["input"]["source_artifact"],
@@ -528,7 +539,7 @@ class MakeSignedSourceWorkflowTests(TestCase):
                     workflow_root=root,
                     expect_type=LookupChildType.ARTIFACT,
                 ).artifact,
-                source_package,
+                upload_artifacts.upload,
             )
 
             # The sbuild sub-workflow is now marked pending, and will be
