@@ -8,6 +8,8 @@
 # contained in the LICENSE file.
 
 """Unit tests for the package upload workflow."""
+
+from collections.abc import Sequence
 from typing import Any
 from unittest.mock import call, patch
 
@@ -70,26 +72,34 @@ class PackageUploadWorkflowTests(TestCase):
         )
         return artifact
 
-    def create_source_binary_artifacts(self) -> tuple[Artifact, Artifact]:
-        """Return one source and one binary package."""
+    def create_source_binary_artifacts(
+        self, architectures: Sequence[str] | None = None
+    ) -> tuple[Artifact, list[Artifact]]:
+        """Return one source and a list of binary packages (one per arch)."""
+        if architectures is None:
+            architectures = ("amd64",)
         collection = self.playground.create_collection(
             "test", CollectionCategory.TEST
         )
 
         source = self.create_source_package()
-        binary = self.playground.create_minimal_binary_packages_artifact(
-            "hello", "1.0-1", "1.0-1", "amd64"
-        )
-        CollectionItem.objects.create_from_artifact(
-            binary,
-            parent_collection=collection,
-            name="binary",
-            created_by_user=self.playground.get_default_user(),
-            data={},
-            created_by_workflow=None,
-        )
+        binaries = [
+            self.playground.create_minimal_binary_packages_artifact(
+                "hello", "1.0-1", "1.0-1", architecture
+            )
+            for architecture in architectures
+        ]
+        for binary in binaries:
+            CollectionItem.objects.create_from_artifact(
+                binary,
+                parent_collection=collection,
+                name=f"binary-{binary.data['architecture']}",
+                created_by_user=self.playground.get_default_user(),
+                data={},
+                created_by_workflow=None,
+            )
 
-        return source, binary
+        return source, binaries
 
     def test_validate_input_source_binary_artifacts_not_set(self) -> None:
         """Invalid: source_artifact and binaries_artifacts not set."""
@@ -307,7 +317,7 @@ class PackageUploadWorkflowTests(TestCase):
     def test_populate_package_uploads_binaries_only(self) -> None:
         """The workflow create children package uploads: binaries only."""
         with context.disable_permission_checks():
-            (_, binary) = self.create_source_binary_artifacts()
+            (_, [binary]) = self.create_source_binary_artifacts()
 
         wr = self.orchestrate(
             data={
@@ -326,26 +336,20 @@ class PackageUploadWorkflowTests(TestCase):
 
     def test_populate_merge_uploads(self) -> None:
         """Test mergeuploads task is created (merge_uploads=True)."""
+        architectures = ("amd64", "i386")
         with context.disable_permission_checks():
-            (source, binary_1) = self.create_source_binary_artifacts()
-
-            binary_2 = self.playground.create_minimal_binary_packages_artifact(
-                "hello", "1.0-1", "1.0-1", "amd64"
-            )
-
-            CollectionItem.objects.create_from_artifact(
-                binary_2,
-                parent_collection=binary_1.parent_collections.earliest("id"),
-                name="binary-2",
-                created_by_user=self.playground.get_default_user(),
-                data={},
-                created_by_workflow=None,
+            (source, binaries) = self.create_source_binary_artifacts(
+                architectures=architectures
             )
+        binaries_collection = binaries[0].parent_collections.earliest("id")
 
         wr = self.orchestrate(
             data={
                 "source_artifact": source.id,
-                "binary_artifacts": [binary_1.id, binary_2.id],
+                "binary_artifacts": {
+                    "collection": binaries_collection.id,
+                    "name__startswith": "binary-",
+                },
                 "require_signature": False,
                 "target_distribution": "debian/match:codename=bookworm",
                 "merge_uploads": True,
@@ -365,9 +369,12 @@ class PackageUploadWorkflowTests(TestCase):
             {
                 "input": {
                     "uploads": [
-                        "internal@collections/name:package-upload-source",
-                        f"{binary_1.id}@artifacts",
-                        f"{binary_2.id}@artifacts",
+                        "internal@collections/name:package-upload-source"
+                    ]
+                    + [
+                        f"{binaries_collection.id}@collections/"
+                        f"name:binary-{architecture}"
+                        for architecture in architectures
                     ]
                 },
             },
@@ -398,7 +405,7 @@ class PackageUploadWorkflowTests(TestCase):
     def test_upload_debsign(self) -> None:
         """Upload signed artifacts."""
         with context.disable_permission_checks():
-            source, binary = self.create_source_binary_artifacts()
+            source, [binary] = self.create_source_binary_artifacts()
 
         wr = self.orchestrate(
             data={
@@ -487,7 +494,7 @@ class PackageUploadWorkflowTests(TestCase):
     def test_upload_externaldebsign(self) -> None:
         """Upload external signed artifacts."""
         with context.disable_permission_checks():
-            source, binary = self.create_source_binary_artifacts()
+            source, [binary] = self.create_source_binary_artifacts()
 
         wr = self.orchestrate(
             data={
@@ -591,20 +598,37 @@ class PackageUploadWorkflowTests(TestCase):
     def test_orchestrate_idempotent(self) -> None:
         """Calling orchestrate twice does not create new work requests."""
         with context.disable_permission_checks():
-            source, binary = self.create_source_binary_artifacts()
+            source, binaries = self.create_source_binary_artifacts(
+                architectures=("amd64", "i386")
+            )
+        binaries_collection = binaries[0].parent_collections.earliest("id")
 
-        wr = self.orchestrate(
-            data={
-                "source_artifact": source.id,
-                "binary_artifacts": [binary.id],
-                "vendor": "debian",
-                "codename": "bookworm",
-            },
-        )
+        # Force a predictable default ordering.
+        with patch(
+            "debusine.db.models.collections.CollectionItem._meta.ordering",
+            ["id"],
+        ):
+            wr = self.orchestrate(
+                data={
+                    "source_artifact": source.id,
+                    "binary_artifacts": {
+                        "collection": binaries_collection.id,
+                        "name__startswith": "binary-",
+                    },
+                    "vendor": "debian",
+                    "codename": "bookworm",
+                    "merge_uploads": True,
+                },
+            )
 
         children = set(wr.children.all())
 
-        PackageUploadWorkflow(wr).populate()
+        # Force a different predictable default ordering.
+        with patch(
+            "debusine.db.models.collections.CollectionItem._meta.ordering",
+            ["-id"],
+        ):
+            PackageUploadWorkflow(wr).populate()
 
         self.assertQuerySetEqual(wr.children.all(), children, ordered=False)
 
