@@ -1,4 +1,4 @@
-# Copyright 2021-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -30,8 +30,9 @@ import tempfile
 import traceback
 from abc import ABCMeta, abstractmethod
 from collections import defaultdict
-from collections.abc import Generator
+from collections.abc import Collection, Generator
 from contextlib import contextmanager
+from datetime import datetime, timezone
 from pathlib import Path
 from typing import (
     Any,
@@ -40,6 +41,8 @@ from typing import (
     ClassVar,
     Generic,
     IO,
+    Literal,
+    Self,
     TYPE_CHECKING,
     TextIO,
     TypeVar,
@@ -51,9 +54,9 @@ if TYPE_CHECKING:
     from _typeshed import OpenBinaryModeWriting, OpenTextModeWriting
 
 from debusine.artifacts import WorkRequestDebugLogs
-from debusine.artifacts.models import CollectionCategory
+from debusine.artifacts.models import ArtifactCategory, CollectionCategory
 from debusine.client.debusine import Debusine
-from debusine.client.models import ArtifactResponse
+from debusine.client.models import ArtifactResponse, RelationType
 from debusine.tasks.executors import (
     ExecutorImageCategory,
     ExecutorInterface,
@@ -61,11 +64,14 @@ from debusine.tasks.executors import (
     executor_class,
 )
 from debusine.tasks.models import (
+    ActionRecordInTaskHistory,
     BackendType,
     BaseDynamicTaskData,
     BaseDynamicTaskDataWithExecutor,
     BaseTaskData,
     BaseTaskDataWithExecutor,
+    BaseTaskDataWithExtraRepositories,
+    EventReaction,
     LookupSingle,
     TaskTypes,
     WorkerType,
@@ -74,7 +80,7 @@ from debusine.tasks.models import (
     parse_key_value_lookup_segment,
     parse_lookup_string_segments,
 )
-from debusine.tasks.server import TaskDatabaseInterface
+from debusine.tasks.server import ArtifactInfo, TaskDatabaseInterface
 from debusine.utils import extract_generic_type_arguments
 
 
@@ -82,7 +88,7 @@ class TaskConfigError(Exception):
     """Exception raised when there is an issue with a task configuration."""
 
     def __init__(
-        self, message: str, original_exception: Exception | None = None
+        self, message: str | None, original_exception: Exception | None = None
     ):
         """
         Initialize the TaskConfigError.
@@ -94,7 +100,14 @@ class TaskConfigError(Exception):
         super().__init__(message)
         self.original_exception = original_exception
 
-    def __str__(self):
+    def add_parent_message(self, msg: str) -> None:
+        """Prepend the error message with one from the containing scope."""
+        if self.args[0]:
+            self.args = (f"{msg}: {self.args[0]}",)
+        else:
+            self.args = (msg,)
+
+    def __str__(self) -> str:
         """
         Return a string representation.
 
@@ -102,12 +115,15 @@ class TaskConfigError(Exception):
         to the message for additional context.
         """
         if self.original_exception:
-            return (
-                f"{self.args[0]} (Original exception: "
-                f"{self.original_exception})"
-            )
-
-        return self.args[0]
+            if self.args[0] is not None:
+                return (
+                    f"{self.args[0]} (Original exception: "
+                    f"{self.original_exception})"
+                )
+            else:
+                return str(self.original_exception)
+        else:
+            return str(self.args[0])
 
 
 TD = TypeVar("TD", bound=BaseTaskData)
@@ -243,6 +259,18 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
             tempfile.TemporaryDirectory[str]
         ) = None
 
+    def instantiate_with_new_data(
+        self,
+        task_data: dict[str, Any],
+        dynamic_task_data: dict[str, Any] | None = None,
+    ) -> Self:
+        """Create a new task like this one, with different data."""
+        new_task = self.__class__(task_data, dynamic_task_data)
+        new_task.work_request_id = self.work_request_id
+        new_task.workspace_name = self.workspace_name
+        new_task.worker_host_architecture = self.worker_host_architecture
+        return new_task
+
     def append_to_log_file(self, filename: str, lines: list[str]) -> None:
         """
         Open log file and write contents into it.
@@ -253,6 +281,29 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
         with self.open_debug_log_file(filename) as file:
             file.writelines([line + "\n" for line in lines])
 
+    @staticmethod
+    def ensure_artifact_categories(
+        *,
+        configuration_key: str,
+        category: str,
+        expected: Collection[ArtifactCategory],
+    ) -> None:
+        """
+        Validate that the artifact's category is one of the expected categories.
+
+        :param configuration_key: Optional key to identify the source of the
+          artifact. Provides additional context in error messages.
+        :param category: The category to validate.
+        :param expected: A collection of valid artifact categories.
+        """
+        valid_categories = [cat.value for cat in sorted(expected)]
+        if category not in valid_categories:
+            raise TaskConfigError(
+                f"{configuration_key}: unexpected artifact "
+                f"category: '{category}'. "
+                f"Valid categories: {valid_categories}"
+            )
+
     @overload
     def open_debug_log_file(
         self, filename: str, *, mode: "OpenTextModeWriting" = "a"
@@ -324,7 +375,7 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
         }
 
     @classmethod
-    def analyze_worker_all_tasks(cls):
+    def analyze_worker_all_tasks(cls) -> dict[str, Any]:
         """
         Return dictionary with metadata for each task in BaseTask._sub_tasks.
 
@@ -351,6 +402,7 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
         """
         task_architecture = getattr(self.data, "host_architecture", None)
         if task_architecture is not None:
+            assert isinstance(task_architecture, str)
             return task_architecture
         return self.worker_host_architecture
 
@@ -408,15 +460,24 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
 
         return True
 
+    @abstractmethod
+    def build_dynamic_data(self, task_database: TaskDatabaseInterface) -> DTD:
+        """
+        Build a dynamic task data structure for this task.
+
+        :param task_database: TaskDatabaseInterface to use for lookups
+        :returns: the newly created dynamic task data
+        """
+
     def compute_dynamic_data(
         self, task_database: TaskDatabaseInterface  # noqa: U100
-    ) -> DTD | None:
+    ) -> DTD:
         """
         Compute dynamic data for this task.
 
         This may involve resolving artifact lookups.
         """
-        return None
+        return self.build_dynamic_data(task_database)
 
     def _configure(
         self,
@@ -442,7 +503,7 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
                 else self.dynamic_task_data_type(**dynamic_task_data)
             )
         except ValueError as exc:
-            raise TaskConfigError(str(exc), original_exception=exc)
+            raise TaskConfigError(None, original_exception=exc)
 
     def execute_logging_exceptions(self) -> bool:
         """Execute self.execute() logging any raised exceptions."""
@@ -580,6 +641,30 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
         :return: None if no label could be computed from task data
         """
 
+    def get_subject(self) -> str | None:
+        """Return the subject if known or None."""
+        if self.dynamic_data is not None:
+            return self.dynamic_data.subject
+
+        return None
+
+    def get_event_reactions(
+        self,
+        event_name: Literal[  # noqa: U100
+            "on_creation", "on_unblock", "on_success", "on_failure"
+        ],
+    ) -> list[EventReaction]:
+        """
+        Return event reactions for this task.
+
+        This allows tasks to provide actions that are processed by the
+        server at various points in the lifecycle of the work request.
+        """
+        event_reactions: list[EventReaction] = []
+        if event_name in {"on_success", "on_failure"}:
+            event_reactions.append(ActionRecordInTaskHistory())
+        return event_reactions
+
 
 class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
     r"""
@@ -714,7 +799,7 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
         """Clean up after running the task."""
         # Do nothing by default.
 
-    def _execute(self) -> bool:
+    def _execute(self) -> bool:  # noqa: C901
         """
         Fetch the required input, execute the command and upload artifacts.
 
@@ -735,18 +820,29 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
         with (
             self._temporary_directory() as execute_directory,
             self._temporary_directory() as download_directory,
+            self.open_debug_log_file("stages.log") as stages_log,
         ):
-            self.logger.info(
-                "Work request %s: Fetching input", self.work_request_id
-            )
-            if not self.fetch_input(download_directory):
+
+            def log_stage(message: str) -> None:
+                """Log a stage, both to the worker log and to a WR debug log."""
+                self.logger.info(
+                    "Work request %s: %s", self.work_request_id, message
+                )
+                print(
+                    f"{datetime.now(timezone.utc).isoformat()} {message}",
+                    file=stages_log,
+                )
+
+            try:
+                log_stage("Fetching input")
+                if not self.fetch_input(download_directory):
+                    return False
+            except Exception as exc:
+                self._log_exception(exc, stage="fetch_input")
                 return False
 
             try:
-                self.logger.info(
-                    "Work request %s: Configuring for execution",
-                    self.work_request_id,
-                )
+                log_stage("Configuring for execution")
                 if not self.configure_for_execution(download_directory):
                     return False
             except Exception as exc:
@@ -755,15 +851,10 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
 
             try:
                 try:
-                    self.logger.info(
-                        "Work request %s: Preparing to run",
-                        self.work_request_id,
-                    )
+                    log_stage("Preparing to run")
                     self.prepare_to_run(download_directory, execute_directory)
 
-                    self.logger.info(
-                        "Work request %s: Running", self.work_request_id
-                    )
+                    log_stage("Running")
 
                     execution_succeeded = self.run(execute_directory)
                 except Exception as exc:
@@ -771,9 +862,7 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
                     return False
 
                 try:
-                    self.logger.info(
-                        "Work request %s: Checking output", self.work_request_id
-                    )
+                    log_stage("Checking output")
                     if errors := self.check_directory_for_consistency_errors(
                         execute_directory
                     ):
@@ -782,10 +871,7 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
                         )
                         return False
 
-                    self.logger.info(
-                        "Work request %s: Uploading artifacts",
-                        self.work_request_id,
-                    )
+                    log_stage("Uploading artifacts")
                     self.upload_artifacts(
                         execute_directory, execution_success=execution_succeeded
                     )
@@ -794,15 +880,19 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
                     return False
             finally:
                 try:
-                    self.logger.info(
-                        "Work request %s: Cleaning up", self.work_request_id
-                    )
+                    log_stage("Cleaning up")
                     self.cleanup()
                 except Exception as exc:
                     self._log_exception(exc, stage="post_execution")
                     return False
 
-            return execution_succeeded
+        if execution_succeeded:
+            # We mainly care about stages.log for failures.  It's rather
+            # boring for successes.
+            assert self._debug_log_files_directory is not None
+            (Path(self._debug_log_files_directory.name) / "stages.log").unlink()
+
+        return execution_succeeded
 
     def _upload_work_request_debug_logs(self) -> None:
         """
@@ -832,7 +922,7 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
             self.debusine.relation_create(
                 remote_artifact.id,
                 source_artifact_id,
-                "relates-to",
+                RelationType.RELATES_TO,
             )
 
         self._debug_log_files_directory.cleanup()
@@ -846,7 +936,7 @@ def get_environment(
     backend: str | None,
     default_category: CollectionCategory | None = None,
     image_category: ExecutorImageCategory | None = None,
-) -> int:
+) -> ArtifactInfo:
     """
     Get an environment.
 
@@ -863,7 +953,7 @@ def get_environment(
     :param image_category: try to use an environment with this image
       category; defaults to the image category needed by the executor for
       `self.backend`
-    :return: the ID of a suitable environment artifact
+    :return: the ArtifactInfo of a suitable environment artifact
     """
     if (
         isinstance(lookup, str)
@@ -931,7 +1021,7 @@ class BaseTaskWithExecutor(BaseExternalTask[TDE, DTDE], Generic[TDE, DTDE]):
         default_category: CollectionCategory | None = None,
         image_category: ExecutorImageCategory | None = None,
         set_backend: bool = True,
-    ) -> int:
+    ) -> ArtifactInfo:
         """
         Get an environment for an executor-capable task.
 
@@ -948,7 +1038,7 @@ class BaseTaskWithExecutor(BaseExternalTask[TDE, DTDE], Generic[TDE, DTDE]):
           for `self.backend`
         :param set_backend: if True (default), try to use an environment
           matching `self.backend`
-        :return: the ID of a suitable environment artifact
+        :return: the ArtifactInfo of a suitable environment artifact
         """
         return get_environment(
             task_database=task_database,
@@ -1019,6 +1109,29 @@ class BaseTaskWithExecutor(BaseExternalTask[TDE, DTDE], Generic[TDE, DTDE]):
                 group=non_root_user,
             )
 
+    def run_executor_command(
+        self,
+        cmd: list[str],
+        log_filename: str,
+        run_as_root: bool = False,
+        check: bool = True,
+    ) -> None:
+        """Run cmd within the executor, logging the output to log_name."""
+        assert self.executor_instance
+        with self.open_debug_log_file(log_filename, mode="ab") as cmd_log:
+            msg = f"Executing: {shlex.join(cmd)}\n"
+            cmd_log.write(msg.encode())
+            cmd_log.flush()
+            p = self.executor_instance.run(
+                cmd,
+                run_as_root=run_as_root,
+                stdout=cmd_log,
+                stderr=cmd_log,
+                check=check,
+            )
+            msg = f"Execution completed (exit code {p.returncode})\n"
+            cmd_log.write(msg.encode())
+
     def cleanup(self) -> None:
         """
         Clean up after running the task.
@@ -1071,7 +1184,7 @@ class RunCommandTask(
     CMD_LOG_SEPARATOR = "--------------------"
     CMD_LOG_FILENAME = "cmd-output.log"
 
-    def run(self, execute_directory) -> bool:
+    def run(self, execute_directory: Path) -> bool:
         """
         Run a single command via the executor.
 
@@ -1097,7 +1210,7 @@ class RunCommandTask(
         return self.task_succeeded(returncode, execute_directory)
 
     def task_succeeded(
-        self,  # noqa: U100
+        self,
         returncode: int | None,  # noqa: U100
         execute_directory: Path,  # noqa: U100
     ) -> bool:
@@ -1129,7 +1242,9 @@ class RunCommandTask(
     def _write_utf8(file: BinaryIO, text: str) -> None:
         file.write(text.encode("utf-8", errors="replace") + b"\n")
 
-    def _write_popen_result(self, file: BinaryIO, p: subprocess.Popen[AnyStr]):
+    def _write_popen_result(
+        self, file: BinaryIO, p: subprocess.Popen[AnyStr]
+    ) -> None:
         self._write_utf8(file, f"\naborted: {self.aborted}")
         self._write_utf8(file, f"returncode: {p.returncode}")
 
@@ -1340,7 +1455,7 @@ class RunCommandTask(
         return popen.wait(timeout)
 
     @staticmethod
-    def _send_signal_pid(pid, signal) -> bool:
+    def _send_signal_pid(pid: int, signal: signal.Signals) -> bool:
         try:
             os.kill(pid, signal)
         except ProcessLookupError:
@@ -1349,9 +1464,103 @@ class RunCommandTask(
         return True
 
     @staticmethod
-    def _send_signal_group(process_group, signal) -> None:
+    def _send_signal_group(process_group: int, signal: signal.Signals) -> None:
         """Send signal to the process group."""
         try:
             os.killpg(process_group, signal)
         except ProcessLookupError:
             pass
+
+
+TDER = TypeVar("TDER", bound=BaseTaskDataWithExtraRepositories)
+
+
+class ExtraRepositoryMixin(
+    BaseExternalTask[TDER, DTD], Generic[TDER, DTD], metaclass=ABCMeta
+):
+    """Methods for configuring external APT repositories."""
+
+    data: TDER
+
+    # Deb822 style sources:
+    extra_repository_sources: list[Path]
+    # GPG keys (for apt < 2.3.10):
+    extra_repository_keys: list[Path]
+
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        """Initialize class variables."""
+        super().__init__(*args, **kwargs)
+        self.extra_repository_sources = []
+        self.extra_repository_keys = []
+
+    def supports_deb822_sources(self, codename: str) -> bool:
+        """Determine if codename supports deb822 sources."""
+        return codename not in ("jessie", "trusty")
+
+    def supports_inline_signed_by(self, codename: str) -> bool:
+        """Determine if codename supports inline signatures."""
+        # apt < 2.3.10 has no support for keys inline in Signed-By
+        return codename not in (
+            "jessie",
+            "stretch",
+            "buster",
+            "bullseye",
+            "trusty",
+            "xenial",
+            "bionic",
+            "focal",
+        )
+
+    def write_extra_repository_config(
+        self, codename: str, destination: Path
+    ) -> None:
+        """
+        Write extra_repositories config to files in destination.
+
+        extra_repository_keys will be populated with keys to install into
+        /etc/apt/keyrings, if needed.
+
+        extra_repository_sources will be populated with deb822 sources files,
+        if supported.
+        """
+        inline_signed_by = self.supports_inline_signed_by(codename)
+        for i, extra_repo in enumerate(self.data.extra_repositories or []):
+            if not inline_signed_by:
+                if extra_repo.signing_key:
+                    path = destination / f"extra_apt_key_{i}.asc"
+                    path.write_text(extra_repo.signing_key + "\n")
+                    self.extra_repository_keys.append(path)
+
+            # apt < 1.1 has no support for deb822 sources
+            if self.supports_deb822_sources(codename):
+                path = destination / f"extra_repository_{i}.sources"
+                signed_by = None
+                if not inline_signed_by:
+                    signed_by = f"/etc/apt/keyrings/extra_apt_key_{i}.asc"
+                path.write_text(
+                    extra_repo.as_deb822_source(signed_by_filename=signed_by)
+                )
+                self.extra_repository_sources.append(path)
+
+    def iter_oneline_sources(self) -> Generator[str]:
+        """
+        Generate apt one-line sources.list entries.
+
+        Use this for releases that don't supports_deb822_sources.
+        """
+        for i, extra_repository in enumerate(
+            self.data.extra_repositories or []
+        ):
+            yield extra_repository.as_oneline_source(
+                signed_by_filename=f"/etc/apt/keyrings/extra_apt_key_{i}.asc"
+            )
+
+
+class DefaultDynamicData(BaseTask[TD, BaseDynamicTaskData], metaclass=ABCMeta):
+    """Base class for tasks that do not add to dynamic task data."""
+
+    def build_dynamic_data(
+        self, task_database: TaskDatabaseInterface  # noqa: U100
+    ) -> BaseDynamicTaskData:
+        """Return default dynamic data."""
+        return BaseDynamicTaskData()
