@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -13,20 +13,33 @@ Executors for Debusine Tasks.
 Provides containment for tasks using a variety of backends.
 """
 
-from debusine.tasks.executors.base import (  # noqa: F401
+from debusine.tasks.executors.base import (
     ExecutorImageCategory,
     ExecutorInterface,
+    ExecutorStatistics,
     InstanceInterface,
     analyze_worker_all_executors,
     executor_class,
 )
-from debusine.tasks.executors.incus import (  # noqa: F401
+from debusine.tasks.executors.incus import (
     IncusInstance,
     IncusLXCExecutor,
     IncusVMExecutor,
 )
-from debusine.tasks.executors.qemu import QemuExecutor  # noqa: F401
-from debusine.tasks.executors.unshare import (  # noqa: F401
-    UnshareExecutor,
-    UnshareInstance,
-)
+from debusine.tasks.executors.qemu import QemuExecutor
+from debusine.tasks.executors.unshare import UnshareExecutor, UnshareInstance
+
+__all__ = [
+    "ExecutorImageCategory",
+    "ExecutorInterface",
+    "ExecutorStatistics",
+    "IncusInstance",
+    "IncusLXCExecutor",
+    "IncusVMExecutor",
+    "InstanceInterface",
+    "QemuExecutor",
+    "UnshareExecutor",
+    "UnshareInstance",
+    "analyze_worker_all_executors",
+    "executor_class",
+]
