@@ -1,4 +1,4 @@
-# Copyright 2021 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -11,12 +11,11 @@
 Task to build Debian packages with sbuild.
 
 This task implements the PackageBuild generic task for its task_data:
-https://freexian-team.pages.debian.net/debusine/reference/tasks.html#task-packagebuild
+https://freexian-team.pages.debian.net/debusine/reference/tasks/ontology-generic-tasks.html#task-packagebuild
 """
 
 import email.utils
 import os
-import subprocess
 from pathlib import Path
 from typing import Any, cast
 
@@ -39,10 +38,18 @@ from debusine.artifacts import (
 from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
+    DebianSourcePackage,
+    DebianSystemImage,
+    DebianSystemTarball,
     DoseDistCheck,
+    get_source_package_name,
+)
+from debusine.client.models import RelationType, RemoteArtifact
+from debusine.tasks import (
+    BaseTaskWithExecutor,
+    ExtraRepositoryMixin,
+    RunCommandTask,
 )
-from debusine.client.models import RemoteArtifact
-from debusine.tasks import BaseTaskWithExecutor, RunCommandTask, TaskConfigError
 from debusine.tasks.models import SbuildData, SbuildDynamicData
 from debusine.tasks.sbuild_validator_mixin import SbuildValidatorMixin
 from debusine.tasks.server import TaskDatabaseInterface
@@ -51,6 +58,7 @@ from debusine.utils import read_dsc
 
 class Sbuild(
     SbuildValidatorMixin,
+    ExtraRepositoryMixin[SbuildData, SbuildDynamicData],
     RunCommandTask[SbuildData, SbuildDynamicData],
     BaseTaskWithExecutor[SbuildData, SbuildDynamicData],
 ):
@@ -65,37 +73,82 @@ class Sbuild(
     ) -> None:
         """Initialize the sbuild task."""
         super().__init__(task_data, dynamic_task_data)
-        self.chroots = None
+        self.chroots: list[str] | None = None
         self.builder = "sbuild"
 
-        # dsc_file Path. Set by self.configure_for_execution()
+        # dsc_file Path. Set by self.fetch_input()
         self._dsc_file: Path | None = None
         self._extra_packages: list[Path] = []
 
-    def compute_dynamic_data(
+    def build_dynamic_data(
         self, task_database: TaskDatabaseInterface
     ) -> SbuildDynamicData:
-        """Resolve artifact lookups for this task."""
+        """
+        Resolve artifact lookups for this task.
+
+        :subject: source package name
+        :runtime_context:
+          ``$build_components:$host_architecture:$build_architecture`` or
+          None if ``build_options`` is provided
+        :configuration_context: codename of ``environment``
+        """
+        input_source_artifact = task_database.lookup_single_artifact(
+            self.data.input.source_artifact
+        )
+        self.ensure_artifact_categories(
+            configuration_key="input.source_artifact",
+            category=input_source_artifact.category,
+            expected=(ArtifactCategory.SOURCE_PACKAGE,),
+        )
+        assert isinstance(input_source_artifact.data, DebianSourcePackage)
+
+        environment = self.get_environment(
+            task_database,
+            self.data.environment,
+            default_category=CollectionCategory.ENVIRONMENTS,
+        )
+        self.ensure_artifact_categories(
+            configuration_key="environment",
+            category=environment.category,
+            expected=(
+                ArtifactCategory.SYSTEM_TARBALL,
+                ArtifactCategory.SYSTEM_IMAGE,
+            ),
+        )
+        assert isinstance(
+            environment.data, (DebianSystemTarball, DebianSystemImage)
+        )
+
         DEBUSINE_FQDN = task_database.get_server_setting("DEBUSINE_FQDN")
+
+        if self._build_options():
+            # Do not record stats for non-standard builds
+            runtime_context = None
+        else:
+            build_components = "+".join(sorted(self.data.build_components))
+
+            # Currently defining build_architecture for Sbuild is not
+            # supported. It defaults to the host_architecture
+            build_architecture = self.data.host_architecture
+
+            runtime_context = (
+                f"{build_components}:"
+                f"{self.data.host_architecture}:"
+                f"{build_architecture}"
+            )
+
         return SbuildDynamicData(
-            environment_id=(
-                None
-                if self.data.environment is None
-                else self.get_environment(
-                    task_database,
-                    self.data.environment,
-                    default_category=CollectionCategory.ENVIRONMENTS,
-                )
-            ),
-            input_source_artifact_id=task_database.lookup_single_artifact(
-                self.data.input.source_artifact
-            ),
+            environment_id=environment.id,
+            input_source_artifact_id=input_source_artifact.id,
             input_extra_binary_artifacts_ids=(
                 task_database.lookup_multiple_artifacts(
                     self.data.input.extra_binary_artifacts
-                )
+                ).get_ids()
             ),
             binnmu_maintainer=f"Debusine <noreply@{DEBUSINE_FQDN}>",
+            subject=get_source_package_name(input_source_artifact.data),
+            runtime_context=runtime_context,
+            configuration_context=environment.data.codename,
         )
 
     def get_source_artifacts_ids(self) -> list[int]:
@@ -114,34 +167,14 @@ class Sbuild(
         result.extend(self.dynamic_data.input_extra_binary_artifacts_ids)
         return result
 
-    @property
-    def chroot_name(self) -> str:
-        """Build name of required chroot."""
-        return "{}-{}".format(
-            self.data.distribution,
-            self.data.host_architecture,
-        )
-
-    @staticmethod
-    def _call_dpkg_architecture():  # pragma: no cover
-        return (
-            subprocess.check_output(["dpkg", "--print-architecture"])
-            .decode("utf-8")
-            .strip()
-        )
-
     @classmethod
-    def analyze_worker(cls):
+    def analyze_worker(cls) -> dict[str, Any]:
         """Report metadata for this task on this worker."""
         metadata = super().analyze_worker()
 
         available_key = cls.prefix_with_task_name("available")
         metadata[available_key] = debusine.utils.is_command_available("sbuild")
 
-        if debusine.utils.is_command_available("schroot"):
-            chroots_key = cls.prefix_with_task_name("chroots")
-            metadata[chroots_key] = cls._list_chroots()
-
         return metadata
 
     def can_run_on(self, worker_metadata: dict[str, Any]) -> bool:
@@ -153,69 +186,15 @@ class Sbuild(
         if not worker_metadata.get(available_key, False):
             return False
 
-        if self.backend == "schroot":
-            chroot_key = self.prefix_with_task_name("chroots")
-            if self.chroot_name not in worker_metadata.get(chroot_key, []):
+        executor_available_key = f"executor:{self.backend}:available"
+        if not worker_metadata.get(executor_available_key, False):
+            return False
+        if self.backend != "unshare":
+            if not worker_metadata.get("autopkgtest:available", False):
                 return False
-        else:
-            executor_available_key = f"executor:{self.backend}:available"
-            if not worker_metadata.get(executor_available_key, False):
-                return False
-            if self.backend != "unshare":
-                if not worker_metadata.get("autopkgtest:available", False):
-                    return False
 
         return True
 
-    @staticmethod
-    def _call_schroot_list():  # pragma: no cover
-        return (
-            subprocess.check_output(["schroot", "--list"])
-            .decode("utf-8")
-            .strip()
-        )
-
-    @classmethod
-    def _list_chroots(cls) -> list[str]:
-        """
-        Provide support for finding available chroots.
-
-        Ensure that aliases are supported as the DSC may explicitly refer to
-        <codename>-security (or -backports) etc.
-
-        Return the list of detected chroots.
-        """
-        chroots = []
-        output = cls._call_schroot_list()
-        for line in output.split("\n"):
-            if line.startswith("chroot:") and line.endswith("-sbuild"):
-                chroots.append(line[7:-7])
-        return chroots
-
-    def _update_chroots_list(self):
-        """
-        Populate the self.chroots list, if the list is empty.
-
-        No return value, this is a find, not a get.
-        """
-        if self.chroots is not None:
-            return
-        self.chroots = self._list_chroots()
-
-    def _verify_schroot(self):
-        """Verify a suitable schroot exists."""
-        self._update_chroots_list()
-
-        if not self.chroots:
-            self.logger.error("No sbuild chroots found")
-            return False
-
-        if self.chroot_name in self.chroots:
-            return True
-
-        self.logger.error("No suitable chroot found for %s", self.chroot_name)
-        return False
-
     def fetch_input(self, destination: Path) -> bool:
         """Download the source artifact."""
         if not self.debusine:
@@ -247,24 +226,42 @@ class Sbuild(
             if artifact.category in (
                 ArtifactCategory.BINARY_PACKAGE,
                 ArtifactCategory.BINARY_PACKAGES,
+                ArtifactCategory.UPLOAD,
             ):
                 for file in artifact.files:
-                    self._extra_packages.append(destination / file)
+                    if file.endswith(".deb"):
+                        self._extra_packages.append(destination / file)
             else:
                 self.append_to_log_file(
                     "fetch_input.log",
                     [
                         f"input.extra_binary_artifacts includes artifact "
                         f"{artifact_id}, a {artifact.category}, not the "
-                        f"expected {ArtifactCategory.BINARY_PACKAGE} or "
-                        f"{ArtifactCategory.BINARY_PACKAGES}."
+                        f"expected {ArtifactCategory.BINARY_PACKAGE}, "
+                        f"{ArtifactCategory.BINARY_PACKAGES}, or "
+                        f"{ArtifactCategory.UPLOAD}."
                     ],
                 )
                 return False
             self.fetch_artifact(artifact_id, destination)
 
+        self._prepare_executor()
+
+        self.write_extra_repository_config(
+            codename=self._environment_codename(), destination=destination
+        )
+
         return True
 
+    def _environment_codename(self) -> str:
+        """Return the codename of the environment we're building in."""
+        if self.executor is None:
+            raise ValueError(
+                "self.executor not set, call _prepare_executor() first"
+            )
+
+        return cast(str, self.executor.system_image.data["codename"])
+
     def _cmdline(self) -> list[str]:
         """
         Build the sbuild command line.
@@ -273,8 +270,10 @@ class Sbuild(
         """
         cmd = [
             self.builder,
-            "--no-clean",
             "--purge-deps=never",
+            # Use a higher-level workflow instead to run both sbuild and
+            # lintian, such as debian_pipeline.
+            "--no-run-lintian",
         ]
         if "any" in self.data.build_components:
             cmd.append("--arch-any")
@@ -290,52 +289,9 @@ class Sbuild(
             cmd.append("--no-source")
 
         cmd.append("--arch=" + self.data.host_architecture)
-
-        if self.backend == "schroot":
-            # Using cast because SbuildTaskData validators enforce that
-            # distribution is not None if backend is schroot
-            cmd.append("--dist=" + cast(str, self.data.distribution))
-        else:
-            # set in configure_for_execution
-            if self.executor is None:
-                raise AssertionError("self.executor not set")
-            distribution = self.executor.system_image.data["codename"]
-            cmd.append(f"--dist={distribution}")
-            if self.backend == "unshare":
-                cmd += [
-                    "--chroot-mode=unshare",
-                    f"--chroot={self.executor.image_name()}",
-                    # Remove any dangling resolv.conf symlink (from
-                    # systemd-resolved installed in the environment, #1071736)
-                    "--chroot-setup-commands=rm -f /etc/resolv.conf",
-                ]
-            else:
-                virt_server = self.executor.autopkgtest_virt_server()
-                cmd += [
-                    "--chroot-mode=autopkgtest",
-                    f"--autopkgtest-virt-server={virt_server}",
-                ] + [
-                    f"--autopkgtest-virt-server-opt={opt.replace('%', '%%')}"
-                    for opt in self.executor.autopkgtest_virt_args()
-                ]
-
-        # binNMU
-        if self.data.binnmu is not None:
-            cmd.append(f"--make-binNMU={self.data.binnmu.changelog}")
-            cmd.append(f"--append-to-version={self.data.binnmu.suffix}")
-            cmd.append("--binNMU=0")
-            if self.data.binnmu.timestamp is not None:
-                rfc5322date = email.utils.format_datetime(
-                    self.data.binnmu.timestamp
-                )
-                cmd.append(f"--binNMU-timestamp={rfc5322date}")
-            maintainer: str | None
-            if self.data.binnmu.maintainer is not None:
-                maintainer = str(self.data.binnmu.maintainer)
-            else:
-                assert self.dynamic_data
-                maintainer = self.dynamic_data.binnmu_maintainer
-            cmd.append(f"--maintainer={maintainer}")
+        cmd += self._cmdline_backend()
+        cmd += self._cmdline_extra_repository()
+        cmd += self._cmdline_binnmu()
 
         if self.data.build_profiles:
             cmd.append(f"--profiles={','.join(self.data.build_profiles)}")
@@ -350,14 +306,91 @@ class Sbuild(
 
         return cmd
 
-    def _cmd_env(self) -> dict[str, str] | None:
-        """Set DEB_BUILD_OPTIONS for build profiles."""
-        build_options = [
+    def _cmdline_backend(self) -> list[str]:
+        """Generate command line arguments for the backend and environment."""
+        distribution = self._environment_codename()
+        args: list[str] = [f"--dist={distribution}"]
+
+        assert self.executor
+        if self.backend == "unshare":
+            args += [
+                "--chroot-mode=unshare",
+                f"--chroot={self.executor.image_name()}",
+                # Remove any dangling resolv.conf symlink (from
+                # systemd-resolved installed in the environment, #1071736)
+                "--chroot-setup-commands=rm -f /etc/resolv.conf",
+            ]
+        else:
+            virt_server = self.executor.autopkgtest_virt_server()
+            args += [
+                "--chroot-mode=autopkgtest",
+                f"--autopkgtest-virt-server={virt_server}",
+            ] + [
+                f"--autopkgtest-virt-server-opt={opt.replace('%', '%%')}"
+                for opt in self.executor.autopkgtest_virt_args()
+            ]
+        return args
+
+    def _cmdline_extra_repository(self) -> list[str]:
+        """Generate the command line arguments for extra_repositories."""
+        args: list[str] = []
+
+        if self.supports_deb822_sources(self._environment_codename()):
+            # sbuild has no native deb822 support, yet: #1089735
+            for path in self.extra_repository_sources:
+                args.append(
+                    f"--pre-build-commands=cat {path} | "
+                    f"%SBUILD_CHROOT_EXEC sh -c 'cat > "
+                    f"/etc/apt/sources.list.d/{path.name}'"
+                )
+        else:
+            for source in self.iter_oneline_sources():
+                args.append(f"--extra-repository={source}")
+
+        # apt < 2.3.10 has no support for keys embedded in Signed-By
+        for path in self.extra_repository_keys:
+            args.append(
+                f"--pre-build-commands=cat {path} | "
+                f"%SBUILD_CHROOT_EXEC sh -c 'mkdir -p /etc/apt/keyrings && "
+                f"cat > /etc/apt/keyrings/{path.name}'"
+            )
+
+        return args
+
+    def _cmdline_binnmu(self) -> list[str]:
+        """Generate the command line arguments for binnmu."""
+        if self.data.binnmu is None:
+            return []
+
+        args: list[str] = [
+            f"--make-binNMU={self.data.binnmu.changelog}",
+            f"--append-to-version={self.data.binnmu.suffix}",
+            "--binNMU=0",
+        ]
+        if self.data.binnmu.timestamp is not None:
+            rfc5322date = email.utils.format_datetime(
+                self.data.binnmu.timestamp
+            )
+            args.append(f"--binNMU-timestamp={rfc5322date}")
+        maintainer: str | None
+        if self.data.binnmu.maintainer is not None:
+            maintainer = str(self.data.binnmu.maintainer)
+        else:
+            assert self.dynamic_data
+            maintainer = self.dynamic_data.binnmu_maintainer
+        args.append(f"--maintainer={maintainer}")
+        return args
+
+    def _build_options(self) -> list[str]:
+        return [
             profile
             for profile in self.data.build_profiles or []
             if profile in ("nocheck", "nodoc")
         ]
-        if build_options:
+
+    def _cmd_env(self) -> dict[str, str] | None:
+        """Set DEB_BUILD_OPTIONS for build profiles."""
+        if build_options := self._build_options():
             env = dict(os.environ)
             env["DEB_BUILD_OPTIONS"] = " ".join(
                 os.environ.get("DEB_BUILD_OPTIONS", "").split() + build_options
@@ -365,19 +398,34 @@ class Sbuild(
             return env
         return None
 
-    def execute(self) -> bool:
+    def task_succeeded(
+        self,
+        returncode: int | None,  # noqa: U100
+        execute_directory: Path,  # noqa: U100
+    ) -> bool:
         """
-        Verify task can be executed and super().execute().
+        Check whether the task succeeded.
 
-        :raises: TaskConfigError.
-        """  # noqa: D402
-        if self.backend == "schroot" and not self._verify_schroot():
-            raise TaskConfigError(
-                "No suitable schroot for"
-                f" {self.data.distribution}-{self.data.host_architecture}"
-            )
+        `sbuild` returns 0 for "Status: skipped" (i.e. host architecture not
+        supported by source package); we consider that a failure.
+        """
+        if not super().task_succeeded(returncode, execute_directory):
+            return False
 
-        return super().execute()
+        build_log_path = debusine.utils.find_file_suffixes(
+            execute_directory, [".build"]
+        )
+        if build_log_path is None:
+            return False
+
+        status = ""
+        # Reading the whole build log is inefficient, but shouldn't usually
+        # be too bad in practice.
+        with open(build_log_path, errors="replace") as f:
+            for line in f:
+                if line.startswith("Status: "):
+                    status = line[len("Status: ") :].rstrip("\n")
+        return status == "successful"
 
     @staticmethod
     def _extract_dose3_explanation(
@@ -565,7 +613,7 @@ class Sbuild(
         remote_binary_upload: RemoteArtifact | None,
         remote_binary_packages: list[RemoteArtifact],
         remote_signing_input: RemoteArtifact | None,
-    ):
+    ) -> None:
         if not self.debusine:
             raise AssertionError("self.debusine not set")
 
@@ -574,24 +622,26 @@ class Sbuild(
                 self.debusine.relation_create(
                     remote_binary_package.id,
                     source_artifact_id,
-                    "built-using",
+                    RelationType.BUILT_USING,
                 )
 
             if remote_build_log is not None:
                 self.debusine.relation_create(
-                    remote_build_log.id, remote_binary_package.id, "relates-to"
+                    remote_build_log.id,
+                    remote_binary_package.id,
+                    RelationType.RELATES_TO,
                 )
 
             if remote_binary_upload is not None:
                 self.debusine.relation_create(
                     remote_binary_upload.id,
                     remote_binary_package.id,
-                    "extends",
+                    RelationType.EXTENDS,
                 )
                 self.debusine.relation_create(
                     remote_binary_upload.id,
                     remote_binary_package.id,
-                    "relates-to",
+                    RelationType.RELATES_TO,
                 )
 
         if (
@@ -599,7 +649,9 @@ class Sbuild(
             and remote_signing_input is not None
         ):
             self.debusine.relation_create(
-                remote_signing_input.id, remote_binary_upload.id, "relates-to"
+                remote_signing_input.id,
+                remote_binary_upload.id,
+                RelationType.RELATES_TO,
             )
 
     def configure_for_execution(
@@ -617,9 +669,6 @@ class Sbuild(
             )
             return False
 
-        if self.backend != "schroot":
-            self._prepare_executor()
-
         return True
 
     def upload_artifacts(
@@ -649,7 +698,7 @@ class Sbuild(
                     self.debusine.relation_create(
                         remote_build_log.id,
                         source_artifact_id,
-                        "relates-to",
+                        RelationType.RELATES_TO,
                     )
 
             if execution_success:
@@ -674,6 +723,8 @@ class Sbuild(
 
     def get_label(self) -> str:
         """Return the task label."""
-        # TODO: copy the source package information in dynamic task data and
-        # use them here if available
-        return "build a package"
+        return (
+            f"build {subject}"
+            if (subject := self.get_subject())
+            else "build a package"
+        )
