@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -26,27 +26,35 @@ from debusine.artifacts import LocalArtifact, SourcePackage, Upload
 from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
-    KeyPurpose,
+    DebianBinaryPackage,
+    DebusineSigningOutput,
+    EmptyArtifactData,
 )
 from debusine.artifacts.playground import ArtifactPlayground
+from debusine.assets import KeyPurpose
 from debusine.client.models import (
     ArtifactResponse,
     RelationsResponse,
     RemoteArtifact,
 )
-from debusine.tasks import AssembleSignedSource
+from debusine.tasks import AssembleSignedSource, TaskConfigError
 from debusine.tasks.assemble_signed_source import AssembleError
 from debusine.tasks.models import (
     AssembleSignedSourceDynamicData,
     LookupMultiple,
     WorkerType,
 )
+from debusine.tasks.server import ArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
     FakeTaskDatabase,
 )
 from debusine.test import TestCase
-from debusine.test.utils import create_artifact_response, create_file_response
+from debusine.test.test_utils import (
+    create_artifact_response,
+    create_file_response,
+    create_system_tarball_data,
+)
 
 
 class AssembleSignedSourceTests(
@@ -116,23 +124,90 @@ class AssembleSignedSourceTests(
                     "debian/match:codename=bookworm:format=tarball:"
                     "backend=unshare",
                     CollectionCategory.ENVIRONMENTS,
-                ): 1,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
                 # template
-                (11, None): 11,
+                (11, None): ArtifactInfo(
+                    id=11,
+                    category=ArtifactCategory.BINARY_PACKAGE,
+                    data=DebianBinaryPackage(
+                        srcpkg_name="template",
+                        srcpkg_version="1.0",
+                        deb_fields={"Package": "hello"},
+                        deb_control_files=[],
+                    ),
+                ),
             },
             multiple_lookups={
                 # signed
-                (LookupMultiple.parse_obj([12]), None): [12]
+                (LookupMultiple.parse_obj([12]), None): [
+                    ArtifactInfo(
+                        id=12,
+                        category=ArtifactCategory.SIGNING_OUTPUT,
+                        data=DebusineSigningOutput(
+                            purpose=KeyPurpose.UEFI, fingerprint="", results=[]
+                        ),
+                    )
+                ]
             },
         )
 
         self.assertEqual(
             self.task.compute_dynamic_data(task_db),
             AssembleSignedSourceDynamicData(
-                environment_id=1, template_id=11, signed_ids=[12]
+                environment_id=1,
+                template_id=11,
+                signed_ids=[12],
+                subject="hello",
+                runtime_context=None,
             ),
         )
 
+    def test_compute_dynamic_data_subject_raise_task_config_error(self) -> None:
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:format=tarball:"
+                    "backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # template
+                (11, None): ArtifactInfo(
+                    id=11,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=EmptyArtifactData(),
+                ),
+            },
+            multiple_lookups={
+                # signed
+                (LookupMultiple.parse_obj([12]), None): [
+                    ArtifactInfo(
+                        id=12,
+                        category=ArtifactCategory.SIGNING_OUTPUT,
+                        data=DebusineSigningOutput(
+                            purpose=KeyPurpose.UEFI, fingerprint="", results=[]
+                        ),
+                    )
+                ]
+            },
+        )
+
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            r"^template: unexpected artifact category: "
+            r"'debian:source-package'. Valid categories: "
+            r"\['debian:binary-package'\]$",
+        ):
+            self.task.compute_dynamic_data(task_db)
+
     def test_fetch_input_template_wrong_category(self) -> None:
         """fetch_input checks the category of the template artifact."""
         self.task.work_request_id = 147
@@ -1100,7 +1175,9 @@ class AssembleSignedSourceTests(
         debusine_mock.upload_artifact.side_effect = upload_artifact
 
         def run(
-            args: list[str], run_as_root: bool = False, **kwargs  # noqa: U100
+            args: list[str],
+            run_as_root: bool = False,  # noqa: U100
+            **kwargs: Any,
         ) -> subprocess.CompletedProcess[AnyStr]:
             if args[0] == "apt-get":
                 return subprocess.CompletedProcess(args, 0)
