@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -14,7 +14,6 @@ import shlex
 import textwrap
 from pathlib import Path
 from typing import cast
-from unittest import TestCase
 from unittest.mock import MagicMock, call, create_autospec, patch
 
 try:
@@ -24,12 +23,19 @@ except ImportError:
 
 from debusine.artifacts.local_artifact import AutopkgtestArtifact
 from debusine.artifacts.models import (
+    ArtifactCategory,
     CollectionCategory,
     DebianAutopkgtest,
     DebianAutopkgtestResult,
 )
 from debusine.artifacts.models import DebianAutopkgtestResultStatus as rs
-from debusine.artifacts.models import DebianAutopkgtestSource
+from debusine.artifacts.models import (
+    DebianAutopkgtestSource,
+    DebianBinaryPackage,
+    DebianBinaryPackages,
+    DebianSourcePackage,
+    DebianUpload,
+)
 from debusine.client.debusine import Debusine
 from debusine.client.models import (
     ArtifactResponse,
@@ -46,16 +52,16 @@ from debusine.tasks.models import (
     LookupMultiple,
     WorkerType,
 )
+from debusine.tasks.server import ArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
     FakeTaskDatabase,
 )
-from debusine.test import TestHelpersMixin
+from debusine.test import TestCase
+from debusine.test.test_utils import create_system_tarball_data
 
 
-class AutopkgtestTests(
-    TestHelpersMixin, ExternalTaskHelperMixin[Autopkgtest], TestCase
-):
+class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
     """Tests for Autopkgtest."""
 
     SAMPLE_TASK_DATA = {
@@ -332,8 +338,18 @@ class AutopkgtestTests(
         self.assertEqual(self.task.data.debug_level, 0)
 
     def test_configure_extra_apt_sources(self) -> None:
-        """Field "extra_apt_sources" is allowed."""
-        self.configure_task(override={"extra_apt_sources": ["A", "B"]})
+        """Field "extra_repositories" is allowed."""
+        self.configure_task(
+            override={
+                "extra_repositories": [
+                    {
+                        "url": "http://example.net",
+                        "suite": "bookworm",
+                        "components": ["main"],
+                    }
+                ]
+            }
+        )
 
     def test_configure_use_packages_from_base_repository(self) -> None:
         """Field "use_package_from_base_repository" is allowed."""
@@ -501,6 +517,28 @@ class AutopkgtestTests(
             )
         )
 
+    def create_binary_packages_data(
+        self, srcpkg_name: str
+    ) -> DebianBinaryPackages:
+        """Create sample `debian:binary-packages` artifact data."""
+        return DebianBinaryPackages(
+            srcpkg_name=srcpkg_name,
+            srcpkg_version="1.0",
+            version="1.0",
+            architecture="amd64",
+            packages=[],
+        )
+
+    def create_upload_data(self, name: str) -> DebianUpload:
+        """Create sample `debian:upload` artifact data."""
+        return DebianUpload(
+            type="dpkg",
+            changes_fields={
+                "Architecture": "amd64",
+                "Files": [{"name": f"{name}_1.0_amd64.deb"}],
+            },
+        )
+
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
         binary_artifacts_lookup = LookupMultiple.parse_obj([10, 11])
@@ -511,13 +549,34 @@ class AutopkgtestTests(
                     "debian/match:codename=bookworm:architecture=amd64:"
                     "format=tarball:backend=unshare",
                     CollectionCategory.ENVIRONMENTS,
-                ): 1,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
                 # input.source_artifact
-                (5, None): 5,
+                (5, None): ArtifactInfo(
+                    id=5,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianSourcePackage(
+                        name="hello", version="1.0", type="dpkg", dsc_fields={}
+                    ),
+                ),
             },
             multiple_lookups={
                 # input.binary_artifacts
-                (binary_artifacts_lookup, None): [10, 11]
+                (binary_artifacts_lookup, None): [
+                    ArtifactInfo(
+                        id=10,
+                        category=ArtifactCategory.BINARY_PACKAGES,
+                        data=self.create_binary_packages_data("hello"),
+                    ),
+                    ArtifactInfo(
+                        id=11,
+                        category=ArtifactCategory.UPLOAD,
+                        data=self.create_upload_data("hello"),
+                    ),
+                ],
             },
         )
 
@@ -528,6 +587,9 @@ class AutopkgtestTests(
                 input_source_artifact_id=5,
                 input_binary_artifacts_ids=[10, 11],
                 input_context_artifacts_ids=[],
+                subject="hello",
+                runtime_context="amd64:unshare",
+                configuration_context="sid",
             ),
         )
 
@@ -544,15 +606,47 @@ class AutopkgtestTests(
                     "debian/match:codename=bookworm:architecture=amd64:"
                     "format=tarball:backend=unshare",
                     CollectionCategory.ENVIRONMENTS,
-                ): 1,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
                 # input.source_artifact
-                (5, None): 5,
+                (5, None): ArtifactInfo(
+                    id=5,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianSourcePackage(
+                        name="hello", version="1.0", type="dpkg", dsc_fields={}
+                    ),
+                ),
             },
             multiple_lookups={
                 # input.binary_artifacts
-                (binary_artifacts_lookup, None): [10, 11],
+                (binary_artifacts_lookup, None): [
+                    ArtifactInfo(
+                        id=10,
+                        category=ArtifactCategory.BINARY_PACKAGES,
+                        data=self.create_binary_packages_data("hello"),
+                    ),
+                    ArtifactInfo(
+                        id=11,
+                        category=ArtifactCategory.UPLOAD,
+                        data=self.create_upload_data("hello"),
+                    ),
+                ],
                 # input.context_artifacts
-                (self.task.data.input.context_artifacts, None): [2, 3],
+                (self.task.data.input.context_artifacts, None): [
+                    ArtifactInfo(
+                        id=2,
+                        category=ArtifactCategory.BINARY_PACKAGES,
+                        data=self.create_binary_packages_data("context"),
+                    ),
+                    ArtifactInfo(
+                        id=3,
+                        category=ArtifactCategory.UPLOAD,
+                        data=self.create_upload_data("context"),
+                    ),
+                ],
             },
         )
 
@@ -563,9 +657,84 @@ class AutopkgtestTests(
                 input_source_artifact_id=5,
                 input_binary_artifacts_ids=[10, 11],
                 input_context_artifacts_ids=[2, 3],
+                subject="hello",
+                runtime_context="amd64:unshare",
+                configuration_context="sid",
             ),
         )
 
+    def test_compute_dynamic_data_raise_task_config_error(self) -> None:
+        """
+        Test compute_dynamic_data raise TaskConfigError.
+
+        source_artifact's category is unexpected.
+        """
+        binary_artifacts_lookup = LookupMultiple.parse_obj([10, 11])
+        self.task.data.input.context_artifacts = LookupMultiple.parse_obj(
+            {"collection": "bookworm@debian:suite"}
+        )
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # input.source_artifact
+                (5, None): ArtifactInfo(
+                    id=5,
+                    category=ArtifactCategory.BINARY_PACKAGE,
+                    data=DebianBinaryPackage(
+                        srcpkg_name="hello",
+                        srcpkg_version="1.0",
+                        deb_fields={},
+                        deb_control_files=[],
+                    ),
+                ),
+            },
+            multiple_lookups={
+                # input.binary_artifacts
+                (binary_artifacts_lookup, None): [
+                    ArtifactInfo(
+                        id=10,
+                        category=ArtifactCategory.BINARY_PACKAGES,
+                        data=self.create_binary_packages_data("hello"),
+                    ),
+                    ArtifactInfo(
+                        id=11,
+                        category=ArtifactCategory.UPLOAD,
+                        data=self.create_upload_data("hello"),
+                    ),
+                ],
+                # input.context_artifacts
+                (self.task.data.input.context_artifacts, None): [
+                    ArtifactInfo(
+                        id=2,
+                        category=ArtifactCategory.BINARY_PACKAGES,
+                        data=self.create_binary_packages_data("context"),
+                    ),
+                    ArtifactInfo(
+                        id=3,
+                        category=ArtifactCategory.UPLOAD,
+                        data=self.create_upload_data("context"),
+                    ),
+                ],
+            },
+        )
+
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            "^input.source_artifact: unexpected artifact "
+            "category: 'debian:binary-package'. Valid categories: "
+            r"\['debian:source-package', 'debian:upload'\]$",
+        ):
+            self.task.compute_dynamic_data(task_db)
+
     def test_cmdline_executor_not_set(self) -> None:
         """Raise AssertionError: self.executor is not set."""
         msg = (
@@ -620,21 +789,54 @@ class AutopkgtestTests(
 
         self.assertIn("-dd", cmdline)
 
-    def test_cmdline_extra_apt_sources(self) -> None:
-        """Command line include "--add-apt-source=XXX"."""
+    def test_cmdline_extra_repositories_jessie(self) -> None:
+        """Ensure the --add-apt-source= parameter is passed to autopkgtest."""
+        self.configure_task(
+            override={
+                "extra_repositories": [
+                    {
+                        "url": "http://example.com/",
+                        "suite": "unstable",
+                        "components": ["main"],
+                    },
+                    {
+                        "url": "http://example.net/",
+                        "suite": "flat/",
+                    },
+                ],
+            }
+        )
         self.mock_executor()
-        extra_apt_sources = [
-            "deb https://deb.debian.org/debian bookworm main non-free "
-            "non-free-firmware contrib",
-            "deb https://deb.debian.org/debian-security bookworm-security "
-            "main non-free contrib",
-        ]
-        self.task.data.extra_apt_sources = extra_apt_sources
-
+        assert self.task.executor
+        self.task.executor.system_image.data["codename"] = "jessie"
         cmdline = self.task._cmdline()
 
-        for extra_apt_source in extra_apt_sources:
-            self.assertIn(f"--add-apt-source={extra_apt_source}", cmdline)
+        self.assertIn(
+            "--add-apt-source=deb http://example.com/ unstable main", cmdline
+        )
+        self.assertIn("--add-apt-source=deb http://example.net/ flat/", cmdline)
+
+    def test_cmdline_extra_repositories_trixie(self) -> None:
+        """Ensure the --copy= parameter is passed to autopkgtest."""
+        debusine_mock = self.mock_debusine()
+        self.mock_image_download(debusine_mock)
+        self.task._prepare_executor()
+        self.task.extra_repository_sources = [
+            Path("/tmp/extra_repository_0.sources")
+        ]
+        self.task.extra_repository_keys = [
+            Path("/tmp/key.asc"),
+        ]
+        cmdline = self.task._cmdline()
+
+        self.assertIn(
+            (
+                "--copy=/tmp/extra_repository_0.sources:"
+                "/etc/apt/sources.list.d/extra_repository_0.sources"
+            ),
+            cmdline,
+        )
+        self.assertIn("--copy=/tmp/key.asc:/etc/apt/keyrings/key.asc", cmdline)
 
     def test_cmdline_apt_default_release_not_included(self) -> None:
         """Command line NOT include "--apt-default-release={distribution}."""
@@ -786,9 +988,23 @@ class AutopkgtestTests(
 
         self.assertEqual(self.task.executor.backend_name, "unshare")
 
+    def test_environment_codename_executor(self) -> None:
+        """Test _environment_codename gets the executor distribution."""
+        self.mock_executor()
+        self.assertEqual(self.task._environment_codename(), "bookworm")
+
+    def test_environment_codename_executor_unconfigured(self) -> None:
+        """Test _environment_codename raises ValueError without executor."""
+        with self.assertRaisesRegex(
+            ValueError,
+            r"self\.executor not set, call _prepare_executor\(\) first",
+        ):
+            self.task._environment_codename()
+
     def test_fetch_input_no_context_artifacts(self) -> None:
         """Download "source_artifact" and "binary_artifacts"."""
         debusine_mock = self.mock_debusine()
+        self.mock_image_download(debusine_mock)
         workspace = "Testing"
         debusine_mock.download_artifact.return_value = ArtifactResponse(
             id=10,
@@ -856,6 +1072,7 @@ class AutopkgtestTests(
     def test_fetch_input_context_artifacts(self) -> None:
         """Download the "context_artifacts"."""
         debusine_mock = self.mock_debusine()
+        self.mock_image_download(debusine_mock)
 
         destination = self.create_temporary_directory()
 
@@ -874,8 +1091,7 @@ class AutopkgtestTests(
 
     def test_configure_for_execution(self) -> None:
         """Autopkgtest configure_for_execution() return True."""
-        debusine_mock = self.mock_debusine()
-        self.mock_image_download(debusine_mock)
+        self.mock_executor()
 
         directory = self.create_temporary_directory()
 
@@ -1100,6 +1316,25 @@ class AutopkgtestTests(
             [],
         )
 
-    def test_label(self) -> None:
-        """Test get_label."""
+    def test_get_label_dynamic_data_is_none(self) -> None:
+        """Test get_label if dynamic_data.subject is None."""
         self.assertEqual(self.task.get_label(), "autopkgtest")
+
+    def test_get_label_dynamic_data_subject_is_none(self) -> None:
+        """Test get_label if dynamic_data is None."""
+        self.task.dynamic_data = AutopkgtestDynamicData(
+            environment_id=1,
+            input_source_artifact_id=2,
+            input_binary_artifacts_ids=[],
+        )
+        self.assertEqual(self.task.get_label(), "autopkgtest")
+
+    def test_get_label_dynamic_data_subject_is_hello(self) -> None:
+        """Test get_label if dynamic_data.subject is set."""
+        self.task.dynamic_data = AutopkgtestDynamicData(
+            environment_id=1,
+            input_source_artifact_id=2,
+            input_binary_artifacts_ids=[],
+            subject="hello",
+        )
+        self.assertEqual(self.task.get_label(), "autopkgtest hello")
