@@ -1,4 +1,4 @@
-# Copyright 2021-2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -10,20 +10,17 @@
 """Unit tests for the SbuildValidatorMixin class."""
 
 from pathlib import Path
-from unittest import TestCase
 
-from debusine.tasks import BaseExternalTask
 from debusine.tasks.models import BaseDynamicTaskData, BaseTaskData
 from debusine.tasks.sbuild_validator_mixin import SbuildValidatorMixin
-from debusine.tasks.tests.helper_mixin import TestTaskMixin
-from debusine.test import TestHelpersMixin
+from debusine.tasks.tests.helper_mixin import TestBaseExternalTask
+from debusine.test import TestCase
 from debusine.utils import calculate_hash
 
 
 class TaskWithSbuildValidatorMixin(
-    TestTaskMixin,
     SbuildValidatorMixin,
-    BaseExternalTask[BaseTaskData, BaseDynamicTaskData],
+    TestBaseExternalTask[BaseTaskData, BaseDynamicTaskData],
 ):
     """Minimum class to test SbuildValidatorMixin."""
 
@@ -32,7 +29,7 @@ class TaskWithSbuildValidatorMixin(
         raise NotImplementedError()
 
 
-class SbuildValidatorMixinTests(TestHelpersMixin, TestCase):
+class SbuildValidatorMixinTests(TestCase):
     """Tests for SbuildValidatorMixin class."""
 
     def setUp(self) -> None:
