@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -11,10 +11,12 @@
 import functools
 import shutil
 import tempfile
+from collections.abc import Callable
 from datetime import timedelta
 from pathlib import Path
 from typing import Any, Generic, TYPE_CHECKING, TypeVar, cast
 
+from django.contrib.auth.models import AnonymousUser
 from django.core.exceptions import ValidationError
 from django.db import transaction
 from django.forms import (
@@ -30,9 +32,12 @@ from django.forms import (
     IntegerField,
     ModelChoiceField,
     ModelForm,
+    MultipleChoiceField,
     Textarea,
 )
 
+from debusine.db.context import context
+
 try:
     import pydantic.v1 as pydantic
 except ImportError:
@@ -48,13 +53,15 @@ from debusine.db.models import (
     CollectionItem,
     File,
     FileInArtifact,
+    Group,
     Token,
+    User,
     WorkRequest,
+    WorkflowTemplate,
     Workspace,
 )
 from debusine.tasks import BaseTask
 from debusine.tasks.models import TaskTypes
-from debusine.web.utils import ui_prototype
 
 if TYPE_CHECKING:
     ModelFormBase = ModelForm
@@ -94,12 +101,14 @@ class DictWithCallback(dict[_KT, _VT], Generic[_KT, _VT]):
     by the callback.
     """
 
-    def __init__(self, callback, *args, **kwargs):
+    def __init__(
+        self, callback: Callable[[_VT], None], *args: Any, **kwargs: Any
+    ) -> None:
         """Create the object."""
         self._callback = callback
         super().__init__(*args, **kwargs)
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, key: _KT, value: _VT) -> None:
         """Call self._callback(value) and set the value."""
         self._callback(value)
         super().__setitem__(key, value)
@@ -112,18 +121,18 @@ class BootstrapMixin:
     This mixin is intended to be used in combination with Django's form classes.
     """
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         """Initialize the mixin."""
         super().__init__(*args, **kwargs)
 
         self._adjust_bootstrap_classes_all_fields()
 
-        self.fields = DictWithCallback[str, Field](
+        self.fields: dict[str, Field] = DictWithCallback(
             self._adjust_bootstrap_for_field, self.fields
         )
 
     @staticmethod
-    def _adjust_bootstrap_for_field(field: Field):
+    def _adjust_bootstrap_for_field(field: Field) -> None:
         """Adjust the CSS class for a field."""
         existing_class = field.widget.attrs.get("class", "")
         bootstrap_class = None
@@ -137,13 +146,15 @@ class BootstrapMixin:
             bootstrap_class = "form-select"
         elif isinstance(field, (CharField, FileField, DateTimeField)):
             bootstrap_class = "form-control"
+        elif isinstance(field, BooleanField):
+            bootstrap_class = "form-check-input"
 
         if bootstrap_class and bootstrap_class not in existing_class.split():
             field.widget.attrs["class"] = (
                 f"{existing_class} {bootstrap_class}".strip()
             )
 
-    def _adjust_bootstrap_classes_all_fields(self):
+    def _adjust_bootstrap_classes_all_fields(self) -> None:
         """Adjust the CSS classes of form fields to be Bootstrap-compatible."""
         for field in self.fields.values():
             self._adjust_bootstrap_for_field(field)
@@ -152,7 +163,7 @@ class BootstrapMixin:
 class TokenForm(BootstrapMixin, ModelFormBase[Token]):
     """Form for creating or editing a token."""
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         """Initialize TokenForm."""
         self.user = kwargs.pop("user")
         super().__init__(*args, **kwargs)
@@ -161,7 +172,7 @@ class TokenForm(BootstrapMixin, ModelFormBase[Token]):
             # New instance (not loaded from the DB). Set defaults
             self.fields["enabled"].initial = True
 
-    def save(self, commit=True):
+    def save(self, commit: bool = True) -> Token:
         """Save TokenForm."""
         instance = super().save(commit=False)
 
@@ -184,7 +195,9 @@ class TokenForm(BootstrapMixin, ModelFormBase[Token]):
 class WorkspaceChoiceField(ModelChoiceFieldBase[Workspace]):
     """ChoiceField for the workspaces: set the label and order by name."""
 
-    def __init__(self, user, *args, **kwargs):
+    def __init__(
+        self, user: User | AnonymousUser | None, *args: Any, **kwargs: Any
+    ) -> None:
         """Set the queryset."""
         kwargs["queryset"] = Workspace.objects.order_by("name")
 
@@ -209,23 +222,27 @@ class YamlMixin:
     yaml_fields = ["task_data", "some_other_field"]
     """
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         """For each self.yaml_fields: create the CharField."""
         super().__init__(*args, **kwargs)
 
+        assert isinstance(self, BaseForm)
+        assert hasattr(self, "yaml_fields")
         for name in self.yaml_fields:
-            self.fields[name] = CharField(widget=Textarea, required=False)
-
-            # This might need to change if it's possible to edit
-            # a form loading data from the database
-            self.fields[name].value = ""
+            self.fields[name] = CharField(
+                widget=Textarea,
+                required=False,
+                # This might need to change if it's possible to edit
+                # a form loading data from the database.
+                initial="",
+            )
 
             self.__setattr__(
                 f"clean_{name}", functools.partial(self._clean_data_yaml, name)
             )
 
-    def _clean_data_yaml(self, field_name: str) -> dict[Any, Any]:
-        """Return dictionary representing the YAML input."""
+    def _clean_data_yaml(self, field_name: str) -> Any:
+        """Return object representing the YAML input."""
         data_yaml = cast(BaseForm, self).cleaned_data[field_name]
 
         try:
@@ -241,53 +258,29 @@ class WorkRequestForm(YamlMixin, BootstrapMixin, ModelFormBase[WorkRequest]):
 
     yaml_fields = ["task_data"]
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         """Initialize WorkRequestForm."""
         self.user = kwargs.pop("user")
-
-        # self.field_order = ["workspace", "task_name", "data"]
-
+        self.workspace = kwargs.pop("workspace")
         super().__init__(*args, **kwargs)
 
-        self.fields["workspace"] = WorkspaceChoiceField(user=self.user)
-        if self.user.is_superuser:
-            tasks = BaseTask.task_names(TaskTypes.WORKER) + BaseTask.task_names(
-                TaskTypes.SERVER
-            )
-        else:
-            tasks = BaseTask.worker_task_names()
+        tasks = BaseTask.task_names(TaskTypes.WORKER)
         self.fields["task_name"] = ChoiceField(
-            choices=lambda: [[name, name] for name in sorted(tasks)]
+            choices=lambda: [(name, name) for name in sorted(tasks)]
         )
 
-    def _get_task_type(self, name: str) -> TaskTypes:
-        """Infer a task type from a task name."""
-        if self.user.is_superuser and not BaseTask.is_worker_task(name):
-            return TaskTypes.SERVER
-        return TaskTypes.WORKER
-
-    def full_clean(self):
-        """Make sure task_type is filled."""
-        if "task_name" in self.data:
-            self.instance.task_type = self._get_task_type(
-                self.data["task_name"]
-            )
-        super().full_clean()
-
-    def save(self, commit=True):
+    def save(self, commit: bool = True) -> WorkRequest:
         """Save the work request."""
         instance = super().save(commit=False)
         instance.created_by = self.user
-        instance.task_type = self._get_task_type(instance.task_name)
-
+        instance.workspace = self.workspace
         if commit:
             instance.save()
-
         return instance
 
     class Meta:
         model = WorkRequest
-        fields = ["workspace", "task_name", "task_data"]
+        fields = ["task_name", "task_data"]
 
 
 class WorkRequestUnblockForm(BootstrapMixin, Form):
@@ -314,12 +307,14 @@ class MultipleFileField(FileField):
     https://docs.djangoproject.com/en/4.2/topics/http/file-uploads/#uploading-multiple-files
     """
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         """Initialize object: use MultipleFileInput() as a widget."""
         kwargs.setdefault("widget", MultipleFileInput())
         super().__init__(*args, **kwargs, allow_empty_file=True)
 
-    def clean(self, data: list[Any] | tuple[Any, ...], initial=None):
+    def clean(
+        self, data: list[Any] | tuple[Any, ...], initial: Any = None
+    ) -> list[Any]:
         """Call super().clean() for each file."""  # noqa: D402
         single_file_clean = super().clean
         return [single_file_clean(file, initial) for file in data]
@@ -340,11 +335,11 @@ class ArtifactForm(BootstrapMixin, YamlMixin, ModelFormBase[Artifact]):
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         """Initialize object."""
         self.user = kwargs.pop("user")
+        self.workspace = kwargs.pop("workspace")
 
         super().__init__(*args, **kwargs)
 
-        self.fields["workspace"] = WorkspaceChoiceField(user=self.user)
-
+        assert isinstance(self.fields["category"], ChoiceField)
         self.fields["category"].choices = [
             (artifact_category, artifact_category)
             for artifact_category in sorted(LocalArtifact.artifact_categories())
@@ -352,13 +347,16 @@ class ArtifactForm(BootstrapMixin, YamlMixin, ModelFormBase[Artifact]):
 
         # Populated on clean(), used on save()
         self._local_files: dict[str, Path] = {}
-        self._temporary_directory = None
+        self._temporary_directory: tempfile.TemporaryDirectory[str] | None = (
+            None
+        )
 
     @transaction.atomic
-    def save(self, commit=True):
+    def save(self, commit: bool = True) -> Artifact:
         """Create the artifact."""
         instance = super().save(commit=False)
         instance.created_by = self.user
+        instance.workspace = self.workspace
         if self.cleaned_data["expiration_delay_in_days"] is not None:
             instance.expiration_delay = timedelta(
                 days=self.cleaned_data["expiration_delay_in_days"]
@@ -372,8 +370,8 @@ class ArtifactForm(BootstrapMixin, YamlMixin, ModelFormBase[Artifact]):
                 # Add file to the store
                 file_obj = File.from_local_path(local_file_path)
 
-                file_backend = (
-                    instance.workspace.default_file_store.get_backend_object()
+                file_backend = instance.workspace.scope.upload_file_backend(
+                    file_obj
                 )
                 file_backend.add_file(local_file_path, fileobj=file_obj)
 
@@ -388,13 +386,14 @@ class ArtifactForm(BootstrapMixin, YamlMixin, ModelFormBase[Artifact]):
 
         return instance
 
-    def clean(self):
+    def clean(self) -> dict[str, Any]:
         """
         Create a LocalArtifact model and validate it.
 
         :raise ValidationError: if the LocalArtifact model is not valid.
         """
         cleaned_data = super().clean()
+        assert cleaned_data is not None
 
         artifact_category = cleaned_data["category"]
 
@@ -433,11 +432,11 @@ class ArtifactForm(BootstrapMixin, YamlMixin, ModelFormBase[Artifact]):
                 # need to add a mapping or add errors via
                 # self.add_error(None, ...) which adds the errors on the
                 # top of the form.
-                self.add_error(field_name, error["msg"])
+                self.add_error(str(field_name), error["msg"])
 
         return cleaned_data
 
-    def cleanup(self):
+    def cleanup(self) -> None:
         """Clean up resources."""
         if self._temporary_directory is not None:  # pragma: no cover
             self._temporary_directory.cleanup()
@@ -445,18 +444,17 @@ class ArtifactForm(BootstrapMixin, YamlMixin, ModelFormBase[Artifact]):
 
     class Meta:
         model = Artifact
-        fields = ["category", "workspace", "files", "data"]
+        fields = ["category", "files", "data"]
 
 
-@ui_prototype
-class CollectionSearchForm(Form):
+class CollectionSearchForm(BootstrapMixin, Form):
     """Form for collection search fields."""
 
     category = ChoiceField(required=False)
     name = CharField(required=False)
     historical = BooleanField(required=False)
 
-    def __init__(self, *args, instance: Collection, **kwargs):
+    def __init__(self, *args: Any, instance: Collection, **kwargs: Any) -> None:
         """Initialize category choices from the database."""
         super().__init__(*args, **kwargs)
         self.instance = instance
@@ -471,3 +469,149 @@ class CollectionSearchForm(Form):
             .order_by("category")
         )
         cast(ChoiceField, self.fields["category"]).choices = choices
+
+
+class WorkflowFilterForm(Form):
+    """Form for filtering workflows."""
+
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        """Initialize form."""
+        super().__init__(*args, **kwargs)
+
+        workflow_templates_choices = [
+            (template_name, template_name)
+            for template_name in WorkflowTemplate.objects.in_current_workspace()
+            .can_display(context.user)
+            .order_by("name")
+            .values_list("name", flat=True)
+        ]
+
+        self.fields["workflow_templates"] = MultipleChoiceField(
+            choices=workflow_templates_choices,
+            required=False,
+        )
+
+        statuses: list[tuple[str, str | list[tuple[str, str]]]] = []
+        for status_value, status_label in WorkRequest.Statuses.choices:
+            if status_value == WorkRequest.Statuses.RUNNING:
+                running_options: list[tuple[str, str]] = []
+                for (
+                    runtime_value,
+                    runtime_label,
+                ) in WorkRequest.RuntimeStatuses.choices:
+                    running_options.append(
+                        (f"running__{runtime_value}", runtime_label)
+                    )
+
+                # Add "Any" for any Running status.
+                running_options.append(("running__any", "Any"))
+                statuses.append(("Running", running_options))
+            else:
+                statuses.append((status_value, status_label))
+
+        self.fields["statuses"] = MultipleChoiceField(
+            choices=statuses,
+            required=False,
+        )
+
+        runtime_statuses_choices = [
+            (status.value, status.label)
+            for status in WorkRequest.RuntimeStatuses
+        ]
+
+        self.fields["runtime_statuses"] = MultipleChoiceField(
+            choices=runtime_statuses_choices,
+            required=False,
+        )
+
+        self.fields["results"] = MultipleChoiceField(
+            choices=sorted(
+                choice for choice in WorkRequest.Results.choices if choice[0]
+            ),
+            required=False,
+        )
+
+        usernames = [
+            (username, username)
+            for username in WorkRequest.objects.in_current_workspace()
+            .filter(task_type=TaskTypes.WORKFLOW, parent__isnull=True)
+            .order_by("created_by__username")
+            .distinct("created_by__username")
+            .values_list("created_by__username", flat=True)
+        ]
+
+        self.fields["started_by"] = MultipleChoiceField(
+            choices=usernames,
+            required=False,
+        )
+
+        self.fields["with_failed_work_requests"] = BooleanField(
+            required=False,
+        )
+
+
+class DaysField(IntegerField):
+    """Edit a timedelta as a number of days."""
+
+    def prepare_value(self, value: Any) -> Any:
+        """Convert a timedelta to days."""
+        if isinstance(value, timedelta):
+            return super().prepare_value(value.days)
+        return super().prepare_value(value)
+
+    def to_python(self, value: Any) -> Any:
+        """Convert days to timedelta."""
+        res = super().to_python(value)
+        if res is not None:
+            return timedelta(days=res)
+        return res
+
+
+class WorkspaceForm(BootstrapMixin, ModelFormBase[Workspace]):
+    """Form for configuring a workspace."""
+
+    class Meta:
+        model = Workspace
+        fields = ["public", "expiration_delay", "default_expiration_delay"]
+        field_classes = {
+            "expiration_delay": DaysField,
+            "default_expiration_delay": DaysField,
+        }
+
+
+class GroupAddUserForm(BootstrapMixin, Form):
+    """Form for adding a user to a group."""
+
+    username = CharField(required=True)
+
+    def __init__(self, group: Group, *args: Any, **kwargs: Any) -> None:
+        """Set the group the user would be added to."""
+        self.group = group
+        super().__init__(*args, **kwargs)
+
+    def clean(self) -> dict[str, Any]:
+        """
+        Validate username.
+
+        The user needs to exist and not be already a member of the group.
+        """
+        cleaned_data = super().clean()
+        assert cleaned_data is not None
+
+        if "username" not in cleaned_data:
+            return cleaned_data
+
+        try:
+            user = User.objects.get(username=cleaned_data["username"])
+        except User.DoesNotExist:
+            self.add_error(
+                "username", f"User {cleaned_data['username']} does not exist"
+            )
+            return cleaned_data
+
+        if self.group.users.filter(pk=user.pk).exists():
+            self.add_error(
+                "username", f"User is already a member of {self.group}"
+            )
+
+        return cleaned_data
