@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -25,7 +25,7 @@ import pygments.lexers
 from django.http import FileResponse
 from django.http.response import HttpResponseBase
 from django.shortcuts import redirect
-from django.template.context import Context
+from django.template import Context
 from django.utils.safestring import SafeString
 from django.views.generic.base import View
 from rest_framework import status
@@ -68,7 +68,7 @@ class FileUI(NamedTuple):
         """Get a FileUI for a file."""
         # TODO: this is currently rather simple minded.
         # If it will need more complexity in the future,
-        # debusine.artfacts.models.ArtifactData can be extended with a
+        # debusine.artifacts.models.ArtifactData can be extended with a
         # get_content_type or get_file_ui method.
         # Another thing we may want to consider is adding a content_type
         # argument to File.
@@ -119,16 +119,22 @@ class LinenoHtmlFormatter(HtmlFormatter[Any]):
         self, *args: Any, **kwargs: Any
     ) -> Generator[tuple[int, str], None, None]:
         """Count rendered lines."""
-        for val in super().wrap(*args, **kwargs):
+        # mypy complains that pygments.formatters.HtmlFormatter.wrap is
+        # untyped, which is true, but we can't fix that here.
+        for val in super().wrap(
+            *args, **kwargs
+        ):  # type: ignore[no-untyped-call]
             self.line_count += val[0]
             yield val
 
-    def render_linenos(self) -> str:
+    def render_linenos(self, file_tag: str | None = None) -> str:
         """Render line numbers to go alongside rendered contents."""
         width = len(str(self.line_count))
+        suffix = f"-{file_tag}" if file_tag else ""
         return SafeString(
             "\n".join(
-                f"<a id='L{n}' href='#L{n}'>{str(n).rjust(width)}</a>"
+                f"<a id='L{n}{suffix}' href='#L{n}{suffix}'>"
+                f"{str(n).rjust(width)}</a>"
                 for n in range(1, self.line_count + 1)
             )
         )
@@ -140,25 +146,46 @@ class FileWidget(Widget):
     template_name: str
 
     def __init__(
-        self, file_in_artifact: FileInArtifact, file_ui: FileUI
+        self,
+        file_in_artifact: FileInArtifact,
+        *,
+        file_ui: FileUI,
+        file_tag: str | None = None,
     ) -> None:
-        """Initialize object with a file to display."""
+        """
+        Initialize object with a file to display.
+
+        @param file_tag: there might be pages with more
+          than one FileWidget (even referring to the same file). In order
+          to create unique anchors to the lines, it's possible to pass a
+          file_tag. The anchors will contain it, currently in the form of:
+          "#L{line_number}-{file_tag}"
+        """
         self.file_in_artifact = file_in_artifact
         self.file_ui = file_ui or FileUI.from_file_in_artifact(file_in_artifact)
+        self.file_tag = file_tag
 
     @classmethod
     def create(
-        cls, file_in_artifact: FileInArtifact, file_ui: FileUI | None = None
+        cls,
+        file_in_artifact: FileInArtifact,
+        *,
+        file_ui: FileUI | None = None,
+        file_tag: str | None = None,
     ) -> "FileWidget":
         """Create a file widget for the given file."""
         file_ui = file_ui or FileUI.from_file_in_artifact(file_in_artifact)
-        return file_ui.widget_class(file_in_artifact, file_ui)
+        return file_ui.widget_class(
+            file_in_artifact,
+            file_ui=file_ui,
+            file_tag=file_tag,
+        )
 
     @contextlib.contextmanager
     def _open_file(self) -> Generator["SupportsRead[bytes]"]:
         assert self.file_in_artifact is not None
-        workspace = self.file_in_artifact.artifact.workspace
-        file_backend = workspace.default_file_store.get_backend_object()
+        scope = self.file_in_artifact.artifact.workspace.scope
+        file_backend = scope.download_file_backend(self.file_in_artifact.file)
         with file_backend.get_stream(self.file_in_artifact.file) as stream:
             yield stream
 
@@ -187,32 +214,44 @@ class TextFileWidget(FileWidget):
     def get_context_data(self) -> dict[str, Any]:
         """Add context data for text files."""
         context = super().get_context_data()
-        # Generate CSS with:
-        # python3 -m pygments -S github-dark -f html \
-        #     > debusine/web/static/web/css/debusine-code-highlight.css
-        # See https://pygments.org/styles/ for a list of styles
-        if self.file_ui.pygments_lexer:
-            lexer = pygments.lexers.get_lexer_by_name(
-                self.file_ui.pygments_lexer
-            )
+
+        if self.file_tag is None:
+            div_id = "file-contents"
         else:
-            try:
-                lexer = pygments.lexers.get_lexer_for_mimetype(
-                    self.file_ui.content_type
+            div_id = f"file-contents-{self.file_tag}"
+
+        context["file_contents_div_id"] = div_id
+
+        if self.file_in_artifact.complete:
+            # Generate CSS with:
+            # python3 -m pygments -S github-dark -f html \
+            #     > debusine/web/static/web/css/debusine-code-highlight.css
+            # See https://pygments.org/styles/ for a list of styles
+            if self.file_ui.pygments_lexer:
+                lexer = pygments.lexers.get_lexer_by_name(
+                    self.file_ui.pygments_lexer
                 )
-            except pygments.util.ClassNotFound:
-                lexer = pygments.lexers.get_lexer_for_mimetype("text/plain")
+            else:
+                try:
+                    lexer = pygments.lexers.get_lexer_for_mimetype(
+                        self.file_ui.content_type
+                    )
+                except pygments.util.ClassNotFound:
+                    lexer = pygments.lexers.get_lexer_for_mimetype("text/plain")
 
-        formatter = LinenoHtmlFormatter(
-            cssclass="file_highlighted",
-            linenos=False,
-        )
+            formatter = LinenoHtmlFormatter(
+                cssclass="file_highlighted",
+                linenos=False,
+            )
 
-        with self._open_file() as stream:
-            formatted = pygments.highlight(stream.read(), lexer, formatter)
+            with self._open_file() as stream:
+                formatted = pygments.highlight(stream.read(), lexer, formatter)
+
+            context["file_linenumbers"] = SafeString(
+                formatter.render_linenos(self.file_tag)
+            )
+            context["file_content"] = SafeString(formatted)
 
-        context["file_linenumbers"] = SafeString(formatter.render_linenos())
-        context["file_content"] = SafeString(formatted)
         return context
 
 
@@ -244,6 +283,12 @@ class FileDownloadMixin(View):
         download: bool = True,
     ) -> HttpResponseBase:
         """Return a response that streams the_given file."""
+        if not file_in_artifact.complete:
+            return ProblemResponse(
+                "Cannot download incomplete file",
+                status_code=status.HTTP_404_NOT_FOUND,
+            )
+
         try:
             content_range = parse_range_header(self.request.headers)
         except ValueError as exc:
@@ -251,14 +296,11 @@ class FileDownloadMixin(View):
             # by end users directly
             return ProblemResponse(str(exc))
 
-        workspace = file_in_artifact.artifact.workspace
-        file_backend = workspace.default_file_store.get_backend_object()
+        scope = file_in_artifact.artifact.workspace.scope
+        file_backend = scope.download_file_backend(file_in_artifact.file)
+        entry = file_backend.get_entry(file_in_artifact.file)
 
-        # Ignore func-returns-value for now: LocalFileBackend.get_url always
-        # returns None, but the backend might be something else in future.
-        url = file_backend.get_url(
-            file_in_artifact.file
-        )  # type: ignore[func-returns-value]
+        url = entry.get_url(content_type=ui_info.content_type)
         if url is not None:
             # The client can download the file from the backend
             # TODO: this does not allow to set content-disposition, that is, to
@@ -266,7 +308,7 @@ class FileDownloadMixin(View):
             # be solved.
             return redirect(url)
 
-        with file_backend.get_stream(file_in_artifact.file) as file:
+        with entry.get_stream() as file:
             file_size = file_in_artifact.file.size
             status_code: int
             if content_range is None:
