@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -10,52 +10,68 @@
 """debusine WorkRequest view."""
 
 import functools
-from typing import Any
+from typing import Any, cast
 
 from django.contrib import messages
 from django.contrib.auth import get_user_model
-from django.core.exceptions import PermissionDenied, ValidationError
-from django.db.models import QuerySet
-from django.http import Http404, HttpRequest, HttpResponse
-from django.shortcuts import get_object_or_404, redirect, render
-from django.urls import reverse
+from django.core.exceptions import ValidationError
+from django.http import HttpRequest, HttpResponse
+from django.shortcuts import redirect, render
 from django.views.generic.base import View
 from rest_framework import status
-from rest_framework.permissions import DjangoModelPermissions
 
-from debusine.db.models import (
-    Artifact,
-    TaskDatabase,
-    User,
-    WorkRequest,
-    Workspace,
-)
+from debusine.db.context import context
+from debusine.db.models import Artifact, TaskDatabase, User, WorkRequest
 from debusine.db.models.work_requests import (
     CannotRetry,
     CannotUnblock,
     InternalTaskError,
+    WorkRequestQuerySet,
 )
-from debusine.server.views import IsUserAuthenticated, ValidatePermissionsMixin
 from debusine.server.workflows.models import WorkRequestManualUnblockAction
 from debusine.tasks import TaskConfigError
 from debusine.tasks.models import TaskTypes
 from debusine.web.forms import WorkRequestForm, WorkRequestUnblockForm
-from debusine.web.utils import ui_prototype
 from debusine.web.views import sidebar, ui_shortcuts
 from debusine.web.views.base import (
-    BaseUIView,
     CreateViewBase,
     DetailViewBase,
     FormMixinBase,
     ListViewBase,
+    SingleObjectMixinBase,
+    WorkspaceView,
 )
 from debusine.web.views.base_rightbar import RightbarUIView
 from debusine.web.views.http_errors import HttpError400, catch_http_errors
-from debusine.web.views.utils import PaginationMixin, format_yaml
+from debusine.web.views.table import TableMixin
+from debusine.web.views.tables import WorkRequestTable
+from debusine.web.views.view_utils import format_yaml
 
 
-class WorkRequestDetailView(RightbarUIView, DetailViewBase[WorkRequest]):
-    """List work requests."""
+class WorkRequestObjectMixin(WorkspaceView):
+    """
+    Add permission filtering to a Single/MultipleObjectMixin.
+
+    Since this inherits from WorkspaceView, this also sets the workspace in the
+    context and checks that it is accessible.
+    """
+
+    def get_queryset(self) -> WorkRequestQuerySet[WorkRequest]:
+        """Filter work requests by current workspace."""
+        # Both SingleObjectMixin and MultipleObjectsMixin define a get_queryset
+        # without a common base, so we cannot inherit from a base class with
+        # the right signature.
+        qs = cast(
+            WorkRequestQuerySet[WorkRequest],
+            super().get_queryset(),  # type: ignore[misc]
+        )
+        return qs.in_current_workspace().can_display(context.user)
+
+
+class WorkRequestDetailView(
+    WorkRequestObjectMixin, RightbarUIView, DetailViewBase[WorkRequest]
+):
+    """Show a work request."""
 
     model = WorkRequest
     context_object_name = "work_request"
@@ -79,7 +95,9 @@ class WorkRequestDetailView(RightbarUIView, DetailViewBase[WorkRequest]):
         request passes validation.
         """
         use_specialized = self.request.GET.get("view", "default") != "generic"
-        plugin_class = WorkRequestPlugin.plugin_for(self.object.task_name)
+        plugin_class = WorkRequestPlugin.plugin_for(
+            self.object.task_type, self.object.task_name
+        )
 
         return (
             use_specialized
@@ -90,11 +108,10 @@ class WorkRequestDetailView(RightbarUIView, DetailViewBase[WorkRequest]):
     def get_main_ui_shortcuts(self) -> list[ui_shortcuts.UIShortcut]:
         """Return a list of UI shortcuts for this view."""
         actions = super().get_main_ui_shortcuts()
-        if self.request.user.is_authenticated and self.object.can_retry():
+        if self.request.user.is_authenticated and self.object.verify_retry():
             actions.append(ui_shortcuts.create_work_request_retry(self.object))
         return actions
 
-    @ui_prototype
     def get_sidebar_items(self) -> list[sidebar.SidebarItem]:
         """Return a list of sidebar items."""
         items = super().get_sidebar_items()
@@ -123,7 +140,7 @@ class WorkRequestDetailView(RightbarUIView, DetailViewBase[WorkRequest]):
     def get_manual_unblock_log(self) -> list[dict[str, Any]]:
         """Get a representation of the manual unblock log."""
         workflow_data = self.object.workflow_data
-        if workflow_data is None or workflow_data.manual_unblock is None:
+        if workflow_data.manual_unblock is None:
             return []
 
         users = {
@@ -148,18 +165,14 @@ class WorkRequestDetailView(RightbarUIView, DetailViewBase[WorkRequest]):
 
         Add the artifacts related to the work request.
         """
-        if not (
-            self.object.workspace.public or self.request.user.is_authenticated
-        ):
-            raise PermissionDenied(
-                "You need to be authenticated to view this Work Request"
-            )
         context = super().get_context_data(**kwargs)
 
-        plugin_view = WorkRequestPlugin.plugin_for(self.object.task_name)
+        plugin_view = WorkRequestPlugin.plugin_for(
+            self.object.task_type, self.object.task_name
+        )
 
         if plugin_view is not None and self._current_view_is_specialized():
-            return {**context, **plugin_view().get_context_data(self.object)}
+            return {**context, **plugin_view(self.object).get_context_data()}
 
         context["manual_unblock_log_entries"] = self.get_manual_unblock_log()
         context["manual_unblock_form"] = WorkRequestUnblockForm()
@@ -181,7 +194,9 @@ class WorkRequestDetailView(RightbarUIView, DetailViewBase[WorkRequest]):
                 source_artifact_ids := task.get_source_artifacts_ids()
             ):
                 source_artifacts.extend(
-                    Artifact.objects.filter(pk__in=source_artifact_ids)
+                    Artifact.objects.filter(
+                        pk__in=source_artifact_ids
+                    ).annotate_complete()
                 )
                 context["source_artifacts"] = source_artifacts
         except NotImplementedError:
@@ -190,9 +205,9 @@ class WorkRequestDetailView(RightbarUIView, DetailViewBase[WorkRequest]):
             context["source_artifacts_not_implemented"] = True
 
         built_artifacts = list(
-            Artifact.objects.filter(
-                created_by_work_request=self.object
-            ).order_by("category", "id")
+            Artifact.objects.filter(created_by_work_request=self.object)
+            .annotate_complete()
+            .order_by("category", "id")
         )
 
         # Generate UI shortcuts for artifacts
@@ -204,17 +219,25 @@ class WorkRequestDetailView(RightbarUIView, DetailViewBase[WorkRequest]):
             )
 
         context["built_artifacts"] = built_artifacts
-        context["task_data"] = format_yaml(self.object.task_data)
+        context["task_data"] = format_yaml(self.object.used_task_data)
+        if self.object.task_data != self.object.used_task_data:
+            context["task_data_original"] = format_yaml(self.object.task_data)
+            context["task_data_configured"] = True
+        else:
+            context["task_data_configured"] = False
+
         return context
 
-    def get_template_names(self):
+    def get_template_names(self) -> list[str]:
         """Return the plugin's template_name or the default one."""
         if self._current_view_is_specialized():
-            return WorkRequestPlugin.plugin_for(
-                self.object.task_name
-            ).template_name
+            plugin_class = WorkRequestPlugin.plugin_for(
+                self.object.task_type, self.object.task_name
+            )
+            assert plugin_class is not None
+            return [plugin_class.template_name]
 
-        return self.default_template_name
+        return [self.default_template_name]
 
 
 class WorkRequestPlugin:
@@ -223,46 +246,53 @@ class WorkRequestPlugin:
 
     When subclassing, the subclass:
     - Is automatically used by the /work-request/ID/ endpoints
-    - Must define the "template_name" and "task_name"
-    - Must implement "get_context_data(work_request)"
+    - Must define "template_name", "task_type", and "task_name"
+    - Must implement "get_context_data()"
     """
 
-    _work_request_plugins: dict[str, type["WorkRequestPlugin"]] = {}
+    model = WorkRequest
+    template_name: str
+    task_type: TaskTypes
+    task_name: str
 
-    def __init_subclass__(cls, **kwargs):  # noqa: U100
+    _work_request_plugins: dict[tuple[str, str], type["WorkRequestPlugin"]] = {}
+
+    def __init_subclass__(cls, **kwargs: Any) -> None:  # noqa: U100
         """Register the plugin."""
-        cls._work_request_plugins[cls.task_name] = cls
+        cls._work_request_plugins[(cls.task_type, cls.task_name)] = cls
+
+    def __init__(self, work_request: WorkRequest) -> None:
+        """Store the work_request to display."""
+        self.work_request = work_request
 
     @classmethod
-    def plugin_for(cls, task_name: str) -> type["WorkRequestPlugin"] | None:
+    def plugin_for(
+        cls, task_type: str, task_name: str
+    ) -> type["WorkRequestPlugin"] | None:
         """Return WorkRequestPlugin for task_name or None."""
-        return cls._work_request_plugins.get(task_name)
+        return cls._work_request_plugins.get((task_type, task_name))
 
-    def get_context_data(self, work_request: WorkRequest) -> dict[str, Any]:
+    def get_context_data(self) -> dict[str, Any]:
         """Must be implemented by subclasses."""
         raise NotImplementedError()
 
 
 class WorkRequestListView(
-    BaseUIView, PaginationMixin, ListViewBase[WorkRequest]
+    TableMixin[WorkRequest], WorkRequestObjectMixin, ListViewBase[WorkRequest]
 ):
     """List work requests."""
 
     model = WorkRequest
+    table_class = WorkRequestTable
     template_name = "web/work_request-list.html"
     context_object_name = "work_request_list"
+    title = "List of work requests"
     paginate_by = 50
 
-    def get_workspace(self) -> Workspace | None:
-        """Return the current workspace, if any."""
-        if workspace_name := self.request.GET.get("workspace"):
-            return get_object_or_404(Workspace, name=workspace_name)
-        return None
-
     @staticmethod
     def _filter_by_architecture(
-        work_requests: QuerySet[WorkRequest], architecture: str
-    ) -> QuerySet[WorkRequest]:
+        work_requests: WorkRequestQuerySet[WorkRequest], architecture: str
+    ) -> WorkRequestQuerySet[WorkRequest]:
         pending_without_host_architecture = work_requests.filter(
             task_data__host_architecture__isnull=True
         )
@@ -301,18 +331,10 @@ class WorkRequestListView(
 
         return queryset
 
-    def get_queryset(self):
+    def get_queryset(self) -> WorkRequestQuerySet[WorkRequest]:
         """Filter work requests displayed by the workspace GET parameter."""
         queryset = super().get_queryset().exclude(task_type=TaskTypes.INTERNAL)
 
-        if self.workspace is not None:
-            queryset = queryset.filter(workspace=self.workspace)
-
-        if not self.request.user.is_authenticated:
-            # Non-authenticated users can only list WorkRequests in
-            # a public workspace
-            queryset = queryset.filter(workspace__public=True)
-
         arch_param = self.request.GET.get("arch")
         status_param = self.request.GET.get("status")
 
@@ -324,7 +346,9 @@ class WorkRequestListView(
             "blocked": WorkRequest.Statuses.BLOCKED,
         }
 
-        status_value = status_mapping.get(status_param)
+        status_value = (
+            None if status_param is None else status_mapping.get(status_param)
+        )
 
         if status_param is not None and status_value is None:
             messages.warning(
@@ -351,52 +375,36 @@ class WorkRequestListView(
 
         return queryset
 
-    def get_ordering(self):
-        """Return field used for sorting."""
-        order = self.request.GET.get("order")
-        if order in ("id", "created_at", "task_name", "status", "result"):
-            if self.request.GET["asc"] == "0":
-                return "-" + order
-            else:
-                return order
-
-        return "-created_at"
-
-    def get_context_data(self, **kwargs):
-        """Add context to the default ListView data."""
-        context = super().get_context_data(**kwargs)
-
-        context["order"] = self.get_ordering().removeprefix("-")
-        context["asc"] = self.request.GET.get("asc", "0")
-
-        return context
-
 
 class WorkRequestCreateView(
-    ValidatePermissionsMixin, CreateViewBase[WorkRequest, WorkRequestForm]
+    WorkRequestObjectMixin,
+    CreateViewBase[WorkRequest, WorkRequestForm],
 ):
     """Form view for creating a work request."""
 
     model = WorkRequest
     template_name = "web/work_request-create.html"
     form_class = WorkRequestForm
+    title = "Create work request"
 
-    permission_denied_message = (
-        "You need to be authenticated to create a Work Request"
-    )
-    permission_classes = [IsUserAuthenticated]
+    def init_view(self) -> None:
+        """Set the current workspace."""
+        super().init_view()
+        self.enforce(context.require_workspace().can_create_work_requests)
 
     def get_form_kwargs(self) -> dict[str, Any]:
         """Extend the default kwarg arguments: add "user"."""
         kwargs = super().get_form_kwargs()
-        kwargs["user"] = self.request.user
+        kwargs["user"] = context.user
+        kwargs["workspace"] = context.workspace
         return kwargs
 
-    def get_success_url(self):
+    def get_success_url(self) -> str:
         """Redirect to work_requests:detail for the created WorkRequest."""
-        return reverse("work_requests:detail", kwargs={"pk": self.object.id})
+        assert self.object is not None
+        return self.object.get_absolute_url()
 
-    def form_valid(self, form):
+    def form_valid(self, form: WorkRequestForm) -> HttpResponse:
         """Validate the work request."""
         self.object = form.save(commit=False)
 
@@ -411,76 +419,62 @@ class WorkRequestCreateView(
         return super().form_valid(form)
 
 
-class WorkRequestRetryView(ValidatePermissionsMixin, View):
+class WorkRequestRetryView(
+    WorkRequestObjectMixin, SingleObjectMixinBase[WorkRequest], View
+):
     """Form view for retrying a work request."""
 
-    permission_denied_message = (
-        "You need to be authenticated to retry a Work Request"
-    )
-    permission_classes = [IsUserAuthenticated]
+    model = WorkRequest
 
-    @functools.cached_property
-    def work_request(self) -> WorkRequest:
-        """Return the work request for this request."""
+    def retry(self, work_request: WorkRequest) -> HttpResponse:
+        """Retry the work request."""
         try:
-            return WorkRequest.objects.get(pk=self.kwargs["pk"])
-        except WorkRequest.DoesNotExist:
-            raise Http404(f"Work request {self.kwargs['pk']} not found")
-
-    def post(
-        self, request: HttpRequest, *args, **kwargs  # noqa: U100
-    ) -> HttpResponse:
-        """Handle POST requests."""
-        try:
-            new_work_request = self.work_request.retry()
+            new_work_request = work_request.retry()
         except CannotRetry as e:
             messages.error(self.request, f"Cannot retry: {e}")
-            return redirect("work_requests:detail", pk=self.work_request.pk)
+            return redirect(work_request.get_absolute_url())
 
-        return redirect("work_requests:detail", pk=new_work_request.pk)
+        return redirect(new_work_request.get_absolute_url())
 
-
-class WorkRequestUnblockViewPermissions(DjangoModelPermissions):
-    """Custom permissions for :py:class:`WorkRequestUnblockView`."""
-
-    perms_map = {
-        **DjangoModelPermissions.perms_map,
-        # By default DjangoModelPermissions assumes that POST is used to
-        # create new objects, but in this case we're using it to change an
-        # existing object.
-        "POST": ["%(app_label)s.change_%(model_name)s"],
-    }
+    def post(
+        self, request: HttpRequest, *args: Any, **kwargs: Any  # noqa: U100
+    ) -> HttpResponse:
+        """Handle POST requests."""
+        work_request = self.get_object()
+        self.enforce(work_request.can_retry)
+        return self.retry(work_request)
 
 
 class WorkRequestUnblockView(
-    ValidatePermissionsMixin, FormMixinBase[WorkRequestUnblockForm], View
+    WorkRequestObjectMixin,
+    SingleObjectMixinBase[WorkRequest],
+    FormMixinBase[WorkRequestUnblockForm],
+    View,
 ):
     """Form view for reviewing a work request awaiting manual approval."""
 
-    permission_denied_message = (
-        "You need to be authenticated to review a blocked Work Request"
-    )
-    # TODO: We should replace this with some finer-grained ACL once we have
-    # that sort of thing.  It would be useful to grant the ability to edit
-    # manual unblocks without being able to freely edit work requests, or
-    # the ability to edit manual unblocks only in certain contexts.
-    permission_classes = [WorkRequestUnblockViewPermissions]
+    model = WorkRequest
     form_class = WorkRequestUnblockForm
-    queryset = WorkRequest.objects
 
-    @functools.cached_property
-    def work_request(self) -> WorkRequest:
-        """Return the work request for this request."""
-        try:
-            return WorkRequest.objects.get(pk=self.kwargs["pk"])
-        except WorkRequest.DoesNotExist:
-            raise Http404(f"Work request {self.kwargs['pk']} not found")
+    def get_action(self) -> WorkRequestManualUnblockAction | None:
+        """Get the unblock action from query arguments."""
+        raw_action = self.request.POST["action"]
+        match raw_action:
+            case "Accept":
+                return WorkRequestManualUnblockAction.ACCEPT
+            case "Reject":
+                return WorkRequestManualUnblockAction.REJECT
+            case "Record notes only":
+                return None
+            case _:
+                raise HttpError400(f"Invalid action parameter: {raw_action!r}")
+
+    def unblock(self, work_request: WorkRequest) -> HttpResponse:
+        """Perform the unblock."""
+        # This view requires a permission, so the user can't be an
+        # AnonymousUser by this point.
+        assert isinstance(self.request.user, User)
 
-    @catch_http_errors
-    def post(
-        self, request: HttpRequest, *args, **kwargs  # noqa: U100
-    ) -> HttpResponse:
-        """Handle POST requests."""
         form = self.get_form()
         if not form.is_valid():
             # TODO: This is ugly and we should probably try to render
@@ -489,34 +483,27 @@ class WorkRequestUnblockView(
             # actually hit this case in practice, so it doesn't seem worth
             # the effort for now.
             return render(
-                request,
+                self.request,
                 "400.html",
                 context={"error": form.errors.as_json()},
                 status=status.HTTP_400_BAD_REQUEST,
             )
 
-        # This view requires a permission, so the user can't be an
-        # AnonymousUser by this point.
-        assert isinstance(request.user, User)
-
-        raw_action = request.POST["action"]
-        action: WorkRequestManualUnblockAction | None
-        match raw_action:
-            case "Accept":
-                action = WorkRequestManualUnblockAction.ACCEPT
-            case "Reject":
-                action = WorkRequestManualUnblockAction.REJECT
-            case "Record notes only":
-                action = None
-            case _:
-                raise HttpError400(f"Invalid action parameter: {raw_action!r}")
-
         try:
-            self.work_request.review_manual_unblock(
-                user=request.user,
+            work_request.review_manual_unblock(
+                user=self.request.user,
                 notes=form.cleaned_data["notes"],
-                action=action,
+                action=self.get_action(),
             )
         except CannotUnblock as e:
             raise HttpError400(f"Cannot unblock: {e}")
-        return redirect("work_requests:detail", pk=self.work_request.pk)
+        return redirect(work_request.get_absolute_url())
+
+    @catch_http_errors
+    def post(
+        self, request: HttpRequest, *args: Any, **kwargs: Any  # noqa: U100
+    ) -> HttpResponse:
+        """Handle POST requests."""
+        work_request = self.get_object()
+        self.enforce(work_request.can_unblock)
+        return self.unblock(work_request)
